#### Cargamos la librerias que se necesitan
library(ggplot2)
library(plotly)
library(car)
library(GGally)
library(ltm)
library(naniar)
library(lubridate)
library(dplyr)

#### Lectura de los datos (modificar para R)
datos<- read.table("UH_2023_TRAIN.txt", sep = "|", header = TRUE,encoding = "UTF-8")

length(unique(datos$ID_FINCA)) # 1231 fincas distintas
length(unique(datos$ID_ZONA))  # 125 zonas
length(unique(datos$ID_ESTACION))  #20 estaciones

table(datos$COLOR,datos$TIPO) #Color es uva blanca o negra, Tipo es tipo de cultivo


#### Transformación de datos 
datos20.Sno0<-datos[datos$CAMPAÑA==20 & datos$SUPERFICIE>0,] #datos del 2020 con SUPERFICIE>0

#Transformación de los datos en datos20.Sno0 ----------
datos_sep <- datos20.Sno0 %>%
  mutate(aux = ifelse(is.na(ALTITUD), "999", ALTITUD)) %>%  #Variable auxiliar, no interesa
  mutate(contiene_guion = grepl("-", aux, fixed = TRUE)) %>% #TRUE/FALSE si ALTITUD está formado por 
  #dos números o uno solo
  mutate(ALT.max =ifelse(contiene_guion,as.numeric(substr(aux, start = 5, stop = nchar(aux))),NA)) %>% 
  #Si ALTITUD tiene dos valores ("XXX-XXX") ALT.max guarda el máximo y 
  #ALT.min el mínimo. En caso contrario, se guarda NA.
  mutate(ALT.min =ifelse(contiene_guion,as.numeric(substr(aux, start = 1, stop = 3)),NA)) %>%     
  mutate(ALT_med = ifelse(contiene_guion,
                          (as.numeric(substr(aux, start = 5, stop = nchar(aux)))+
                             as.numeric(substr(aux, start = 1, stop = 3)))/2
                          ,as.numeric(aux)))%>% #ALT_med la altitud en caso de ser única o la media si 
  #hay 2. Guarda los faltantes como 999, en ALT_med2 se guardan como NA
  mutate(ALT_med2 = ifelse(is.na(ALTITUD), NA, ALT_med)) %>%
  mutate(EFICIENCIA =PRODUCCION/SUPERFICIE) %>% 
  mutate(indicador= ifelse(ALT_med2>550,1,0 )) %>% #indicador es una variable que toma el valor 1 si 
  #altitud media de la finca supera 550 y 0 en caso contrario.
  mutate(VARIEDAD.f=as.factor(VARIEDAD))  %>% #VARIEDAD es numérico y VARIEDAD.f factor
  mutate(ID_ESTACION.f = as.factor(ID_ESTACION)) %>%
  mutate(TIPO.f = as.character(TIPO))
#Mantengo los dos (VARIEDAD y VARIEDAD.f) porque a veces interesa tener ordenados de manera ascendente 
#las variedades

#datos.At tiene las mismas transformaciones que datos_sep pero sobre todos los años
#el nombre viene de todos los Años, Transformados.
datos.At <- datos %>%
  mutate(aux = ifelse(is.na(ALTITUD), "999", ALTITUD)) %>%
  mutate(contiene_guion = grepl("-", aux, fixed = TRUE)) %>%
  mutate(ALT.max =ifelse(contiene_guion,as.numeric(substr(aux, start = 5, stop = nchar(aux))),NA)) %>%
  mutate(ALT.min =ifelse(contiene_guion,as.numeric(substr(aux, start = 1, stop = 3)),NA)) %>%     
  mutate(ALT_med = ifelse(contiene_guion,
                          (as.numeric(substr(aux, start = 5, stop = nchar(aux)))+
                             as.numeric(substr(aux, start = 1, stop = 3)))/2
                          ,as.numeric(aux)))%>% 
  mutate(ALT_med2 = ifelse(is.na(ALTITUD), NA, ALT_med)) %>%
  mutate(EFICIENCIA =PRODUCCION/SUPERFICIE) %>%
  mutate(indicador= ifelse(ALT_med2>550,1,0 ))%>%
  mutate(VARIEDAD.f=as.factor(VARIEDAD))  %>% #VARIEDAD es numérico y VARIEDAD.f factor
  mutate(ID_ESTACION.f = as.factor(ID_ESTACION)) %>%
  mutate(TIPO.f = as.character(TIPO))%>%
  mutate(ID_UNICO=paste(ID_FINCA,VARIEDAD,TIPO,MODO,sep="-"))


# En esta sección se encuentran los gráficos que permiten ver cómo evolucionan distintos indicadores a lo largo de las distintas campañas.

#Gráfico: En qué años está cada finca?
IDF.N<- unique(datos.At %>% dplyr::select("ID_UNICO")) %>% dplyr::mutate(nfID_UNICO=row_number())
dim(IDF.N) #Hay 1959 ID_UNICOs distintos

datos.CF<- full_join(datos.At,IDF.N,by="ID_UNICO") %>% dplyr::select(nfID_UNICO,CAMPAÑA,ID_FINCA)
#datos.CF 

aux2<-seq(from=1,to=1959,by=205)

par(mfrow=c(3,3))
for (i in 2:9){
  x.min=aux2[i-1]
  x.max=aux2[i]
  print(i)
  plot(datos.CF$CAMPAÑA[datos.CF$nfID_UNICO%in%x.min:x.max]~
         datos.CF$nfID_UNICO[datos.CF$nfID_UNICO%in%x.min:x.max],ylab="CAMPAÑA",xlab="nfID_UNICO")
  segments(x0 = x.min:x.max,x1=x.min:x.max ,col = "red", 
           lty = 2, y0 = sapply(x.min:x.max,function(x){min(datos.CF$CAMPAÑA[datos.CF$nfID_UNICO==x])}), 
           sapply(x.min:x.max,function(x){max(datos.CF$CAMPAÑA[datos.CF$nfID_UNICO==x])}))
}

par(mfrow=c(1,1))

# De las gráficas anteriores se pueden sacar tres conclusiones:
#    1. Hay fincas que dejan la cooperativa y hay fincas que vuelven a la cooperativa después de haberse ido, aunque no es lo común.
#    2. Las ID_FINCAS van apareciendo en datos según estas se incorporaban a la cooperativa
#    3. Puede ser de interés crear una variable nueva que registre cuántos años lleva cada finca en la cooperativa


#Evolución del número de fincas que participan en cada campaña
numfinc<-function(x){length(unique(datos$ID_FINCA[datos$CAMPAÑA==x]))}
x<-14:22
plot(x,sapply(x,FUN="numfinc"),type="b")


#Evolución de la producción total de la cooperativa
sumprod.A<-function(x){sum(datos$PRODUCCION[datos$CAMPAÑA==x])}
x<-14:22
plot(x,sapply(x,FUN="sumprod.A"),type="b")

# Es destacable la disminución en el 2020, muy posiblemente debida al impacto de la pandemia.


# En esta sección se trabaja haciendo el EDA sólo sobre los datos de 2020 
#Además también se han descartado las filas cuyos valores de SUPERFICIE es NA. 

# Empezamos representando un histograma de la variable SUPERFICIE en datos_sep
hist(datos_sep$SUPERFICIE)

#Gráfico de producción por alturas
ggplot(datos_sep, aes(SUPERFICIE, PRODUCCION, color = ALT_med2)) +
  geom_point() +
  scale_color_gradient(low = "red",high = "green",na.value="black")

#Gráfico de producción por alturas ZOOM
ggplot(datos_sep[datos_sep$SUPERFICIE<5 & datos_sep$PRODUCCION< 20000,], aes(SUPERFICIE, PRODUCCION, color = ALT_med2)) +
  geom_point() +
  scale_color_gradient(low = "red",high = "green",na.value="black")

#Gráfico EFICIENCIA
ggplot(datos_sep,aes(x=ALT_med,y=EFICIENCIA,color=MODO)) +  #Los valores NA en ALT_med se fijaron a 999m, en ALT_med2 son NA
  geom_point()
#En la última gráfica se ve un atípico muy claro, se trata de la finca 3579.

# Se repetirá el último gráfico pero sin el valor atípico.

#Gráfico EFICIENCIA
ggplot(datos_sep[datos_sep$ID_FINCA!=3579,],aes(x=ALT_med,y=EFICIENCIA,color=MODO)) +  #Los valores NA en ALT_med se fijaron a 999m, en ALT_med2 son NA
  geom_point()
#En la última gráfica se ve un atípico muy claro, se trata de la finca 3579.


ggplot(datos_sep, aes(SUPERFICIE, PRODUCCION, color = indicador)) + # indicador= ifelse(ALT_med2>550,1,0 )
  geom_point()+
  geom_text(aes(label=ifelse(SUPERFICIE>17,as.character(ID_FINCA),'')),hjust=0,vjust=0)+
  geom_text(aes(label=ifelse(ID_FINCA==3579,as.character(ID_FINCA),'')),hjust=0,vjust=0)


# Parece que las fincas de menos altura tienen mejores producciones. 


#ESTUDIO DEL RESTO DE VARIABLES

# 1. VARIEDAD

#   1.1 VARIEDAD-COLOR

# En primer lugar, hay que destacar que a cada variedad le corresponde un único color:
t(table(datos$VARIEDAD,datos$COLOR)) #transpuesta para que ocupe menos

# A continuación se crea el df VAR.COL que contiene la correspondencia entre cada variedad y su color

VAR.COL<-arrange(distinct(
  datos.At %>% 
    mutate(VARIEDAD.f=as.factor(VARIEDAD))
  %>% dplyr::select(CAMPAÑA,ID_FINCA,VARIEDAD,COLOR) 
  %>% dplyr::select(VARIEDAD,COLOR))
  ,VARIEDAD) 
t(VAR.COL) #Transpuesta para que ocupe menos


#    1.2 Boxplots y ANOVAs 
# 
# A continuación se procede a representar gráficos de barras para las categorías PRODUCCIÓN, ALT_med2 (la que tiene NAs), 
# SUPERFICIE Y EFICIENCIA. Además se incluyen anovas para dichas variables, resultando significativo solo el de SUPERFICIE~VARIEDAD. 
# Al aplicar el test de Tuckey se obtiene que la variedad 4 está plantada en fincas significativamente más grandes 
# que muchas de las otras variedades (9,15,17,23,26...).

# La única información que parece relevante es la presencia de un atípico en la EFICIENCIA (producción/superficie) de la finca 3579.

# Los boxplots generados son los siguientes:

par(mfrow=c(2,2))
#PRODUCCION ~VARIEDAD
boxplot(datos_sep$PRO~datos_sep$VARIEDAD,col=ifelse(VAR.COL$COLOR==0,"#69b3a2", "#404080"))
#El vector que hay que darle a col es el del número de clases, por eso se usa VAR.COL.
#Otra cosa que hay que tener en cuenta es que se está usando VARIEDAD y por eso salen las clases
#ordenadas. En otro caso el orden no coincidiría con el de VAR.COL

#ALTITUD ~VARIEDAD
boxplot(datos_sep$ALT_med2~datos_sep$VARIEDAD,col=ifelse(VAR.COL$COLOR==0,"#69b3a2", "#404080"))

#SUPERFICIE ~VARIEDAD 
boxplot(datos_sep$SUPERFICIE~datos_sep$VARIEDAD.f,col=ifelse(VAR.COL$COLOR==0,"#69b3a2", "#404080"))

#EFICIENCIA ~VARIEDAD
boxplot(datos_sep$EFICIENCIA~datos_sep$VARIEDAD.f,col=ifelse(VAR.COL$COLOR==0,"#69b3a2", "#404080"))
par(mfrow=c(1,1))
# También se han realizado ANOVAs que no han resultado significativos salvo en el caso de la superficie, en la que se detecta que la variedad 4 
#es plantada en fincas significativamente más grandes que otras variedades (9,15,17,23,26,32,40...).

anv1<-aov(datos_sep$PRO~datos_sep$VARIEDAD)
summary(anv1)
anv2<-aov(datos_sep$ALT_med2~datos_sep$VARIEDAD)
summary(anv2)
anv3<-aov(SUPERFICIE~VARIEDAD.f,data=datos_sep)
summary(anv3) #Se rechaza la hipótesis de igualdad de medias
anv4<-aov(datos_sep$PRO~datos_sep$COLOR)
summary(anv4)#Tampoco es significativo

TukeyHSD(anv3,"VARIEDAD.f")$VARIEDAD.f[TukeyHSD(anv3,"VARIEDAD.f")$VARIEDAD.f[,4]<0.05,]
#La variedad 4 es más popular que el resto* (no todas, pero bastantes)




#   1.3 Barplots de VARIEDAD

# Se han obtenido la cantidad de veces que aparece cada variedad entre las filas de datos.At y se ha representado en un barplot.
# También se representa el barplot de cada variedad con la suma de las superficies que ocupan.

#Barplot de la cantidad de fincas con cada variedad
datos_sep.Vcount<-datos.At %>% group_by(VARIEDAD.f) %>% summarise(count = n()) #Se agrupa por variedad y se suma el nº de elementos con count
datos_sep.Vcount<-datos_sep.Vcount %>% dplyr::mutate(VARIEDAD=as.numeric(as.character(VARIEDAD.f))) #Es necesario tener VARIEDAD como nº para el 
#full_join con VAR.COL
datos_sep.Vcount<-full_join(datos_sep.Vcount,VAR.COL,by="VARIEDAD") %>% dplyr::select(VARIEDAD.f,count,COLOR) 
#datos_sep.Vcount
par(mfrow=c(1,2))
barplot(datos_sep.Vcount$count~datos_sep.Vcount$VARIEDAD.f,col=ifelse(datos_sep.Vcount$COLOR==0,"#69b3a2", "#404080"))


#Barplot de la suma de superficie por cada variedad
datos_sep.VSs<-datos.At %>% group_by(VARIEDAD.f) %>% summarise(count = n(),SUP=sum(SUPERFICIE)) 
datos_sep.VSs<-datos_sep.VSs %>% mutate(VARIEDAD=as.numeric(as.character(VARIEDAD.f))) #Es necesario tener VARIEDAD como nº para el 
#full_join con VAR.COL
datos_sep.VSs<-full_join(datos_sep.VSs,VAR.COL,by="VARIEDAD") %>% dplyr::select(VARIEDAD.f,SUP,COLOR) 
#datos_sep.VSs
barplot(datos_sep.VSs$SUP~datos_sep.VSs$VARIEDAD.f,col=ifelse(datos_sep.VSs$COLOR==0,"#69b3a2", "#404080"))
par(mfrow=c(1,1))


# Por último, se incluyen heatmaps de VARIEDAD con respecto a otras variables

heatmap(table(datos.At$COLOR,datos.At$ID_ZONA))
heatmap(table(datos.At$VARIEDAD,datos.At$ID_ESTACION))
heatmap(table(datos.At$VARIEDAD,datos.At$MODO))
heatmap(table(datos.At$VARIEDAD,datos.At$TIPO))



# 2. ALT_Med2 (ALTITUD)


#   2.1 Información previa

print(paste('Registros faltantes de la altitud:',sum(is.na(datos$ALTITUD)))) # registros faltantes de la altitud
print(paste('Número de fincas con altitud faltante:',length(unique((subset(datos,is.na(ALTITUD))$ID_FINCA))))) # fincas con altitud faltante
unique(subset(datos,CAMPAÑA==22 & is.na(ALTITUD))$ID_FINCA) # fincas con altitud faltante en la CAMPAÑA 22

# Por tanto, para empezar vemos que tenemos 54 registros faltantes de la altitud, los cuales se corresponden a 15 fincas diferentes 
#(fincas de las cuales tenemos múltiples registros por distintos modos de cultivo, variedad...). 
#En la campaña que queremos predecir tenemos 3 altitudes faltantes. Tenemos que ver como queremos manejar este tipo de datos.


# A continuación, modificamos los registros de las alturas que vienen dadas por rangos ('XXX-XXX') por la altitud media 
#(está hecho antes pero prefería pegar mi código y utilizar unos datos que llamo 'datos_alv' para evitar tocar cosas que puedan afectar a 
#todo lo que ya está hecho, lo unificaré más adelante si hace falta).

altitud_cor <- ifelse(grepl('-',datos$ALTITUD,fixed=TRUE),
                      as.character((as.numeric(substr(datos$ALTITUD,1,3))+as.numeric(substr(datos$ALTITUD,5,7)))/2),
                      datos$ALTITUD)
altitud_cor <- as.numeric(altitud_cor) # Altitud corregida por el punto medio

datos_alv <- cbind(datos,altitud_cor) # Creo este dataset aparte para evitar problemas con el resto del documento


#   2.2 Diagrama de dispersión
# Ahora representamos el diagrama de dispersión de la PRODUCCION frente a la altitud corregida altitud_cor, primero en global y después por campañas.
#Hay que tener en cuenta que en esta representación se están omitiendo los registros de ALTITUD NA (que recordamos eran 54).
#Al ser pocos los datos faltantes, en principio no debería ser un problema omitirlos a la hora de interpretar visualmente los gráficos. 
#En color rojo y negro se diferencian los modos de cultivo, aunque quizás ahora mismo esta diferenciación no aporta ninguna información.


plot(na.omit(datos_alv)$altitud_cor,na.omit(datos_alv)$PRODUCCION,col=datos_alv$MODO,pch=19
     ,xlab='',ylab='',cex=0.75,main='Producción frente a altitud')

par(mfrow=c(2,4))
for (i in 14:21){
  plot(na.omit(subset(datos_alv,CAMPAÑA==i))$altitud_cor,
       na.omit(subset(datos_alv,CAMPAÑA==i))$PRODUCCION,main=paste('Campaña ',i),
       xlab='',ylab='',cex=0.75,pch=19,col=datos_alv$MODO)}

max(na.omit(datos.At$ALT_med2))
par(mfrow=c(1,1))


# Lo más destacable de estos gráficos es que se intuye que quizás en el rango de los 600-700 metros hay ligeramente mayor producción 
#que en rangos inferiores y superiores. Puede ser que esta impresión también sea debida a que igual en el rango de 600-700 metros el número de fincas
#(cultivos) es mayor, pero una discretización en la ALTITUD sería una idea a valorar.

#A continuación se muestran los diagramas de dispersión indicando con colores otras variables.

# Color = SUPERFICIE
ggplot(datos_sep, aes(ALT_med2, PRODUCCION, color = SUPERFICIE)) +
  geom_point() +
  scale_color_gradient(low = "blue",high = "red",na.value="black")

# Color = MODO
ggplot(datos_sep, aes(ALT_med2, PRODUCCION, color = MODO)) +
  geom_point() +
  scale_color_gradient(low = "blue",high = "red",na.value="black")

# Color = TIPO
ggplot(datos_sep, aes(ALT_med2, PRODUCCION, color = TIPO)) +
  geom_point() +
  scale_color_gradient(low = "blue",high = "red",na.value="black")

# Color = COLOR
ggplot(datos_sep, aes(ALT_med2, PRODUCCION, color = COLOR)) +
  geom_point() +
  scale_color_gradient(low = "blue",high = "red",na.value="black")

#  Histogramas y densidades
# Ahora se muestran tanto histogramas como estimaciones tipo núcleo de la densidad de altitud_cor, globalmente y separando por campañas.

hist(na.omit(datos_alv$altitud_cor),freq=FALSE,breaks='FD',main='Histograma de Altitud',xlab='Altitud') # Histograma

# Histogramas por campaña
par(mfrow=c(3,3))
hist(na.omit(subset(datos_alv,CAMPAÑA==14)$altitud_cor),freq=FALSE,
     main=paste('Campaña ', 14),breaks='FD') # cambiar breaks si se quiere
for (i in 15:22){
  hist(na.omit(subset(datos_alv,CAMPAÑA==i)$altitud_cor),freq=FALSE,
       main=paste('Campaña ',i),breaks='FD')
}
par(mfrow=c(1,1))

# Densidad
plot(density(na.omit(datos_alv$altitud_cor)),main='',lwd=2)
lines(density(na.omit(datos_alv$altitud_cor),bw=15),col='orange',lwd=2)

# Densidades por campaña
par(mfrow=c(3,3))
for (i in 14:22){
  plot(density(na.omit(subset(datos_alv,CAMPAÑA==i)$altitud_cor)),
       main=paste('Campaña ',i))
}
par(mfrow=c(1,1))

# A la vista de estos gráficos, claramente se intuyen dos modas en la altitud, lo cual está relacionado con la diferenciación en grupos que hacíamos antes.




# 3. MODO 

#   3.1 Información previa

levels(as.factor(datos.At$MODO)) # Solamente dos modos distintos
table(as.factor(datos.At$MODO))
table(as.factor(datos.At$MODO))/nrow(datos.At) # Prácticamente el 50% de cada modo

# En base a esta información, vemos que en el total de todas las campañas ambos modos de cultivo tienen una utilización similar. A continuación, veamos campaña por campaña:


# Por campañas:
for (i in 14:22){
  print(paste('Campaña', i))
  print(table(as.factor(subset(datos.At,CAMPAÑA==i)$MODO))/nrow(subset(datos.At,CAMPAÑA==i)))
}

# Con esto vemos que con el paso de los años se está dando un claro incremento del modo de cultivo 2, bien porque se introducen nuevas parcelas en las que se emplea este modo o bien porque en otras se decide cambiar del modo 1 al 2. Esta tendencia nos puede hacer pensar que el modo 2 da un mejor rendimiento en términos de producción (y por eso la cooperativa lo emplea cada vez más).


#   3.2 Boxplots y ANOVAs

# BoxPlots de la producción según el modo de cultivo
plot(datos.At$PRODUCCION~as.factor(datos.At$MODO),xlab='Modo',ylab='Producción')

# Por campaña
par(mfrow=c(2,4))
for (i in 14:21){
  plot(subset(datos.At,CAMPAÑA==i)$PRODUCCION~as.factor(subset(datos.At,CAMPAÑA==i)$MODO),
       main=paste('Campaña ', i),xlab='',ylab='')
}
par(mfrow=c(1,1))

# Parece apreciarse una ligera mayor producción media en parcelas que emplean el **MODO 2**, lo cual justificaría la tendencia mencionada. Además, apreciamos que las parcelas con mayor producción son aquellas que emplean el MODO 2. A continuación realizamos el ANOVA, que vemos resulta significativo.

anv1<-aov(PRODUCCION~as.factor(MODO), data=datos_sep)
summary(anv1)

# Este mismo procedimiento lo aplicaremos con las variables ALTITUD y SUPERFICIE.
# 

# BoxPlots de la altitud según el modo de cultivo
plot(datos.At$ALT_med2~as.factor(datos.At$MODO),xlab='Modo',ylab='Altitud')

# Por campaña
par(mfrow=c(2,4))
for (i in 14:21){
  plot(subset(datos.At,CAMPAÑA==i)$ALT_med2~as.factor(subset(datos.At,CAMPAÑA==i)$MODO),
       main=paste('Campaña ', i),xlab='',ylab='')
}
par(mfrow=c(1,1))

anv2<-aov(ALT_med2~as.factor(MODO), data=datos_sep)
summary(anv2)

# BoxPlots de la superficie según el modo de cultivo
plot(datos_sep$SUPERFICIE~as.factor(datos_sep$MODO),xlab='Modo',ylab='Superficie')

# Anova
anv3<-aov(SUPERFICIE~as.factor(MODO), data=datos_sep)
summary(anv3)

# A modo de resumen vemos que todos los ANOVAs resultan significativos salvo el de la ALTITUD.
#Parece que en las fincas de mayor superficie el modo 2 es más utilizado, aunque las dos fincas más extensas emplean el modo 1.

# Color - Modo

table(datos$COLOR,datos$MODO)
heatmap(table(datos$COLOR,datos$MODO))

# Para el COLOR 0 predomina el MODO 1 y para el COLOR 1 predomina el MODO 2. Aún así, las cantidades no son muy diferentes.
#Probablemente si nos restringimos a campañas posteriores a 2019 veríamos que el MODO predominante es el 2 para ambos colores.
table(datos_sep$COLOR,datos_sep$MODO)
heatmap(table(datos_sep$COLOR,datos_sep$MODO))

# Variedad - Modo

t(table(datos$VARIEDAD,datos$MODO))
heatmap(table(datos$VARIEDAD,datos$MODO))

# Se aprecia un claro dominio del MODO 2 entre las distintas variedades, con algunas excepciones.

t(table(datos_sep$VARIEDAD,datos_sep$MODO))
heatmap(table(datos_sep$VARIEDAD,datos_sep$MODO))

# Estación - Modo

t(table(datos$ID_ESTACION,datos$MODO))
heatmap(table(datos$ID_ESTACION,datos$MODO))

# Vemos que en función de las estaciones se aprecia una mayor igualdad en los modos de cultivo, auqnue parece que el MODO 2 sigue dominando.
# 

t(table(datos_sep$ID_ESTACION,datos_sep$MODO))
heatmap(table(datos_sep$ID_ESTACION,datos_sep$MODO))

# En el caso de la campaña 20, vemos que el dominio del MODO 2 es claro, también debido a que hay un menor número de estaciones.






# 4. ESTACIÓN 


#   4.1 Gráfico de barras

table(datos$ID_ESTACION)
datos2 = datos %>% mutate(ID_ESTACION_F = as.factor(ID_ESTACION)) %>% group_by(ID_ESTACION_F) %>% summarise(CUENTA_ESTACION = n())
barplot(datos2$CUENTA_ESTACION~datos2$ID_ESTACION_F)

# Parece que la variable tiene algunas estaciones que apenas tienen fincas asociadas, mientras que otras tienen muchas. Las estaciones: 0, 1, 4, 13, 17 tienen todas menos de 16 observaciones. Quizás estas resquerirán un tratamiento especial. 

#   4.2 Boxplots y ANOVAs 
# A continuación se procede a representar gráficos de cajas para las categorías PRODUCCIÓN, ALT_med2 (la que tiene NAs), SUPERFICIE Y EFICIENCIA.
# Además se incluyen anovas para dichas variables, resultando significativos TODOS LOS CONTRASTES REALIZADOS. 
#Al aplicar el test de Tuckey se puede ver que la ESTACION 6 genera significativamente más producción que el resto. 
#También se aprecia que las alturas medias de las estaciones parecen estar en dos grupos. 
# 
# Los boxplots generados son los siguientes:

par(mfrow=c(2,2))
#PRODUCCION ~ID_ESTACION
boxplot(datos_sep$PRO~datos_sep$ID_ESTACION) 

#ALTITUD ~ID_ESTACION
boxplot(datos_sep$ALT_med2~datos_sep$ID_ESTACION)

#SUPERFICIE ~ID_ESTACION 
boxplot(datos_sep$SUPERFICIE~datos_sep$ID_ESTACION)

#EFICIENCIA ~ID_ESTACION
boxplot(datos_sep$EFICIENCIA~datos_sep$ID_ESTACION)
par(mfrow=c(1,1))

anv1<-aov(PRODUCCION~ID_ESTACION.f, data=datos_sep)
summary(anv1)
anv2<-aov(ALT_med2~ID_ESTACION.f, data=datos_sep)
summary(anv2)
anv3<-aov(SUPERFICIE~ID_ESTACION.f,data=datos_sep)
summary(anv3) 
anv4<-aov(EFICIENCIA~ID_ESTACION.f,data=datos_sep)
summary(anv4)

TukeyHSD(anv1,"ID_ESTACION.f")$ID_ESTACION.f[TukeyHSD(anv1,"ID_ESTACION.f")$ID_ESTACION.f[,4]<0.05,]
TukeyHSD(anv2,"ID_ESTACION.f")$ID_ESTACION.f[TukeyHSD(anv2,"ID_ESTACION.f")$ID_ESTACION.f[,4]<0.05,]
TukeyHSD(anv3,"ID_ESTACION.f")$ID_ESTACION.f[TukeyHSD(anv3,"ID_ESTACION.f")$ID_ESTACION.f[,4]<0.05,]
TukeyHSD(anv4,"ID_ESTACION.f")$ID_ESTACION.f[TukeyHSD(anv4,"ID_ESTACION.f")$ID_ESTACION.f[,4]<0.05,]

#¿Influye en la producción?

meadia_prod_estac = datos %>% group_by(ID_ESTACION) %>% summarise(media = mean(PRODUCCION,na.rm = T))
plot(meadia_prod_estac$ID_ESTACION, meadia_prod_estac$media)
points(meadia_prod_estac$ID_ESTACION[c(1,3,5,7,14)], meadia_prod_estac$media[c(1,3,5,7,14)], col = 2, pch = 16)
points(meadia_prod_estac$ID_ESTACION[-c(1,3,5,7,14)], meadia_prod_estac$media[-c(1,3,5,7,14)], col = 1, pch = 16)
abline(a = 20500, b = 0)

# Parece que podemos separar en 2 clusters, empleemos kmeans:

cluster = kmeans(meadia_prod_estac, centers = 2, iter.max = 100, nstart = 1)
cluster$cluster #Da el mismo resultado que nuestro exploratorio manual

# En vista de estos resultados, parece que la creación de una variable indicadora del cluster podría dar información acerca de la producción.

t(table(datos$ID_ESTACION, datos$MODO))
heatmap(table(datos$ID_ESTACION, datos$MODO))

# No parece haber una conclusión relevante, únicamente, que el modo 2 parece ser prevalente.

t(table(datos$ID_ESTACION, datos$COLOR))
heatmap(table(datos$ID_ESTACION, datos$COLOR))

# Parece que los el COLOR = 1 domina en todas las estaciones a **excepción de la número 0, donde domina el COLOR = 0.**

meadia_alt = datos_sep %>% group_by(ID_ESTACION) %>% summarise(media = mean(ALT_med2,na.rm = T))
plot(meadia_alt$ID_ESTACION, meadia_alt$media)
points(meadia_alt$ID_ESTACION[c(1,3,4,5,6,7,12,13,14,15,16)], meadia_alt$media[c(1,3,4,5,6,7,12,13,14,15,16)], col = 2, pch = 16)
points(meadia_alt$ID_ESTACION[-c(1,3,4,5,6,7,12,13,14,15,16)], meadia_alt$media[-c(1,3,4,5,6,7,12,13,14,15,16)], col = 1, pch = 16)
abline(a = 560, b = 0)

# De nuevo, se puede ver que los ID_ESTACION se pueden separar en dos grupos diferenciados según su altura. Podría ser útil para distinguir aquellos a gran altitud de lo de menor altitud. Este análisis se podrá completar al emplear los datos meteorológicos.



# 5. TIPO

#   5.1 Gráfico de barras

table(datos$TIPO)
datos2 = datos %>% group_by(TIPO) %>% summarise(CUENTA_TIPO = n())
barplot(datos2$CUENTA_TIPO~datos2$TIPO)

# Parece clara la enorme prevalencia del tipo $0$ frente al $1$.

#   5.1 Boxplots y ANOVAs 
# A continuación se procede a representar gráficos de cajas para las categorías PRODUCCIÓN, ALT_med2 (la que tiene NAs), SUPERFICIE Y EFICIENCIA.
# Además se incluyen anovas para dichas variables, resultando significativos TODOS LOS CONTRASTES REALIZADOS **A EXCEPCIÓN DEL DE PRODUCCIÓN**. Parece que las fincas con TIPO = 1 tienen mayores superficies que las otras (aunque son muchas menos). 
# Los boxplots generados son los siguientes:

par(mfrow=c(2,2))
#PRODUCCION ~ID_ESTACION
boxplot(datos_sep$PRO~datos_sep$TIPO) 

#ALTITUD ~ID_ESTACION
boxplot(datos_sep$ALT_med2~datos_sep$TIPO)

#SUPERFICIE ~ID_ESTACION 
boxplot(datos_sep$SUPERFICIE~datos_sep$TIPO)

#EFICIENCIA ~ID_ESTACION
boxplot(datos_sep$EFICIENCIA~datos_sep$TIPO)
par(mfrow=c(1,1))

anv1<-aov(PRODUCCION~TIPO.f, data=datos_sep)
summary(anv1)
anv2<-aov(ALT_med2~TIPO.f, data=datos_sep)
summary(anv2)
anv3<-aov(SUPERFICIE~TIPO.f,data=datos_sep)
summary(anv3) 
anv4<-aov(EFICIENCIA~TIPO.f,data=datos_sep)
summary(anv4)

summary(datos$TIPO)
table(datos$COLOR,datos$TIPO) #Color es uva blanca o negra, Tipo es tipo de cultivo
heatmap(table(datos$COLOR,datos$TIPO))

table(datos$MODO,datos$TIPO)
heatmap(table(datos$MODO,datos$TIPO))

# Parece que hay muchas más uva de color 1 que de color 0.
# Además el tipo de cultivo es claramente el 0, el 1 apenas hay 118 fincas a lo largo de los años.
# 
# Cabe destacar que los MODOS de cultivo parecen ser mitad y mitad.

t(table(datos$VARIEDAD,datos$TIPO))
heatmap(table(datos$VARIEDAD,datos$TIPO)) 

# Siempre hay mas TIPO 0  que TIPO 1, para casi todas las variedades (como cabía esperar).
# No obstante, en la VARIEDAD 4 el TIPO es unicamente 1

# ¿Influye la ESTACION en el TIPO?

t(table(datos$ID_ESTACION,datos$TIPO))
heatmap(table(datos$ID_ESTACION,datos$TIPO)) 

# Parece que el dominio del TIPO = 0  sucede a lo largo de todas las estaciones.



#### ANÁLISIS MULTIVARIANTE

#Hagamos los plots de todas las variables entre si (FALLA)
scatterplotMatrix(datos_sep[, c(11,10, 14, 15, 17)], diagonal =list(method ="histogram",breaks="FD"), smooth = FALSE, regLine = TRUE)
cor(datos_sep[, c(11,10, 14, 15, 17)])

# Con lo estudiado hasta ahora podemos afirmar que las variables más correlacionadas con la PRODUCCION son la SUPERFICIE y el ID_ESTACION. Ya que las otras categóricas no devuelven unos ANOVAS significativos. Veamos ahora si puede haber alguna otra relación entre las variables, para ello emplearemos distintas correlaciones:

#Al meter la librería que se requiere para hacerla dan errores otras cosas, por lo que no se ha puesto aun (MIRAR COMO RESOLVER)
biserial.cor(x = datos$PRODUCCION, y = datos$COLOR, use =  "complete.obs") #ES BAJA
biserial.cor(x = datos$PRODUCCION, y = datos$TIPO, use =  "complete.obs") #ES BAJA
biserial.cor(x = datos$PRODUCCION, y = datos$MODO, use =  "complete.obs") #PARECE UN POCO MAYOR

# Puede que la variable MODO aporte algo más de información sobre la producción.




#### ESTUDIO DE LOS NAs

vis_miss(datos)
pct_miss(datos)
n_miss(datos)
gg_miss_var(datos, show_pct = TRUE)
gg_miss_upset(datos)

ggplot(datos,
       aes(x = SUPERFICIE,
           y = PRODUCCION)) +
  geom_miss_point() +
  theme_dark()

ggplot(datos,
       aes(x = SUPERFICIE,
           y = PRODUCCION)) +
  geom_miss_point() + 
  facet_wrap(~TIPO) +
  theme_dark()

ggplot(datos,
       aes(x = SUPERFICIE,
           y = PRODUCCION)) +
  geom_miss_point() + 
  facet_wrap(~COLOR) +
  theme_dark()

ggplot(datos,
       aes(x = SUPERFICIE,
           y = PRODUCCION)) +
  geom_miss_point() + 
  facet_wrap(~VARIEDAD) +
  theme_dark()

ggplot(datos,
       aes(x = SUPERFICIE,
           y = PRODUCCION)) +
  geom_miss_point() + 
  facet_wrap(~MODO) +
  theme_dark()

ggplot(datos,
       aes(x = SUPERFICIE,
           y = PRODUCCION)) +
  geom_miss_point() + 
  facet_wrap(~ID_ESTACION) +
  theme_dark()


gg_miss_var(datos, facet = TIPO)
gg_miss_var(datos, facet = MODO)
gg_miss_var(datos, facet = ID_ESTACION)
gg_miss_var(datos, facet = COLOR)
gg_miss_var(datos, facet = VARIEDAD)


gg_miss_case(datos)
gg_miss_case(datos, facet = TIPO)
gg_miss_case(datos, facet = MODO)
gg_miss_case(datos, facet = ID_ESTACION)
gg_miss_case(datos, facet = COLOR)
gg_miss_case(datos, facet = VARIEDAD)

gg_miss_fct(x = datos, fct = TIPO)
gg_miss_fct(x = datos, fct = MODO)
gg_miss_fct(x = datos, fct = COLOR)
gg_miss_fct(x = datos, fct = VARIEDAD)
gg_miss_fct(x = datos, fct = ID_ESTACION)

ggplot(datos,
       aes(x = SUPERFICIE,
           y = as.numeric(ALTITUD))) +
  geom_miss_point() + 
  facet_wrap(~ID_ESTACION) +
  theme_dark()

gg_miss_case_cumsum(datos)
gg_miss_var_cumsum(datos)


# En base a los gráficos estudiados, creemos que es razonable suponer la hipótesis del MAR.
#Lo único que parece extraño es la acumulación de los NAs en la producción de las últimas observaciones.
#Que se debe a que superficie está en lso ultimos años, y tienen NAs y a que faltan todas las producciones de 2022



#Vemos que hay datos en las superficies de los años 20, 21 y 22 que son 0, lo cual es imposible. Procederemos a imputarlos:
dim(datos[datos$CAMPAÑA == 20 & datos$SUPERFICIE == 0, ])[1] / dim(datos[datos$CAMPAÑA == 20, ])[1] #1.2922 %
dim(datos[datos$CAMPAÑA == 21 & datos$SUPERFICIE == 0, ])[1] / dim(datos[datos$CAMPAÑA == 21, ])[1] #1.9157 %
dim(datos[datos$CAMPAÑA == 22 & datos$SUPERFICIE == 0, ])[1] / dim(datos[datos$CAMPAÑA == 22, ])[1] #0.8372 %







##### Análisis exploratorio (DATOS_METEO)

clima <- read.table("DATOS_METEO.TXT", sep = "|", header = TRUE, encoding = "UTF-8")
head(clima)
dim(clima)
any(year(clima$validTimeUtc) == 2014) #NO HAY DATOS METEOROLÉGICOS DE 2014!!!

min(clima$validTimeUtc) #De hecho NO hay datos tampocopara 2015, pues tan solo existen los del dis 29 y 30 de junio

#Separamos por temporadas para que el dataset sea mas manejable (sino las funciones para visualizar NAs no funcionan)
clima_2015 = subset(clima, validTimeUtc >= as.Date('2014-07-1') & validTimeUtc <= as.Date('2015-06-30') )
clima_2016 = subset(clima, validTimeUtc >= as.Date('2015-07-1') & validTimeUtc <= as.Date('2016-06-30') )
clima_2017 = subset(clima, validTimeUtc >= as.Date('2016-07-1') & validTimeUtc <= as.Date('2017-06-30') )
clima_2018 = subset(clima, validTimeUtc >= as.Date('2017-07-1') & validTimeUtc <= as.Date('2018-06-30') )
clima_2019 = subset(clima, validTimeUtc >= as.Date('2018-07-1') & validTimeUtc <= as.Date('2019-06-30') )
clima_2020 = subset(clima, validTimeUtc >= as.Date('2019-07-1') & validTimeUtc <= as.Date('2020-06-30') )
clima_2021 = subset(clima, validTimeUtc >= as.Date('2020-07-1') & validTimeUtc <= as.Date('2021-06-30') )
clima_2022 = subset(clima, validTimeUtc >= as.Date('2021-07-1') & validTimeUtc <= as.Date('2022-06-30') )

# En consecuencia, en el año 2014 no podremos utilizar nada de este modelo. TAMPOCO SE USARÁ DEMASIADO DEL 2015.



#### Análisis de NAs 

# Año 2015:

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_2015_red = clima_2015 %>% sample_frac(0.2)
vis_miss(clima_2015_red)
pct_miss(clima_2015_red)
n_miss(clima_2015_red)
gg_miss_var(clima_2015_red, show_pct = TRUE)
gg_miss_upset(clima_2015_red)

gg_miss_case_cumsum(clima_2015_red)
gg_miss_var_cumsum(clima_2015_red)

# Esto nos indica que no podemos utilizar NINGUNA de las siguientes variables:
# windDirection, snoYtd, snowSeason, snowMtd, snow7Day, snow3Day, snow2Day, pressureMeanSeaLevel, precipYtd, precipMtd, 
# precip7Day, precip3Day, precicp2Day, windGust

#Cabe destacar que en general el año 2015 no se puede emplear ya que los datos meteorológicos que
#podríamos emplear en dicha campaña son de 1 único día.

#Los datos meteorológicos comienzan el 30-06-2015




# Año 2016:
#Para que la librería pueda trabajar, escogemos una submuestra:

clima_2016_red = clima_2016 %>% sample_frac(0.15)
vis_miss(clima_2016_red)
pct_miss(clima_2016_red)
n_miss(clima_2016_red)
gg_miss_var(clima_2016_red, show_pct = TRUE)
gg_miss_upset(clima_2016_red)

gg_miss_case_cumsum(clima_2016_red)
gg_miss_var_cumsum(clima_2016_red)

# Esto nos indica que no podemos utilizar NINGUNA de las siguientes variables:

#windDirection, snoYtd, snowSeason, snowMtd, snow7Day, snow3Day, snow2Day, pressureMeanSeaLevel, precipYtd, precipMtd, precip7Day, precip3Day,
#precicp2Day, windGust


# Año 2017:

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_2017_red = clima_2017 %>% sample_frac(0.15)
vis_miss(clima_2017_red)
pct_miss(clima_2017_red)
n_miss(clima_2017_red)
gg_miss_var(clima_2017_red, show_pct = TRUE)
gg_miss_upset(clima_2017_red)

gg_miss_case_cumsum(clima_2017_red)
gg_miss_var_cumsum(clima_2017_red)

# Esto nos indica que no podemos utilizar NINGUNA de las siguientes variables:

# windDirection, snoYtd, snowSeason, snowMtd, snow7Day, snow3Day, snow2Day, pressureMeanSeaLevel,
# precipYtd, precipMtd, precip7Day, precip3Day, precicp2Day, windGust

# Las mismas que en 2015 y 2016.



#Año 2018
#Para que la librería pueda trabajar, escogemos una submuestra:

clima_2018_red = clima_2018 %>% sample_frac(0.15)
vis_miss(clima_2018_red)
pct_miss(clima_2018_red)
n_miss(clima_2018_red)
gg_miss_var(clima_2018_red, show_pct = TRUE)
gg_miss_upset(clima_2018_red)

gg_miss_case_cumsum(clima_2018_red)
gg_miss_var_cumsum(clima_2018_red)

# Esto nos indica que no podemos utilizar:
# windGust, snowYtd, snowSeason, snowMtd, snow7Day, snow3Day, snow2Day, precipYtd, precipMtd, precip7Day, precip3Day, precicp2Day

# HEMOS DE SER CONSCIENTES DE QUE HABRÍA QUE IMPUTAR EL 75% DE LOS VALORES

# ¿ESTO SUCEDE PARA ALGUNAS ESTACIONES EN CONCRETO O FALTAN ALEATORIAMENTE A LO LARGO DE LAS ESTACIONES?

ggplot(clima_2018_red,
       aes(x = windSpeed,
           y = snow7Day)) +
  geom_miss_point() + 
  facet_wrap(~ID_ESTACION) +
  theme_dark()

# Parece que faltan aleatoriamente a lo largo de las estaciones.




#Año 2019

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_2019_red = clima_2019 %>% sample_frac(0.15)
vis_miss(clima_2019_red)
pct_miss(clima_2019_red)
n_miss(clima_2019_red)
gg_miss_var(clima_2019_red, show_pct = TRUE)
gg_miss_upset(clima_2019_red)

gg_miss_case_cumsum(clima_2019_red)
gg_miss_var_cumsum(clima_2019_red)
# ÚNICAMETE FALTA LA VARIABLE windGust



#Año 2020

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_2020_red = clima_2020 %>% sample_frac(0.15)
vis_miss(clima_2020_red)
pct_miss(clima_2020_red)
n_miss(clima_2020_red)
gg_miss_var(clima_2020_red, show_pct = TRUE)

#ÚNICAMETE FALTA LA VARIABLE windGust 



#Año 2021

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_2021_red = clima_2021 %>% sample_frac(0.15)
vis_miss(clima_2021_red)
pct_miss(clima_2021_red)
n_miss(clima_2021_red)
gg_miss_var(clima_2021_red, show_pct = TRUE)

#ÚNICAMETE FALTA LA VARIABLE windGust



#Año 2022

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_2022_red = clima_2022 %>% sample_frac(0.15)
vis_miss(clima_2022_red)
pct_miss(clima_2022_red)
n_miss(clima_2022_red)
gg_miss_var(clima_2022_red, show_pct = TRUE)

#ÚNICAMETE FALTA LA VARIABLE windGust




#### SEPARACIÓN DE LOS DATOS POR VARIABLES DE INTERÉS


#Temperaturas por años

#Asignaremos las temperaturas que se pueden emplear en cada temporada:
datos_temperaturas_2022 = clima_2022[,c(1,22:27, 33)]
datos_temperaturas_2021 = clima_2021[,c(1,22:27, 33)]
datos_temperaturas_2020 = clima_2020[,c(1,22:27, 33)]
datos_temperaturas_2019 = clima_2019[,c(1,22:27, 33)]
datos_temperaturas_2018 = clima_2018[,c(1,22:27, 33)]
datos_temperaturas_2017 = clima_2017[,c(1,22:27, 33)]
datos_temperaturas_2016 = clima_2016[,c(1,22:27, 33)]
datos_temperaturas_2015 = clima_2015[,c(1,22:27, 33)]

install.packages('fda.usc')
library(fda.usc)

#Ahora los separaremos por estaciones. Cabe destacar que en caso de querer emplear el máximo o el mínimo diario será mejor usar los de la 
#variable resumen!!!
#Pues aquí salen repetidos varias veces a lo largo del día!!!



#### TEMPERATURA TEMPORADA 21/22 ###
Temperaturas_hora_2022 <- subset(datos_temperaturas_2022,ID_ESTACION==0)$temperature #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2022 <- cbind(Temperaturas_hora_2022, subset(datos_temperaturas_2022, ID_ESTACION==i)$temperature)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2022)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2022.fd = smooth.basis(y=Temperaturas_hora_2022, fdParobj=bspline.basis)

plot(Temperaturas_hora_2022.fd,main='Cruvas de temperatura para cada estación 21/22',xlab='Tiempo',ylab='Temperatura')



#### TEMPERATURA TEMPORADA 20/21 ###
Temperaturas_hora_2021 <- subset(datos_temperaturas_2021,ID_ESTACION==0)$temperature #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2021 <- cbind(Temperaturas_hora_2021, subset(datos_temperaturas_2021, ID_ESTACION==i)$temperature)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2021)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2021.fd = smooth.basis(y=Temperaturas_hora_2021, fdParobj=bspline.basis)

plot(Temperaturas_hora_2021.fd,main='Cruvas de temperatura para cada estación 20/21',xlab='Tiempo',ylab='Temperatura')




#### TEMPERATURA TEMPORADA 19/20 ###
Temperaturas_hora_2020 <- subset(datos_temperaturas_2020,ID_ESTACION==0)$temperature #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2020 <- cbind(Temperaturas_hora_2020, subset(datos_temperaturas_2020, ID_ESTACION==i)$temperature)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2020)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2020.fd = smooth.basis(y=Temperaturas_hora_2020, fdParobj=bspline.basis)

plot(Temperaturas_hora_2020.fd,main='Cruvas de temperatura para cada estación 19/20',xlab='Tiempo',ylab='Temperatura')





#### TEMPERATURA TEMPORADA 18/19 ###
Temperaturas_hora_2019 <- subset(datos_temperaturas_2019,ID_ESTACION==0)$temperature #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2019 <- cbind(Temperaturas_hora_2019, subset(datos_temperaturas_2019, ID_ESTACION==i)$temperature)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2019)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2019.fd = smooth.basis(y=Temperaturas_hora_2019, fdParobj=bspline.basis)

plot(Temperaturas_hora_2019.fd,main='Cruvas de temperatura para cada estación 18/19',xlab='Tiempo',ylab='Temperatura')





#### TEMPERATURA TEMPORADA 17/18 ###
Temperaturas_hora_2018 <- subset(datos_temperaturas_2018,ID_ESTACION==0)$temperature #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2018 <- cbind(Temperaturas_hora_2018, subset(datos_temperaturas_2018, ID_ESTACION==i)$temperature)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2018)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2018.fd = smooth.basis(y=Temperaturas_hora_2018, fdParobj=bspline.basis)

plot(Temperaturas_hora_2018.fd,main='Cruvas de temperatura para cada estación 17/18',xlab='Tiempo',ylab='Temperatura')



#### TEMPERATURA TEMPORADA 16/17 ###
Temperaturas_hora_2017 <- subset(datos_temperaturas_2017,ID_ESTACION==0)$temperature #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2017 <- cbind(Temperaturas_hora_2017, subset(datos_temperaturas_2017, ID_ESTACION==i)$temperature)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2017)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2017.fd = smooth.basis(y=Temperaturas_hora_2017, fdParobj=bspline.basis)

plot(Temperaturas_hora_2017.fd,main='Cruvas de temperatura para cada estación 16/17',xlab='Tiempo',ylab='Temperatura')




#### TEMPERATURA TEMPORADA 14/15 ###
Temperaturas_hora_2015 <- subset(datos_temperaturas_2015,ID_ESTACION==0)$temperature #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2015 <- cbind(Temperaturas_hora_2015,subset(datos_temperaturas_2015, ID_ESTACION==i)$temperature)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2015)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2015.fd = smooth.basis(y=Temperaturas_hora_2015, fdParobj=bspline.basis)

plot(Temperaturas_hora_2015.fd,main='Cruvas de temperatura para cada estación 14/15',xlab='Tiempo',ylab='Temperatura')


#Da un error pero al ejecutar pero funciona el plot igual
#Se debe a que tan solo hay dos dias de datos




#### Precipitaciones por años

#Asignaremos las temperaturas que se pueden emplear en cada temporada:
datos_precipitaciones_2022 = clima_2022[,c(1:9, 33)]
datos_precipitaciones_2021 = clima_2021[,c(1:9, 33)]
datos_precipitaciones_2020 = clima_2020[,c(1:9, 33)]
datos_precipitaciones_2019 = clima_2019[,c(1:9, 33)]
datos_precipitaciones_2018 = clima_2018[,c(1:9, 33)]
datos_precipitaciones_2017 = clima_2017[,c(1:9, 33)]
datos_precipitaciones_2016 = clima_2016[,c(1:9, 33)]
datos_precipitaciones_2015 = clima_2015[,c(1:9, 33)]

#Ahora los separaremos por estaciones. Cabe destacar que en caso de querer emplear las precipitaciones 
#por dias podrían obtenerse sumando las horarias, de todas formas, puede ser mejor emplar las variables meteorologicas del resumen
#Pues aquí salen repetidos varias veces a lo largo del día!!!



#### PRECIPITACIONES TEMPORADA 21/22 ###
Precipitaciones_hora_2022 <- subset(datos_precipitaciones_2022,ID_ESTACION==0)$precip1Hour #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2022 <- cbind(Precipitaciones_hora_2022, subset(datos_precipitaciones_2022, ID_ESTACION==i)$precip1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2022)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2022.fd = smooth.basis(y=Precipitaciones_hora_2022, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2022.fd,main='Cruvas de precipitaciones para cada estación 21/22',xlab='Tiempo',ylab='Precipitaciones')



#### PRECIPITACIONES TEMPORADA 20/21 ###
Precipitaciones_hora_2021 <- subset(datos_precipitaciones_2021,ID_ESTACION==0)$precip1Hour #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2021 <- cbind(Precipitaciones_hora_2021, subset(datos_precipitaciones_2021, ID_ESTACION==i)$precip1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2021)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2021.fd = smooth.basis(y=Precipitaciones_hora_2021, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2021.fd,main='Cruvas de precipitaciones para cada estación 20/21',xlab='Tiempo',ylab='Precipitaciones')




#### PRECIPITACIONES TEMPORADA 19/20 ###
Precipitaciones_hora_2020 <- subset(datos_precipitaciones_2020,ID_ESTACION==0)$precip1Hour #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2020 <- cbind(Precipitaciones_hora_2020, subset(datos_precipitaciones_2020, ID_ESTACION==i)$precip1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2020)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2020.fd = smooth.basis(y=Precipitaciones_hora_2020, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2020.fd,main='Cruvas de precipitaciones para cada estación 19/20',xlab='Tiempo',ylab='Precipitaciones')



#### PRECIPITACIONES TEMPORADA 18/19 ###
Precipitaciones_hora_2019 <- subset(datos_precipitaciones_2019,ID_ESTACION==0)$precip1Hour #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2019 <- cbind(Precipitaciones_hora_2019, subset(datos_precipitaciones_2019, ID_ESTACION==i)$precip1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2019)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2019.fd = smooth.basis(y=Precipitaciones_hora_2019, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2019.fd,main='Cruvas de precipitaciones para cada estación 18/19',xlab='Tiempo',ylab='Precipitaciones')




#### PRECIPITACIONES TEMPORADA 17/18 ###
Precipitaciones_hora_2018 <- subset(datos_precipitaciones_2018,ID_ESTACION==0)$precip1Hour #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2018 <- cbind(Precipitaciones_hora_2018, subset(datos_precipitaciones_2018, ID_ESTACION==i)$precip1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2018)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2018.fd = smooth.basis(y=Precipitaciones_hora_2018, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2018.fd,main='Cruvas de precipitaciones para cada estación 17/18',xlab='Tiempo',ylab='Precipitaciones')




#### PRECIPITACIONES TEMPORADA 16/17 ###
Precipitaciones_hora_2017 <- subset(datos_precipitaciones_2017,ID_ESTACION==0)$precip1Hour #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2017 <- cbind(Precipitaciones_hora_2017, subset(datos_precipitaciones_2017, ID_ESTACION==i)$precip1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2017)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2017.fd = smooth.basis(y=Precipitaciones_hora_2017, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2017.fd,main='Cruvas de precipitaciones para cada estación 16/17',xlab='Tiempo',ylab='Precipitaciones')




#### PRECIPITACIONES TEMPORADA 15/16 ###
  #Precipitaciones_hora_2016 <- subset(datos_precipitaciones_2016,ID_ESTACION==0)$precip1Hour #Es la temperatura por hora
  #for (i in 1:19){
    #Precipitaciones_hora_2016 <- cbind(Precipitaciones_hora_2016, subset(datos_precipitaciones_2016, ID_ESTACION==i)$precip1Hour)
  #}

# Creamos la base de splines a partir de la cual se va a construir la curva 
  #bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2016)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
  #Precipitaciones_hora_2016.fd = smooth.basis(y=Precipitaciones_hora_2016, fdParobj=bspline.basis)

  #plot(Precipitaciones_hora_2016.fd,main='Cruvas de precipitaciones para cada estación 15/16',xlab='Tiempo',ylab='Precipitaciones')




#### PRECIPITACIONES TEMPORADA 14/15 ###
Precipitaciones_hora_2015 <- subset(datos_precipitaciones_2015,ID_ESTACION==0)$precip1Hour #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2015 <- cbind(Precipitaciones_hora_2015,subset(datos_precipitaciones_2015, ID_ESTACION==i)$precip1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2015)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2015.fd = smooth.basis(y=Precipitaciones_hora_2015, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2015.fd,main='Cruvas de precipitaciones para cada estación 14/15',xlab='Tiempo',ylab='Precipitaciones')

#No hay precipitaciones en los dias de junio que existen




#NIEVE POR AÑOS:

#Asignaremos las temperaturas que se pueden emplear en cada temporada:
datos_nieves_2022 = clima_2022[,c(1,13:21,33)]
datos_nieves_2021 = clima_2021[,c(1,13:21,33)]
datos_nieves_2020 = clima_2020[,c(1,13:21,33)]
datos_nieves_2019 = clima_2019[,c(1,13:21,33)]
datos_nieves_2018 = clima_2018[,c(1,13:21,33)]
datos_nieves_2017 = clima_2017[,c(1,13:21,33)]
datos_nieves_2016 = clima_2016[,c(1,13:21,33)]
datos_nieves_2015 = clima_2015[,c(1,13:21,33)]
#Ahora los separaremos por estaciones. Cabe destacar que en caso de querer emplear las precipitaciones 



#### NIEVES TEMPORADA 21/22 ###
Nieves_hora_2022 <- subset(datos_nieves_2022,ID_ESTACION==0)$snow1Hour #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2022 <- cbind(Nieves_hora_2022, subset(datos_nieves_2022, ID_ESTACION==i)$snow1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2022)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2022.fd = smooth.basis(y=Nieves_hora_2022, fdParobj=bspline.basis)

plot(Nieves_hora_2022.fd,main='Cruvas de nieve para cada estación 21/22',xlab='Tiempo',ylab='Nieve')



#### NIEVES TEMPORADA 20/21 ###
Nieves_hora_2021 <- subset(datos_nieves_2021,ID_ESTACION==0)$snow1Hour #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2021 <- cbind(Nieves_hora_2021, subset(datos_nieves_2021, ID_ESTACION==i)$snow1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2021)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2021.fd = smooth.basis(y=Nieves_hora_2021, fdParobj=bspline.basis)

plot(Nieves_hora_2021.fd,main='Cruvas de nieve para cada estación 20/21',xlab='Tiempo',ylab='Nieve')



#### NIEVES TEMPORADA 19/20 ###
Nieves_hora_2020 <- subset(datos_nieves_2020,ID_ESTACION==0)$snow1Hour #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2020 <- cbind(Nieves_hora_2020, subset(datos_nieves_2020, ID_ESTACION==i)$snow1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2020)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2020.fd = smooth.basis(y=Nieves_hora_2020, fdParobj=bspline.basis)

plot(Nieves_hora_2020.fd,main='Cruvas de nieve para cada estación 19/20',xlab='Tiempo',ylab='Nieve')



#### NIEVES TEMPORADA 18/19 ###
Nieves_hora_2019 <- subset(datos_nieves_2019,ID_ESTACION==0)$snow1Hour #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2019 <- cbind(Nieves_hora_2019, subset(datos_nieves_2019, ID_ESTACION==i)$snow1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2019)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2019.fd = smooth.basis(y=Nieves_hora_2019, fdParobj=bspline.basis)

plot(Nieves_hora_2019.fd,main='Cruvas de nieve para cada estación 18/19',xlab='Tiempo',ylab='Nieve')

#No nevaría en 2019 


#### NIEVES TEMPORADA 17/18 ###
Nieves_hora_2018 <- subset(datos_nieves_2018,ID_ESTACION==0)$snow1Hour #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2018 <- cbind(Nieves_hora_2018, subset(datos_nieves_2018, ID_ESTACION==i)$snow1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2018)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2018.fd = smooth.basis(y=Nieves_hora_2018, fdParobj=bspline.basis)

plot(Nieves_hora_2018.fd,main='Cruvas de nieve para cada estación 17/18',xlab='Tiempo',ylab='Nieve')




#### NIEVES TEMPORADA 16/17 ###
Nieves_hora_2017 <- subset(datos_nieves_2017,ID_ESTACION==0)$snow1Hour #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2017 <- cbind(Nieves_hora_2017, subset(datos_nieves_2017, ID_ESTACION==i)$snow1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2017)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2017.fd = smooth.basis(y=Nieves_hora_2017, fdParobj=bspline.basis)

plot(Nieves_hora_2017.fd,main='Cruvas de nieve para cada estación 16/17',xlab='Tiempo',ylab='Nieve')



#### NIEVES TEMPORADA 15/16 ###
  #Nieves_hora_2016 <- subset(datos_nieves_2016,ID_ESTACION==0)$snow1Hour #Es la temperatura por hora
  #for (i in 1:19){
    #Nieves_hora_2016 <- cbind(Nieves_hora_2016, subset(datos_nieves_2016, ID_ESTACION==i)$snow1Hour)
  #}

# Creamos la base de splines a partir de la cual se va a construir la curva 
  #bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2016)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
  #Nieves_hora_2016.fd = smooth.basis(y=Nieves_hora_2016, fdParobj=bspline.basis)

  #plot(Nieves_hora_2016.fd,main='Cruvas de nieve para cada estación 15/16',xlab='Tiempo',ylab='Nieve')



#### NIEVES TEMPORADA 14/15 ###
Nieves_hora_2015 <- subset(datos_nieves_2015,ID_ESTACION==0)$snow1Hour #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2015 <- cbind(Nieves_hora_2015,subset(datos_nieves_2015, ID_ESTACION==i)$snow1Hour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2015)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2015.fd = smooth.basis(y=Nieves_hora_2015, fdParobj=bspline.basis)

plot(Nieves_hora_2015.fd,main='Cruvas de nieve para cada estación 14/15',xlab='Tiempo',ylab='Nieve')

#No nieva  en esos dias de junio de los que hay datos


# Por si interesa hacer más de estos creamos la siguiente función que facilita el trabajo

plot_variable_funcional = function(data, var_name, nbasis = 10, year){
  years_plot = c(14, 16:22)
  ind = which(names(data) == var_name)
  Datos_year <- subset(data, ID_ESTACION==0)[,ind] #Es la variable
  for (i in 1:19){
    Datos_year <- cbind(Datos_year,subset(data, ID_ESTACION==i)[,ind])
  }
  
  # Creamos la base de splines a partir de la cual se va a construir la curva 
  bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Datos_year)), nbasis=nbasis)
  
  # Convertimos las temperaturas en curvas (funcionales)
  Datos_year.fd = smooth.basis(y=Datos_year, fdParobj=bspline.basis)
  
  plot(Datos_year.fd, main = paste('Curvas de', var_name, 'para cada estación', paste0(as.character(years_plot[year]),'/',as.character(years_plot[year+ 1]))),xlab='Tiempo',ylab=var_name)    
}

variable = 'snow24Hour'
nb = 4
plot_variable_funcional(data = clima_2015 , var_name = variable, nbasis = nb, year = 1)
plot_variable_funcional(data = clima_2017 , var_name = variable, nbasis = nb, year = 3)
plot_variable_funcional(data = clima_2018 , var_name = variable, nbasis = nb, year = 4)
plot_variable_funcional(data = clima_2019 , var_name = variable, nbasis = nb, year = 5)
plot_variable_funcional(data = clima_2020 , var_name = variable, nbasis = nb, year = 6)
plot_variable_funcional(data = clima_2021 , var_name = variable, nbasis = nb, year = 7)
plot_variable_funcional(data = clima_2022 , var_name = variable, nbasis = nb, year = 8)





##### Análisis exploratorio (DATOS_ETO) 

clima_tr <- read.table("DATOS_ETO.TXT", sep = "|", header = TRUE, encoding = "UTF-8") # Dataset con datos agregados de las variables en distintos periodos de tiempo (mínimo, máximo y media)
clima_tr$date <- ymd(clima_tr$date)
class(clima_tr$date)

dim(clima_tr)
names(clima_tr)

any(year(clima_tr$date) == 2014) #NO HAY DATOS METEOROLÉGICOS DE 2014!!!

#Separamos por años para que el dataset sea mas manejable (sino las funciones para visualizar NAs no funcionan)
min(clima_tr$date) #De hecho NO hay datos tampocopara 2015, pues tan solo existen los del dis 29 y 30 de junio

#Separamos por temporadas para que el dataset sea mas manejable (sino las funciones para visualizar NAs no funcionan)
clima_tr_2015 = subset(clima_tr, date >= as.Date('2014-07-1') & date <= as.Date('2015-06-30') )
clima_tr_2016 = subset(clima_tr, date >= as.Date('2015-07-1') & date <= as.Date('2016-06-30') )
clima_tr_2017 = subset(clima_tr, date >= as.Date('2016-07-1') & date <= as.Date('2017-06-30') )
clima_tr_2018 = subset(clima_tr, date >= as.Date('2017-07-1') & date <= as.Date('2018-06-30') )
clima_tr_2019 = subset(clima_tr, date >= as.Date('2018-07-1') & date <= as.Date('2019-06-30') )
clima_tr_2020 = subset(clima_tr, date >= as.Date('2019-07-1') & date <= as.Date('2020-06-30') )
clima_tr_2021 = subset(clima_tr, date >= as.Date('2020-07-1') & date <= as.Date('2021-06-30') )
clima_tr_2022 = subset(clima_tr, date >= as.Date('2021-07-1') & date <= as.Date('2022-06-30') )

# En consecuencia, en el año 2014 no podremos utilizar nada de este modelo. Y del 2015 tenemos apenas un par de dias.





#### ANÁLISIS DE NAs


#Año 2015:

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_tr_2015_red = clima_tr_2015 %>% sample_frac(0.2)
vis_miss(clima_tr_2015_red)
pct_miss(clima_tr_2015_red)
n_miss(clima_2015_red)
v=gg_miss_var(clima_tr_2015_red, show_pct = TRUE)
v
gg_miss_upset(clima_tr_2015_red)
gg_miss_case_cumsum(clima_tr_2015_red)
gg_miss_var_cumsum(clima_tr_2015_red)
cat('Las que tienen demasiados NAs')
v$data$variable[which(v$data$pct_miss > 1)] #No se pueden emplear (en realidad tienen mas de un 70% de NAs)

cat('Las que tienen pocos o ningún NAs')
v$data$variable[which(v$data$pct_miss < 1)] #Se pueden emplear

#Si nos fijamos en los porcentajes vemos que se repiten mucho los mismos valores exactos, lo que puede hacer pensar que ciertas 
#variables no se han medido por sistema en algunas estaciones!

# Se puede decir que son empleables todas las variables a excepcion de las relacionadas con:
# Evapotranspiration, GlobalHorizontalIrradiance, MSLPLocal, GustLocal

# A mayores hay unas cuantas variables que también tienen bastantes datos faltantes. No obstante, se pueden escoger variables "similares" 
#que no los tengan.




#Año 2016:

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_2016_tr_red = clima_tr_2016 %>% sample_frac(0.15)
vis_miss(clima_2016_tr_red)
pct_miss(clima_2016_tr_red)
n_miss(clima_2016_tr_red)
v = gg_miss_var(clima_2016_tr_red, show_pct = TRUE)
gg_miss_upset(clima_2016_tr_red)

gg_miss_case_cumsum(clima_2016_tr_red)
gg_miss_var_cumsum(clima_2016_tr_red)

cat('Las que tienen demasiados NAs')
v$data$variable[which(v$data$pct_miss > 1)] #No se pueden emplear (en realidad tienen mas de un 70% de NAs)

cat('Las que tienen pocos o ningún NAs')
v$data$variable[which(v$data$pct_miss < 1)] #Se pueden emplear

# Se puede decir que son empleables todas las variables a excepcion de las relacionadas con:
#Evapotranspiration, GlobalHorizontalIrradiance, MSLPLocal, GustLocal



#Año 2017:

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_tr_2017_red = clima_tr_2017 %>% sample_frac(0.15)
vis_miss(clima_tr_2017_red)
pct_miss(clima_tr_2017_red)
n_miss(clima_tr_2017_red)
v = gg_miss_var(clima_tr_2017_red, show_pct = TRUE)
gg_miss_upset(clima_tr_2017_red)

gg_miss_case_cumsum(clima_tr_2017_red)
gg_miss_var_cumsum(clima_tr_2017_red)

cat('Las que tienen demasiados NAs')
v$data$variable[which(v$data$pct_miss > 1)] #No se pueden emplear (en realidad tienen mas de un 70% de NAs)

cat('Las que tienen pocos o ningún NAs')
v$data$variable[which(v$data$pct_miss < 1)] #Se pueden emplear


# Se puede decir que son empleables todas las variables a excepcion de las relacionadas con:
#Evapotranspiration, GlobalHorizontalIrradiance, MSLPLocal, GustLocal



#Año 2018:

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_tr_2018_red = clima_tr_2018 %>% sample_frac(0.15)
vis_miss(clima_tr_2018_red)
pct_miss(clima_tr_2018_red)
n_miss(clima_tr_2018_red)
v = gg_miss_var(clima_tr_2018_red, show_pct = TRUE)
gg_miss_upset(clima_tr_2018_red)

gg_miss_case_cumsum(clima_tr_2018_red)
gg_miss_var_cumsum(clima_tr_2018_red)


cat('Las que tienen demasiados NAs')
v$data$variable[which(v$data$pct_miss > 1)] #No se pueden emplear (en realidad tienen mas de un 70% de NAs)

cat('Las que tienen pocos o ningún NAs')
v$data$variable[which(v$data$pct_miss < 1)] #Se pueden emplear


# Se puede decir que son empleables todas las variables a excepcion de las relacionadas con:
#Evapotranspiration, GlobalHorizontalIrradiance, GustLocal
# Cabe destacar que ahora se puede emplear la variable MLSP pero a partir de enero del 2018


# ¿Están los datos faltantes repartidos por estaciones o serán aleatorios?

ggplot(clima_tr_2018_red,
       aes(x = PrecipAmountLocalEveningMin,
           y = GustLocalNighttimeMax)) +
  geom_miss_point() + 
  facet_wrap(~ID_ESTACION) +
  theme_dark()

#Parece que faltan aleatoriamente a lo largo de las estaciones.



# Año 2019:

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_tr_2019_red = clima_tr_2019 %>% sample_frac(0.15)
vis_miss(clima_tr_2019_red)
pct_miss(clima_tr_2019_red)
n_miss(clima_tr_2019_red)
v = gg_miss_var(clima_tr_2019_red, show_pct = TRUE)
gg_miss_upset(clima_tr_2019_red)

gg_miss_case_cumsum(clima_tr_2019_red)
gg_miss_var_cumsum(clima_tr_2019_red)


cat('Las que tienen demasiados NAs')
v$data$variable[which(v$data$pct_miss > 1)] #No se pueden emplear (en realidad tienen mas de un 70% de NAs)

cat('Las que tienen pocos o ningún NAs')
v$data$variable[which(v$data$pct_miss < 1)] #Se pueden emplear


# Se puede decir que son empleables todas las variables a excepcion de las relacionadas con:
#Evapotranspiration, GlobalHorizontalIrradiance, GustLocal

#Cabe destacar que ahora se puede emplear la variable , MSLPLocal



#Año 2020:

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_tr_2020_red = clima_tr_2020 %>% sample_frac(0.15)
vis_miss(clima_tr_2020_red)
pct_miss(clima_tr_2020_red)
n_miss(clima_tr_2020_red)
v = gg_miss_var(clima_tr_2020_red, show_pct = TRUE)
cat('Las que tienen demasiados NAs')
v$data$variable[which(v$data$pct_miss > 1)] #No se pueden emplear (en realidad tienen mas de un 70% de NAs)

cat('Las que tienen pocos o ningún NAs')
v$data$variable[which(v$data$pct_miss < 1)] #Se pueden emplear

# Se puede decir que son empleables todas las variables a excepcion de las relacionadas con:
#Evapotranspiration, GlobalHorizontalIrradiance, GustLocal

#Cabe destacar que ahora se puede emplear la variable , MSLPLocal




#Año 2021:

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_tr_2021_red = clima_tr_2021 %>% sample_frac(0.15)
vis_miss(clima_tr_2021_red)
pct_miss(clima_tr_2021_red)
n_miss(clima_tr_2021_red)
v = gg_miss_var(clima_tr_2021_red, show_pct = TRUE)

cat('Las que tienen demasiados NAs')
v$data$variable[which(v$data$pct_miss > 1)] #No se pueden emplear (en realidad tienen mas de un 70% de NAs)

cat('Las que tienen pocos o ningún NAs')
v$data$variable[which(v$data$pct_miss < 1)] #Se pueden emplear

# Se puede decir que son empleables todas las variables a excepcion de las relacionadas con:
#Evapotranspiration, GlobalHorizontalIrradiance, GustLocal

#Cabe destacar que ahora se puede emplear la variable , MSLPLocal



#Año 2022:

#Para que la librería pueda trabajar, escogemos una submuestra:
clima_tr_2022_red = clima_tr_2022 %>% sample_frac(0.15)
vis_miss(clima_tr_2022_red)
pct_miss(clima_tr_2022_red)
n_miss(clima_tr_2022_red)
v = gg_miss_var(clima_tr_2022_red, show_pct = TRUE)

cat('Las que tienen demasiados NAs')
v$data$variable[which(v$data$pct_miss > 1)] #No se pueden emplear (en realidad tienen mas de un 70% de NAs)

cat('Las que tienen pocos o ningún NAs')
v$data$variable[which(v$data$pct_miss < 1)] #Se pueden emplear

# Se puede decir que son empleables todas las variables a excepcion de las relacionadas con:
# GustLocal

# Cabe destacar que ahora se puede emplear la variable , MSLPLocal, Evapotranspiration, GlobalHorizontalIrradiance





#### Separación de los datos por variables de interés


# Temperaturas por años

#Asignaremos las temperaturas que se pueden emplear en cada temporada:
datos_temperaturas_tr_2022 = clima_tr_2022[,c(1,194:196, 275)]
datos_temperaturas_tr_2021 = clima_tr_2021[,c(1,194:196, 275)]
datos_temperaturas_tr_2020 = clima_tr_2020[,c(1,194:196, 275)]
datos_temperaturas_tr_2019 = clima_tr_2019[,c(1,194:196, 275)]
datos_temperaturas_tr_2018 = clima_tr_2018[,c(1,194:196, 275)]
datos_temperaturas_tr_2017 = clima_tr_2017[,c(1,194:196, 275)]
datos_temperaturas_tr_2016 = clima_tr_2016[,c(1,194:196, 275)]
datos_temperaturas_tr_2015 = clima_tr_2015[,c(1,194:196, 275)]


#Ahora los separaremos por estaciones. Cabe destacar que en caso de querer emplear el máximo o el mínimo diario 
#será mejor usar los de la variable resumen!!!
#Pues aquí salen repetidos varias veces a lo largo del día!!!

#### TEMPERATURA TEMPORADA 21/22 ###
Temperaturas_hora_2022 <- subset(datos_temperaturas_tr_2022,ID_ESTACION==0)$TemperatureLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2022 <- cbind(Temperaturas_hora_2022, subset(datos_temperaturas_tr_2022, ID_ESTACION==i)$TemperatureLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2022)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2022.fd = smooth.basis(y=Temperaturas_hora_2022, fdParobj=bspline.basis)

plot(Temperaturas_hora_2022.fd,main='Cruvas de temperatura para cada estación 21/22',xlab='Tiempo',ylab='Temperatura')



#### TEMPERATURA TEMPORADA 20/21 ###
Temperaturas_hora_2021 <- subset(datos_temperaturas_tr_2021,ID_ESTACION==0)$TemperatureLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2021 <- cbind(Temperaturas_hora_2021, subset(datos_temperaturas_tr_2021, ID_ESTACION==i)$TemperatureLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2021)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2021.fd = smooth.basis(y=Temperaturas_hora_2021, fdParobj=bspline.basis)

plot(Temperaturas_hora_2021.fd,main='Cruvas de temperatura para cada estación 20/21',xlab='Tiempo',ylab='Temperatura')



#### TEMPERATURA TEMPORADA 19/20 ###
Temperaturas_hora_2020 <- subset(datos_temperaturas_tr_2020,ID_ESTACION==0)$TemperatureLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2020 <- cbind(Temperaturas_hora_2020, subset(datos_temperaturas_tr_2020, ID_ESTACION==i)$TemperatureLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2020)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2020.fd = smooth.basis(y=Temperaturas_hora_2020, fdParobj=bspline.basis)

plot(Temperaturas_hora_2020.fd,main='Cruvas de temperatura para cada estación 19/20',xlab='Tiempo',ylab='Temperatura')



#### TEMPERATURA TEMPORADA 18/19 ###
Temperaturas_hora_2019 <- subset(datos_temperaturas_tr_2019,ID_ESTACION==0)$TemperatureLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2019 <- cbind(Temperaturas_hora_2019, subset(datos_temperaturas_tr_2019, ID_ESTACION==i)$TemperatureLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2019)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2019.fd = smooth.basis(y=Temperaturas_hora_2019, fdParobj=bspline.basis)

plot(Temperaturas_hora_2019.fd,main='Cruvas de temperatura para cada estación 18/19',xlab='Tiempo',ylab='Temperatura')




#### TEMPERATURA TEMPORADA 17/18 ###
Temperaturas_hora_2018 <- subset(datos_temperaturas_tr_2018,ID_ESTACION==0)$TemperatureLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2018 <- cbind(Temperaturas_hora_2018, subset(datos_temperaturas_tr_2018, ID_ESTACION==i)$TemperatureLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2018)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2018.fd = smooth.basis(y=Temperaturas_hora_2018, fdParobj=bspline.basis)

plot(Temperaturas_hora_2018.fd,main='Cruvas de temperatura para cada estación 17/18',xlab='Tiempo',ylab='Temperatura')



#### TEMPERATURA TEMPORADA 16/17 ###
Temperaturas_hora_2017 <- subset(datos_temperaturas_tr_2017,ID_ESTACION==0)$TemperatureLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2017 <- cbind(Temperaturas_hora_2017, subset(datos_temperaturas_tr_2017, ID_ESTACION==i)$TemperatureLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2017)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2017.fd = smooth.basis(y=Temperaturas_hora_2017, fdParobj=bspline.basis)

plot(Temperaturas_hora_2017.fd,main='Cruvas de temperatura para cada estación 16/17',xlab='Tiempo',ylab='Temperatura')



#### TEMPERATURA TEMPORADA 15/16 ###
Temperaturas_hora_2016 <- subset(datos_temperaturas_tr_2016,ID_ESTACION==0)$TemperatureLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2016 <- cbind(Temperaturas_hora_2016, subset(datos_temperaturas_tr_2016, ID_ESTACION==i)$TemperatureLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2016)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2016.fd = smooth.basis(y=Temperaturas_hora_2016, fdParobj=bspline.basis)

plot(Temperaturas_hora_2016.fd,main='Cruvas de temperatura para cada estación 15/16',xlab='Tiempo',ylab='Temperatura')



#### TEMPERATURA TEMPORADA 14/15 ###
Temperaturas_hora_2015 <- subset(datos_temperaturas_tr_2015,ID_ESTACION==0)$TemperatureLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Temperaturas_hora_2015 <- cbind(Temperaturas_hora_2015,subset(datos_temperaturas_tr_2015, ID_ESTACION==i)$TemperatureLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Temperaturas_hora_2015)), nbasis=8)

# Convertimos las temperaturas en curvas (funcionales)
Temperaturas_hora_2015.fd = smooth.basis(y=Temperaturas_hora_2015, fdParobj=bspline.basis)

plot(Temperaturas_hora_2015.fd,main='Cruvas de temperatura para cada estación 14/15',xlab='Tiempo',ylab='Temperatura')
#Da un error pero al ejecutar pero funciona el plot igual
#Se debe a que tan solo hay dos dias de datos




#Precipitaciones por años

#Asignaremos las temperaturas que se pueden emplear en cada temporada:
datos_precipitaciones_tr_2022 = clima_tr_2022[,c(1, 128:148, 275)]
datos_precipitaciones_tr_2021 = clima_tr_2021[,c(1, 125:148, 275)]
datos_precipitaciones_tr_2020 = clima_tr_2020[,c(1, 125:148, 275)]
datos_precipitaciones_tr_2019 = clima_tr_2019[,c(1, 125:148, 275)]
datos_precipitaciones_tr_2018 = clima_tr_2018[,c(1, 125:148, 275)]
datos_precipitaciones_tr_2017 = clima_tr_2017[,c(1, 125:148, 275)]
datos_precipitaciones_tr_2016 = clima_tr_2016[,c(1, 125:148, 275)]
datos_precipitaciones_tr_2015 = clima_tr_2015[,c(1, 125:148, 275)]

#Ahora los separaremos por estaciones. Cabe destacar que en caso de querer emplear las precipitaciones 
#por dias podrían obtenerse sumando las horarias, de todas formas, puede ser mejor emplar las variables meteorologicas del resumen
#Pues aquí salen repetidos varias veces a lo largo del día!!!
var_name = 'PrecipAmountLocalDayAvg'
which(names(datos_precipitaciones_tr_2022) == var_name)

#### PRECIPITACIONES TEMPORADA 21/22 ###
Precipitaciones_hora_2022 <- subset(datos_precipitaciones_tr_2022,ID_ESTACION==0)$PrecipAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2022 <- cbind(Precipitaciones_hora_2022, subset(datos_precipitaciones_tr_2022, ID_ESTACION==i)$PrecipAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2022)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2022.fd = smooth.basis(y=Precipitaciones_hora_2022, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2022.fd,main='Cruvas de precipitaciones para cada estación 21/22',xlab='Tiempo',ylab='Precipitaciones')



#### PRECIPITACIONES TEMPORADA 20/21 ###
Precipitaciones_hora_2021 <- subset(datos_precipitaciones_tr_2021,ID_ESTACION==0)$PrecipAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2021 <- cbind(Precipitaciones_hora_2021, subset(datos_precipitaciones_tr_2021, ID_ESTACION==i)$PrecipAmountLocalDayAvgHour)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2021)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2021.fd = smooth.basis(y=Precipitaciones_hora_2021, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2021.fd,main='Cruvas de precipitaciones para cada estación 20/21',xlab='Tiempo',ylab='Precipitaciones')




#### PRECIPITACIONES TEMPORADA 19/20 ###
Precipitaciones_hora_2020 <- subset(datos_precipitaciones_tr_2020,ID_ESTACION==0)$PrecipAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2020 <- cbind(Precipitaciones_hora_2020, subset(datos_precipitaciones_tr_2020, ID_ESTACION==i)$PrecipAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2020)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2020.fd = smooth.basis(y=Precipitaciones_hora_2020, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2020.fd,main='Cruvas de precipitaciones para cada estación 19/20',xlab='Tiempo',ylab='Precipitaciones')



#### PRECIPITACIONES TEMPORADA 18/19 ###
Precipitaciones_hora_2019 <- subset(datos_precipitaciones_tr_2019,ID_ESTACION==0)$PrecipAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2019 <- cbind(Precipitaciones_hora_2019, subset(datos_precipitaciones_tr_2019, ID_ESTACION==i)$PrecipAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2019)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2019.fd = smooth.basis(y=Precipitaciones_hora_2019, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2019.fd,main='Cruvas de precipitaciones para cada estación 18/19',xlab='Tiempo',ylab='Precipitaciones')



#### PRECIPITACIONES TEMPORADA 17/18 ###
Precipitaciones_hora_2018 <- subset(datos_precipitaciones_tr_2018,ID_ESTACION==0)$PrecipAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2018 <- cbind(Precipitaciones_hora_2018, subset(datos_precipitaciones_tr_2018, ID_ESTACION==i)$PrecipAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2018)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2018.fd = smooth.basis(y=Precipitaciones_hora_2018, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2018.fd,main='Cruvas de precipitaciones para cada estación 17/18',xlab='Tiempo',ylab='Precipitaciones')





#### PRECIPITACIONES TEMPORADA 16/17 ###
Precipitaciones_hora_2017 <- subset(datos_precipitaciones_tr_2017,ID_ESTACION==0)$PrecipAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2017 <- cbind(Precipitaciones_hora_2017, subset(datos_precipitaciones_tr_2017, ID_ESTACION==i)$PrecipAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2017)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2017.fd = smooth.basis(y=Precipitaciones_hora_2017, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2017.fd,main='Cruvas de precipitaciones para cada estación 16/17',xlab='Tiempo',ylab='Precipitaciones')



#### PRECIPITACIONES TEMPORADA 15/16 ###
Precipitaciones_hora_2016 <- subset(datos_precipitaciones_tr_2016,ID_ESTACION==0)$PrecipAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Precipitaciones_hora_2016 <- cbind(Precipitaciones_hora_2016, subset(datos_precipitaciones_tr_2016, ID_ESTACION==i)$PrecipAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Precipitaciones_hora_2016)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Precipitaciones_hora_2016.fd = smooth.basis(y=Precipitaciones_hora_2016, fdParobj=bspline.basis)

plot(Precipitaciones_hora_2016.fd,main='Cruvas de precipitaciones para cada estación 15/16',xlab='Tiempo',ylab='Precipitaciones')




# Así ploteados aparecen valores negativos, pero esto se debe al smoothing Kernel



# NIEVE POR AÑOS:

#Asignaremos las temperaturas que se pueden emplear en cada temporada:
datos_nieves_tr_2022 = clima_tr_2022[,c(1,170:189,275)]
datos_nieves_tr_2021 = clima_tr_2021[,c(1,170:189,275)]
datos_nieves_tr_2020 = clima_tr_2020[,c(1,170:189,275)]
datos_nieves_tr_2019 = clima_tr_2019[,c(1,170:189,275)]
datos_nieves_tr_2018 = clima_tr_2018[,c(1,170:189,275)]
datos_nieves_tr_2017 = clima_tr_2017[,c(1,170:189,275)]
datos_nieves_tr_2016 = clima_tr_2016[,c(1,170:189,275)]
datos_nieves_tr_2015 = clima_tr_2015[,c(1,170:189,275)]
#Ahora los separaremos por estaciones. Cabe destacar que en caso de querer emplear las precipitaciones 


#### NIEVES TEMPORADA 21/22 ###
Nieves_hora_2022 <- subset(datos_nieves_tr_2022,ID_ESTACION==0)$SnowAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2022 <- cbind(Nieves_hora_2022, subset(datos_nieves_tr_2022, ID_ESTACION==i)$SnowAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2022)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2022.fd = smooth.basis(y=Nieves_hora_2022, fdParobj=bspline.basis)

plot(Nieves_hora_2022.fd,main='Cruvas de nieve para cada estación 21/22',xlab='Tiempo',ylab='Nieve')



#### NIEVES TEMPORADA 20/21 ###
Nieves_hora_2021 <- subset(datos_nieves_tr_2021,ID_ESTACION==0)$SnowAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2021 <- cbind(Nieves_hora_2021, subset(datos_nieves_tr_2021, ID_ESTACION==i)$SnowAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2021)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2021.fd = smooth.basis(y=Nieves_hora_2021, fdParobj=bspline.basis)

plot(Nieves_hora_2021.fd,main='Cruvas de nieve para cada estación 20/21',xlab='Tiempo',ylab='Nieve')



#### NIEVES TEMPORADA 19/20 ###
Nieves_hora_2020 <- subset(datos_nieves_tr_2020,ID_ESTACION==0)$SnowAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2020 <- cbind(Nieves_hora_2020, subset(datos_nieves_tr_2020, ID_ESTACION==i)$SnowAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2020)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2020.fd = smooth.basis(y=Nieves_hora_2020, fdParobj=bspline.basis)

plot(Nieves_hora_2020.fd,main='Cruvas de nieve para cada estación 19/20',xlab='Tiempo',ylab='Nieve')




#### NIEVES TEMPORADA 18/19 ###
Nieves_hora_2019 <- subset(datos_nieves_tr_2019,ID_ESTACION==0)$SnowAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2019 <- cbind(Nieves_hora_2019, subset(datos_nieves_tr_2019, ID_ESTACION==i)$SnowAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2019)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2019.fd = smooth.basis(y=Nieves_hora_2019, fdParobj=bspline.basis)

plot(Nieves_hora_2019.fd,main='Cruvas de nieve para cada estación 18/19',xlab='Tiempo',ylab='Nieve')


#No nevo en 2019

#### NIEVES TEMPORADA 17/18 ###
Nieves_hora_2018 <- subset(datos_nieves_tr_2018,ID_ESTACION==0)$SnowAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2018 <- cbind(Nieves_hora_2018, subset(datos_nieves_tr_2018, ID_ESTACION==i)$SnowAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2018)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2018.fd = smooth.basis(y=Nieves_hora_2018, fdParobj=bspline.basis)

plot(Nieves_hora_2018.fd,main='Cruvas de nieve para cada estación 17/18',xlab='Tiempo',ylab='Nieve')




#### NIEVES TEMPORADA 16/17 ###
Nieves_hora_2017 <- subset(datos_nieves_tr_2017,ID_ESTACION==0)$SnowAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2017 <- cbind(Nieves_hora_2017, subset(datos_nieves_tr_2017, ID_ESTACION==i)$SnowAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2017)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2017.fd = smooth.basis(y=Nieves_hora_2017, fdParobj=bspline.basis)

plot(Nieves_hora_2017.fd,main='Cruvas de nieve para cada estación 16/17',xlab='Tiempo',ylab='Nieve')




#### NIEVES TEMPORADA 15/16 ###
Nieves_hora_2016 <- subset(datos_nieves_tr_2016,ID_ESTACION==0)$SnowAmountLocalDayAvg #Es la temperatura por hora
for (i in 1:19){
  Nieves_hora_2016 <- cbind(Nieves_hora_2016, subset(datos_nieves_tr_2016, ID_ESTACION==i)$SnowAmountLocalDayAvg)
}

# Creamos la base de splines a partir de la cual se va a construir la curva 
bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Nieves_hora_2016)), nbasis=15)

# Convertimos las temperaturas en curvas (funcionales)
Nieves_hora_2016.fd = smooth.basis(y=Nieves_hora_2016, fdParobj=bspline.basis)

plot(Nieves_hora_2016.fd,main='Cruvas de nieve para cada estación 15/16',xlab='Tiempo',ylab='Nieve')





#Por si interesa hacer más de estos creamos la siguiente función que facilita el trabajo

plot_variable_funcional = function(data, var_name, nbasis = 10, year){
  years_plot = 14:22
  ind = which(names(data) == var_name)
  Datos_year <- subset(data, ID_ESTACION==0)[,ind] #Es la variable
  for (i in 1:19){
    Datos_year <- cbind(Datos_year,subset(data, ID_ESTACION==i)[,ind])
  }
  
  # Creamos la base de splines a partir de la cual se va a construir la curva 
  bspline.basis = create.bspline.basis(rangeval=c(0,nrow(Datos_year)), nbasis=nbasis)
  
  # Convertimos las temperaturas en curvas (funcionales)
  Datos_year.fd = smooth.basis(y=Datos_year, fdParobj=bspline.basis)
  
  plot(Datos_year.fd, main = paste('Curvas de', var_name, 'para cada estación', paste0(as.character(years_plot[year]),'/',as.character(years_plot[year+ 1]))),xlab='Tiempo',ylab=var_name)    
}

variable = 'RelativeHumidityLocalDayAvg' #UVIndexLocalDayAvg, FeelsLikeLocalDayAvg, FeelsLikeLocalDayAvg, WindSpeedLocalDayAvg
nb = 4
plot_variable_funcional(data = clima_tr_2017 , var_name = variable, nbasis = nb, year = 3)
plot_variable_funcional(data = clima_tr_2018 , var_name = variable, nbasis = nb, year = 4)
plot_variable_funcional(data = clima_tr_2019 , var_name = variable, nbasis = nb, year = 5)
plot_variable_funcional(data = clima_tr_2020 , var_name = variable, nbasis = nb, year = 6)
plot_variable_funcional(data = clima_tr_2021 , var_name = variable, nbasis = nb, year = 7)
plot_variable_funcional(data = clima_tr_2022 , var_name = variable, nbasis = nb, year = 8)
plot_variable_funcional(data = clima_tr_2016 , var_name = variable, nbasis = nb, year = 2)








###############################################################################################
###############################################################################################
    #### AQUÍ ACABA EL EXPLORATORIO Y PONEMOS EL CÓDIGO RELEVANTE PARA LA MEMORIA #####
###############################################################################################
###############################################################################################





#Tendremos que cargar parte del script final, en el que se transforman los datos, de esta forma
#se podrán ejecutar algunos de los modelos mencionados en la memoria.


#Borraremos todo lo creado hasta ahora para evitar problemas:
rm(list = ls())



library(ggplot2)
library(dplyr) #Es importante tener instalada la última versión de dplyr
library(hrbrthemes)
library(lubridate)
library(zoo)
library(caret)
library(tidyr)
library(ParBayesianOptimization)
library(missForest)
library(mice)
library(doParallel)
library(factoextra)
library(xgboost) #Es importante tener instalada la última versión de xgboost

setwd("C:/Users/usuario/Documents/AA COMPETICION VINOS")

datosMETEO<- read.table("DATOS_METEO.TXT", sep = "|", header = TRUE,encoding = "UTF-8")
#str(datosMETEO)

datosETO<- read.table("DATOS_ETO.TXT", sep = "|", header = TRUE,encoding = "UTF-8")
#str(datosETO)

Goidanich<- read.csv("Goidanich.csv", sep = ";", header = TRUE, encoding = "UTF-8")
#str(Goidanich)

datosUH<- read.table("UH_2023_TRAIN.txt", sep = "|", header = TRUE,encoding = "UTF-8")



###############################################################################
###############################################################################

# TRANSFORMACIÓN Y PREPROCESADO DE LOS DATOS

###############################################################################
###############################################################################


#En el ficheroGoidanich.csv se encuaentra la tabla propuesta por Goidanich para la evolución
#del riesgo de contagio de mildeo según factores climáticos. La tabla está discretizada y si bien se puede actualizar
#el indicador por los valores de temperatura más cercanos de la tabla, aquí se ha optado
#por ajustar una regresión no paramétrica que sirviese como método de interpolación,
#ya que era una forma rápida y sencilla de interpolar los datos

plot(x=Goidanich$T.media,y=Goidanich$Desar.Hm.mn.75)
Goid.mn.loess <- loess(Desar.Hm.mn.75~T.media, span=0.3,data=Goidanich)
lines(Goid.mn.loess,lwd=2,col="magenta")

Goid.mn.loess.pred<-function(x){
  print(x)
  if (x>=12){
    p.eval <- data.frame(T.media=c(x))
    predG<-as.numeric(predict(Goid.mn.loess, p.eval))}
  else{
    predG<-0
  }
  return(predG)
}

plot(x=Goidanich$T.media,y=Goidanich$Desar.Hm.my.75)
Goid.my.loess <- loess(Desar.Hm.my.75~T.media, span=0.3,data=Goidanich)
lines(Goid.my.loess,lwd=2,col="magenta")

Goid.my.loess.pred<-function(x){
  if (x>=12){
    p.eval <- data.frame(T.media=c(x))
    predG<-as.numeric(predict(Goid.my.loess, p.eval))}
  else{
    predG<-0
  }
  return(predG)
}


#Modificación de los datos



#Vector de producciones totales por campaña
sumprod.A<-function(x){sum(datosUH$PRODUCCION[datosUH$CAMPAÑA==x])}
x<-14:21
ProdTotal.C<-data.frame(CAMPAÑA=x,PROD_total=sapply(x,FUN="sumprod.A"))


#Creamos la media de las temperaturas de los ultimos 5 dias, 
#y la precipitacion media de los últimos 2 dias

datosETO.red<-datosETO %>% 
  mutate(TempLDA=TemperatureLocalDayAvg-273.15,
         PrecipLDA=PrecipAmountLocalDayAvg,
         HmLDA=RelativeHumidityLocalDayAvg)%>%
  #dplyr::select(date,ID_ESTACION,TempLDA,PrecipLDA,HmLDA) %>%
  mutate(dateOK=as.Date(as.character(date), format = "%Y%m%d"))  #%>% dplyr::select(dateOK,ID_ESTACION,TempLDA,PrecipLDA,HmLDA)


datosETO.tr <- datosETO.red %>%
  group_by(ID_ESTACION) %>%
  arrange(dateOK,.by_group=TRUE) %>%
  mutate(TempLDA_5d = rollapply(TempLDA, width = 5, FUN = mean, align = "right", fill = NA),
         PrecipLDA_2d=rollapply(PrecipLDA, width = 2, FUN = mean, align = "right", fill = NA),
         Esporas=0) %>% 
  ungroup() %>% 
  mutate(PLDAc=PrecipLDA*24)%>%
  #dplyr::select(dateOK,ID_ESTACION,TempLDA_5d,PrecipLDA_2d,Esporas,HmLDA,TempLDA,PLDAc) %>%
  group_by(ID_ESTACION) %>% 
  mutate(Goid.pt=ifelse(HmLDA>=75,sapply(TempLDA,FUN="Goid.my.loess.pred"),sapply(TempLDA,FUN="Goid.my.loess.pred")))%>%
  mutate(Goid.Acum=0)%>%
  ungroup() #dplyr::select(dateOK,ID_ESTACION,TempLDA_5d,PrecipLDA_2d,Esporas,HmLDA,TempLDA,PLDAc,Goid.pt,Goid.Acum) 




#-- Parte 2: Creación de los índices de forma recursiva 

df_grupo_estacion <- datosETO.tr %>%
  group_by(ID_ESTACION) %>%
  arrange(dateOK,.by_group=TRUE)

#Se agrupan los datos por estación y ordenados por fecha en cada grupo

df_grupo_estacion <- df_grupo_estacion %>%
  mutate(
    temp5 = ifelse(is.na(TempLDA_5d), 0, TempLDA_5d),
    prep2 = ifelse(is.na(PrecipLDA_2d), 0, PrecipLDA_2d),
    prep.ant = lag(PLDAc),
    temp.ant = lag(TempLDA))

#Solo se mete en el bucle lo imprescindible (ya que es poco eficiente).
#No se puede paralelizar ya que el valor i-esimo depende del que se va a calcular en i-1, por lo tanto han de llevarse a 
#cabo de forma secuencial.
for(i in 5:nrow(df_grupo_estacion)){ #Se empieza en el 5 pq el rollaply dejo 5 NAs
  esp.ant = df_grupo_estacion$Esporas[i-1]
  df_grupo_estacion$Esporas[i] = ifelse(-0.032*(df_grupo_estacion$temp5[i]) + 0.054*48*df_grupo_estacion$prep2[i] + 0.913*esp.ant>=0,
                                        -0.032*(df_grupo_estacion$temp5[i]) + 0.054*48*df_grupo_estacion$prep2[i] + 0.913*esp.ant,0)   
  pt.ant <- df_grupo_estacion$Goid.Acum[i-1]
  if (pt.ant>=100){
    df_grupo_estacion$Goid.Acum[i]<-0
  }
  else if (pt.ant== 0 & df_grupo_estacion$TempLDA[i]>=12 & df_grupo_estacion$TempLDA[i]<=35 & 
           df_grupo_estacion$TempLDA[i-1]>=12 & df_grupo_estacion$TempLDA[i-1]<=35) {
    df_grupo_estacion$Goid.Acum[i] <- df_grupo_estacion$Goid.pt[i]} 
  else if (pt.ant> 0 & pt.ant<99 & 
           df_grupo_estacion$TempLDA[i]>=12 & df_grupo_estacion$TempLDA[i]<=35) {
    df_grupo_estacion$Goid.Acum[i] <- df_grupo_estacion$Goid.pt[i] +pt.ant
  } 
  else {
    df_grupo_estacion$Goid.Acum[i]<-0
  }
}



#Explicación bucle anterior:

#El código primero inicializa un bucle que comienza desde la quinta fila del data frame 
#y luego realiza algunas operaciones en cada fila posterior.
#Para cada fila, el código calcula un nuevo valor para la columna "Esporas" del data frame 
#usando una ecuación lineal basada en los valores de "TempLDA_5d", "PrecipLDA_2d" y el valor 
#anterior de "Esporas". El nuevo valor se establece en el resultado de la ecuación o en 0, dependiendo 
#de si el resultado es mayor o igual a 0.

#El código también calcula un nuevo valor para la columna "Goid.Acum" basado en los valores 
#de "TempLDA", "Goid.pt" y los valores anteriores de "Goid.Acum", "TempLDA" y "PLDAc". El nuevo valor 
#se determina mediante un conjunto de declaraciones if-else basadas en los valores de estas variables.

#El código actualiza las columnas "Esporas" y "Goid.Acum" del data frame para cada fila del bucle.




df_grupo_estacion2 = df_grupo_estacion %>% ungroup() %>%
  mutate(Day=day(ymd(dateOK)),
         Month=month(ymd(dateOK)),
         Year=year(ymd(dateOK)),
         Temporada=dplyr::case_when(
           Month %in% c(10,11,12,1,2) ~ "Parada",
           Month== 3 ~ "Marzo",
           Month==4 | (Month==5 & Day<=15) ~ "Foliacion",
           Month==6 | (Month==5 & Day>15) ~"Floracion")) %>% 
  dplyr::filter(Month %in% c(1,2,3,4,5,6,10,11,12)) %>%
  mutate(CAMPÑ=dplyr::case_when(
    dateOK>="2013-10-01" & dateOK<="2014-06-30" ~ 14,
    dateOK>="2014-10-01" & dateOK<="2015-06-30" ~ 15,
    dateOK>="2015-10-01" & dateOK<="2016-06-30" ~ 16,
    dateOK>="2016-10-01" & dateOK<="2017-06-30" ~ 17,
    dateOK>="2017-10-01" & dateOK<="2018-06-30" ~ 18,
    dateOK>="2018-10-01" & dateOK<="2019-06-30" ~ 19,
    dateOK>="2019-10-01" & dateOK<="2020-06-30" ~ 20,
    dateOK>="2020-10-01" & dateOK<="2021-06-30" ~ 21,
    dateOK>="2021-10-01" & dateOK<="2022-06-30" ~ 22))
#Asignamos las campañas a las fechas correspondientes y creamos los diferentes periodos de la vid dentro cada campaña.


# Agrupar los datos por ID_ESTACION, YEAR y MONTH: ¿porque se hacen los maximos y minimos de las medias???????
datos_agrupados <- df_grupo_estacion2 %>% group_by(ID_ESTACION, CAMPÑ, Temporada)%>% 
  summarise(TEMPDayAvg = mean(TemperatureLocalDayAvg,na.rm=TRUE)-273.15,
            TEMPDaytimeAvg = mean(TemperatureLocalDaytimeAvg,na.rm=TRUE)-273.15,
            TEMPNighttimeAvg = mean(TemperatureLocalNighttimeAvg,na.rm=TRUE)-273.15,
            TEMPDayMax = max(TemperatureLocalDayMax,na.rm=TRUE)-273.15,
            TEMPDaytimeMax = max(TemperatureLocalDaytimeMax,na.rm=TRUE)-273.15,
            TEMPNighttimeMax = max(TemperatureLocalNighttimeMax,na.rm=TRUE)-273.15,
            TEMPDayMin = min(TemperatureLocalDayMin,na.rm=TRUE)-273.15,
            TEMPDaytimeMin = min(TemperatureLocalDaytimeMin,na.rm=TRUE)-273.15,
            TEMPNighttimeMin = min(TemperatureLocalNighttimeMin,na.rm=TRUE)-273.15,
            
            WindDayAvg = mean(WindSpeedLocalDayAvg,na.rm=TRUE),
            WindDaytimeAvg = mean(WindSpeedLocalDaytimeAvg,na.rm=TRUE),
            WindNighttimeAvg = mean(WindSpeedLocalNighttimeAvg,na.rm=TRUE),
            WindDayMax = max(WindSpeedLocalDayMax,na.rm=TRUE),
            WindDaytimeMax = max(WindSpeedLocalDaytimeMax,na.rm=TRUE),
            WindNighttimeMax = max(WindSpeedLocalNighttimeMax,na.rm=TRUE),
            WindDayMin = min(WindSpeedLocalDayMin,na.rm=TRUE),
            WindDaytimeMin = min(WindSpeedLocalDaytimeMin,na.rm=TRUE),
            WindNighttimeMin = min(WindSpeedLocalNighttimeMin,na.rm=TRUE),
            
            RadDayAvg = mean(UVIndexLocalDayAvg,na.rm=TRUE),
            RadDaytimeAvg = mean(UVIndexLocalDaytimeAvg,na.rm=TRUE),
            RadNighttimeAvg = mean(UVIndexLocalNighttimeAvg,na.rm=TRUE),
            RadDayMax = max(UVIndexLocalDayMax,na.rm=TRUE),
            RadDaytimeMax = max(UVIndexLocalDaytimeMax,na.rm=TRUE),
            RadNighttimeMax = max(UVIndexLocalNighttimeMax,na.rm=TRUE),
            RadDayMin = min(UVIndexLocalDayMin,na.rm=TRUE),
            RadDaytimeMin = min(UVIndexLocalDaytimeMin,na.rm=TRUE),
            RadNighttimeMin = min(UVIndexLocalNighttimeMin,na.rm=TRUE),
            
            PreDayAvg = mean(PrecipAmountLocalDayAvg,na.rm=TRUE),
            PreDaytimeAvg = mean(PrecipAmountLocalDaytimeAvg,na.rm=TRUE),
            PreNighttimeAvg = mean(PrecipAmountLocalNighttimeAvg,na.rm=TRUE),
            PreDayMax = max(PrecipAmountLocalDayMax,na.rm=TRUE),
            PreDaytimeMax = max(PrecipAmountLocalDaytimeMax,na.rm=TRUE),
            PreNighttimeMax = max(PrecipAmountLocalNighttimeMax,na.rm=TRUE),
            PreDayMin = min(PrecipAmountLocalDayMin,na.rm=TRUE),
            PreDaytimeMin = min(PrecipAmountLocalDaytimeMin,na.rm=TRUE),
            PreNighttimeMin = min(PrecipAmountLocalNighttimeMin,na.rm=TRUE),
            
            HumDayAvg = mean(RelativeHumidityLocalDayAvg,na.rm=TRUE),
            HumDaytimeAvg = mean(RelativeHumidityLocalDaytimeAvg,na.rm=TRUE),
            HumNighttimeAvg = mean(RelativeHumidityLocalNighttimeAvg,na.rm=TRUE),
            HumDayMax = max(RelativeHumidityLocalDayMax,na.rm=TRUE),
            HumDaytimeMax = max(RelativeHumidityLocalDaytimeMax,na.rm=TRUE),
            HumNighttimeMax = max(RelativeHumidityLocalNighttimeMax,na.rm=TRUE),
            HumDayMin = min(RelativeHumidityLocalDayMin,na.rm=TRUE),
            HumDaytimeMin = min(RelativeHumidityLocalDaytimeMin,na.rm=TRUE),
            HumNighttimeMin = min(RelativeHumidityLocalNighttimeMin,na.rm=TRUE),
            Goid=sum(Goid.Acum>70),
            Espo=sum(Esporas))

#Se han calculado las medias maximos y minimos de las distintas variables que se consideraron de interés 
# En cada una de las etapas del desarrollo de la vid


datos_wide <- datos_agrupados %>% pivot_wider(names_from = Temporada, values_from = -c(ID_ESTACION,CAMPÑ,Temporada)) 
str(datos_wide)

aux<-datos_wide 



#Añade a datos_wide la suma de las variables Goid y Espo a lo largo de las fases de desarollo de la vid
datos_wide2<- aux %>% mutate(Goid=sum(Goid_Foliacion,Goid_Floracion,na.rm=TRUE),
                             Espo=sum(Espo_Parada,Espo_Marzo,Espo_Foliacion,Espo_Floracion,na.rm = TRUE)) %>% dplyr::select(-c(
                               Espo_Parada,Espo_Marzo,Espo_Foliacion,Espo_Floracion,
                               Goid_Parada,Goid_Marzo,Goid_Foliacion,Goid_Floracion))



MAIN.tr = datosUH %>% mutate(CAMPÑ=CAMPAÑA) %>% group_by(CAMPÑ, ID_ESTACION) %>% summarise(PRODUCCION=mean(PRODUCCION,na.rm=TRUE))%>% 
  left_join(datos_wide2,by=c("CAMPÑ","ID_ESTACION")) %>% arrange(ID_ESTACION,CAMPÑ)

#MAIN.tr da la media de la produccion por estaciones



#A los datos con la información acerca de los cultivos en cada finca se les hace un leftjoin con los datos_wide2 que contienen los datos meteorológicos resumidos de interés

MAIN.tr2<- MAIN.tr %>% mutate(PROD_1 = 0,PROD_1na = 0) #A este a mayores se le introducen las variables que hacen referencia a la producción del año anterior

for (i in unique(MAIN.tr2$ID_ESTACION)){
  for (j in MAIN.tr2$CAMPÑ[MAIN.tr2$ID_ESTACION==i]){
    if ((j-1) %in% MAIN.tr2$CAMPÑ[MAIN.tr2$ID_ESTACION==i]){
      MAIN.tr2[MAIN.tr2$ID_ESTACION==i & MAIN.tr2$CAMPÑ==j,"PROD_1"]<- MAIN.tr2[MAIN.tr2$ID_ESTACION==i & MAIN.tr2$CAMPÑ==j-1,"PRODUCCION"]
    }
    else{
      MAIN.tr2[MAIN.tr2$ID_ESTACION==i & MAIN.tr2$CAMPÑ==j,"PROD_1na"]<-1
    }
  }
}


#El código crea una nueva variable llamada PROD_1 en el data frame MAIN.tr2 y la inicializa en cero. Luego, crea otra variable llamada PROD_1na y también la inicializa en cero.
#Ambas variables se usarán más adelante para hacer referencia a la producción del año anterior.

#El código utiliza un bucle for para recorrer cada estación de manera única y otro bucle for para recorrer cada año (CAMPÑ) dentro de cada estación.
#Dentro del segundo bucle for, el código verifica si el año anterior (j-1) está presente en los datos de producción de la misma estación (ID_ESTACION) y si es así,
#se asigna el valor de producción correspondiente a la variable PROD_1 en la fila correspondiente en el data frame. Si el año anterior no está presente en los datos, 
#se asigna el valor 1 a la variable PROD_1na en la fila correspondiente en el data frame. Al final de la ejecución del código, el data frame MAIN.tr2 contendrá las variables 
#PROD_1 y PROD_1na que hacen referencia a la producción del año anterior para cada fila correspondiente a una estación y un año determinado.

#Cambiamos los nombres de las variables de producción añadiéndoles un "w" para indicar que se trata en realidad de producciones agregadas por estaciones meteorológicas.
MAIN.tr3<-MAIN.tr2%>%rename(wPRODUCCION=PRODUCCION,wPROD_1=PROD_1,wPROD_1na=PROD_1na)%>%mutate(ID_ESTACION=as.factor(ID_ESTACION))

#-------------------------------------------------

datos.At <- datosUH %>%
  mutate(aux = ifelse(is.na(ALTITUD), "999", ALTITUD)) %>%
  mutate(contiene_guion = grepl("-", aux, fixed = TRUE)) %>%
  mutate(ALT_med = ifelse(contiene_guion,
                          (as.numeric(substr(aux, start = 5, stop = nchar(aux)))+
                             as.numeric(substr(aux, start = 1, stop = 3)))/2
                          ,as.numeric(aux)))%>% 
  mutate(ALT_med2 = ifelse(is.na(ALTITUD), NA, ALT_med))

#datos.At se emplea para calcular la variable de altura media como numeric. Además se crea la variable ALT_med2 en la que
# los NAs aparecen como tal, al contrario que en aux en la que aparecen como 999 para que se puedan llevar a cabo las operaciones posteriores

datos = datosUH

#En datos2 pasamos todo a factor:
datos2 = datosUH %>% mutate(ALTITUD = datos.At$ALT_med2) %>% mutate(TIPO = as.factor(TIPO)) %>% mutate(MODO = as.factor(MODO)) %>%
  mutate(COLOR = as.factor(COLOR)) %>% mutate(ID_FINCA = as.factor(ID_FINCA))%>% mutate(ID_ESTACION = as.factor(ID_ESTACION)) %>%
  mutate(ID_ZONA = as.factor(ID_ZONA)) %>% mutate(VARIEDAD = as.factor(VARIEDAD)) 
#Finalmente pasamos todo a factor




#FINALMENTE AÑADIMOS EL ID_UNICO Y LAS PRODUCCIONES DE AÑOS ANTERIORES JUNTO A SUS INDICADORES DE NAs

PROD.ANT = function(v, col.CAMPÑ = 12, diff = 0, total.data = ProdTotal.C) { #col.CAMPÑ es el numero de la columna CAMPÑ
  #diff = el diferencial para que las columnas se correspondan con los años adecuados
  #si se introducen c columnas diff = c
  
  PROD_1 = ifelse(is.na(v[as.numeric(v[col.CAMPÑ]) + 1 + diff]), 0, as.numeric(v[as.numeric(v[col.CAMPÑ]) + 1 + diff]))
  PROD_2 = ifelse(is.na(v[as.numeric(v[col.CAMPÑ]) + diff]), 0, as.numeric(v[as.numeric(v[col.CAMPÑ]) + diff]))
  PROD_mean = mean(as.numeric(v[(14 + diff):(as.numeric(v[col.CAMPÑ]) + 1 + diff)]), na.rm = T)
  return(c(PROD_1, PROD_2, PROD_mean))
}


final.fun = function(x, col.CAMPÑ = 12, diff = 0, total.data = ProdTotal.C){ #col.CAMPÑ es el numero de la columna CAMPÑ
  #diff = el diferencial para que las columnas se correspondan con los años adecuados
  #si se introducen c columnas diff = c
  #Nuevas variables:
  x = x %>% mutate(CAMPÑ=CAMPAÑA,ID_UNICO=paste(ID_FINCA,VARIEDAD,TIPO,MODO,sep="-"))
  
  #Dataset wide:
  x_wide_aux = x %>% pivot_wider(id_cols = ID_UNICO, names_from = CAMPÑ, values_from = PRODUCCION) %>% 
    mutate('12' = NA, '13' = NA) %>% relocate(after = c(1, 11, 12, 2:10))
  colnames(x_wide_aux) = c('ID_UNICO', as.character(12:22))
  
  #Juntamos el dataset normal con el wide
  x_aux = x %>% left_join(x = x, y = x_wide_aux, by = 'ID_UNICO', relationship = 'many-to-one')
  
  #Creamos las variables de PROD_1, PROD_2 y PROD_mean:
  result = t(apply(x_aux, 1 , PROD.ANT, col.CAMPÑ = col.CAMPÑ, diff = diff, total.data = total.data))
  x_aux = cbind(x_aux, PROD_1 = result[,1], PROD_2 = result[,2], PROD_mean = result[,3])
  
  #Creamos las indicadoras de NAs y modificamos mean:
  x_aux$PROD_1na = ifelse(x_aux$PROD_1 == 0, 1, 0)
  x_aux$PROD_2na = ifelse(x_aux$PROD_2 == 0, 1, 0)
  x_aux$PROD_MEAN_NA = ifelse(is.na(x_aux$PROD_mean), 1, 0)
  x_aux$PROD_mean = ifelse(is.na(x_aux$PROD_mean), 0, x_aux$PROD_mean)
  
  #Eliminamos las columnas de wide:
  x_aux = x_aux[, !names(x_aux) %in% as.character(12:22)]
  return(x_aux)
}

datos_finales = final.fun(datos2, col.CAMPÑ = 12, diff = 0, total.data = ProdTotal.C)

datos_finales = datos_finales %>% mutate(PROD_MEAN_NA = as.factor(PROD_MEAN_NA), PROD_2na = as.factor(PROD_2na), PROD_1na = as.factor(PROD_1na))

#En resumen, este código indica la producción de los años anteriores en las columnas correspondientes y pone un valor de "1" en las 
#columnas indicadoras si no hay valores disponibles para los años anteriores. Además, se calcula la proporción dentro del mismo año de dicha produccion

#También se crea una variable que sea la media de la producción de la finca a lo largo de los años (al menos de los años en los cuales produce)
#Y en las fincas para las que no hay valores previos de la predicción, se pone una variable indicadora!!! (De esta forma reducimos las fincas cuyo output depende de la indicadora)








########################################################################################################
######################## IMPUTACIÓN DE VALORES FALTANTES ###############################################
########################################################################################################

#Un primer método de imputación será por randomforest. Este lo llevaremos a cabo separando los años 14-21 y el año 22 ( ya que en este último
#no se pueden emplear las producciones para imputar, pq aun no las tenemos).
#En primer lugar se imputarán las altitudes de las fincas ya que algunas de las zonas tienen todas las fincas con la altitud faltante.
#En segundo lugar, y empleando también los valores imputados de las altitudes, generamos valores para las fincas de superficie 0.
#Este método no es el que produce la mejor imputación (no introduce variabilidad en los datos), no obstante, menos costoso computacionalmente
#que la imputación del paquete MICE (que al generar varios datasets y combinar las respuestas, no sería compatible con aplicar el modelo xgboost planteado
#Pues tardaría demasiado en ejecutarse)

#Posteriormente, llevaremos a cabo una imputacion con el paquete MICE de la misma forma y usaremos los datos para
#ajustar un modelo random forest, de forma que el tiempo extra que consuma la imputación lo ahorremos en la creacion del modelo
#generando así un algoritmo igualmente eficiente

#Asignamos NAs a los valores de superficie = 0 de las campañas que hemos mencionado:
x = which( (datos_finales$CAMPAÑA == 20 & datos_finales$SUPERFICIE == 0) | (datos_finales$CAMPAÑA == 21 & datos_finales$SUPERFICIE == 0) | (datos_finales$CAMPAÑA == 22 & datos_finales$SUPERFICIE == 0) )
n = length(x)
for(i in 1:n){datos_finales[x[i],10] = NA}


#################################### PAQUETE MISS FOREST ###############################################

##----------------------------------------- ALTITUD -----------------------------------------##

#### AÑOS 2014 - 2021:

datos_finales_alt_14_21 = datos_finales[which(datos_finales$CAMPAÑA != 22),]

#En primer lugar imputamos los valores de las alturas para estos años, AQUI PODEMOS EMPLEAR LA PRODUCCION:

#Para computar en paralelo (más eficiente):
numCores = detectCores()
if(numCores > 13){numCores = 13} # The number of parallel cores should not exceed the number of variables (p=13)
registerDoParallel(cores = numCores)
set.seed(1234)
datos_imput_alt_14_21 = missForest(datos_finales_alt_14_21[,-c(1,2,3,10,12,13)], maxiter = 1000, ntree = 1000, parallelize = 'forests')$ximp
stopImplicitCluster()

#Ahora agregaremos los valores imputados al dataset:
x1 = as.numeric(rownames(datos_imput_alt_14_21[is.na(datos_finales_alt_14_21$ALTITUD),]))

#Asignación
datos_finales_alt_14_21[x1,5] = as.numeric(datos_imput_alt_14_21$ALTITUD[x1])

#Finalmente comprobaremos que las alturas de las fincas del mismo ID_ZONA, son similares, ya que en general
#si tienen el mismo ID_zona tienen la misma altitud!!
ZONAS = unique(as.numeric(as.character(datos_finales_alt_14_21[x1,3])))

# Calcular la media de la ALTITUD para cada ID_ZONA en el vector ZONAS
altitud_media = datos_finales_alt_14_21 %>%
  filter(ID_ZONA %in% ZONAS) %>%
  group_by(ID_ZONA) %>%
  summarise(media_altitud = mean(ALTITUD, na.rm = TRUE))




#### AÑO 2022:


#Añadimos los valores de 2022
datos_finales_altitud = rbind(datos_finales_alt_14_21, datos_finales[(nrow(datos_finales_alt_14_21)+1):nrow(datos_finales),])

#Imputamos los valores de las alturas para este año y a partir de los valores imputados en años anteriores:
#En este caso no empleamos la produccion porque no la tenemos
#Para computar en paralelo (más eficiente):
numCores = detectCores()
if(numCores > 12){numCores = 12} # The number of parallel cores should not exceed the number of variables (p=12)
registerDoParallel(cores = numCores)
set.seed(1234)
datos_imput_alt_14_22 = missForest(datos_finales_altitud[,-c(1,2,3,10,11,12,13)], maxiter = 1000, ntree = 1000, parallelize = 'forests')$ximp
stopImplicitCluster()


#Ahora añadiremos los valores imputados al dataset:
x1 = as.numeric(rownames(datos_imput_alt_14_22[is.na(datos_finales_altitud$ALTITUD),]))

#Asignación
datos_finales_altitud[x1,5] = as.numeric(datos_imput_alt_14_22$ALTITUD[x1])

#Finalmente comprobaremos que las alturas de las fincas del mismo ID_ZONA, son similares, ya que en general
#si tienen el mismo ID_zona tienen la misma altitud!!
ZONAS = unique(as.numeric(as.character(datos_finales_altitud[x1,3])))
#Son zonas repetidas de las del dataset del 14_21

# Calcular la media de la ALTITUD para cada ID_ZONA en el vector ZONAS
altitud_media = datos_finales_altitud %>%
  filter(ID_ZONA %in% ZONAS) %>%
  group_by(ID_ZONA) %>%
  summarise(media_altitud = mean(ALTITUD, na.rm = TRUE))

#Devuelve las mismas medias que esas mismas zonas en la anterior asignacion





##----------------------------------------- SUPERFICIE -----------------------------------------##


#### AÑOS 2020 - 2021: Ya que no hay superficies en el resto de años:
datos_finales_20_21 = datos_finales_altitud[which(datos_finales_altitud$CAMPAÑA %in% c(20,21)),]

#En primer lugar imputamos los valores de las superficies para estos años, AQUI PODEMOS EMPLEAR LA PRODUCCION:

#Para computar en paralelo (más eficiente):
numCores = detectCores()
if(numCores > 14){numCores = 14} # The number of parallel cores should not exceed the number of variables (p=14)
registerDoParallel(cores = numCores)
set.seed(1234)
datos_imput_sup_20_21 = missForest(datos_finales_20_21[,-c(1,2,3,12,13)], maxiter = 1000, ntree = 1000, parallelize = 'forests')$ximp
stopImplicitCluster()

#Ahora agregaremos los valores imputados al dataset:
x1 = as.numeric(rownames(datos_imput_sup_20_21[is.na(datos_finales_20_21$SUPERFICIE),]))
dif = as.numeric(rownames(datos_imput_sup_20_21[1,])) - 1
#Asignación
datos_finales_altitud$SUPERFICIE[x1] = as.numeric(datos_imput_sup_20_21[x1 - dif,7])

#Veamos que valores se han imputado
datos_imput_sup_20_21$SUPERFICIE[is.na(datos_finales_20_21$SUPERFICIE)]



#### AÑO 2022: AQUI NO PODEMOS EMPLEAR LAS PRODUCCIONES PARA LA IMPUTACION

datos_finales_20_22 = datos_finales_altitud[which(datos_finales_altitud$CAMPAÑA %in% c(20,21,22)),]

#Para computar en paralelo (más eficiente):
numCores = detectCores()
if(numCores > 13){numCores = 13} # The number of parallel cores should not exceed the number of variables (p=13)
registerDoParallel(cores = numCores)
set.seed(1234)
datos_imput_sup_20_22 = missForest(datos_finales_20_22[,-c(1,2,3,11,12,13)], maxiter = 1000, ntree = 1000, parallelize = 'forests')$ximp
stopImplicitCluster()

#Ahora agregaremos los valores imputados al dataset:
x1 = as.numeric(rownames(datos_imput_sup_20_22[is.na(datos_finales_20_22$SUPERFICIE),]))
dif = as.numeric(rownames(datos_imput_sup_20_22[1,])) - 1
#Asignación
datos_finales_altitud$SUPERFICIE[x1] = as.numeric(datos_imput_sup_20_22[x1 - dif,7])

#Veamos que valores se han imputado
datos_imput_sup_20_22$SUPERFICIE[is.na(datos_finales_20_22$SUPERFICIE)]

#SE HA COMPLETADO LA IMPUTACION:

datos_finales_miss_forest = datos_finales_altitud

#Unimos datos_finales y MAIN.tr3 utilizando las columnas "CAMPÑ" e "ID_ESTACION" como claves de unión y luego filtra las filas en las que la columna "CAMPÑ" 
#es mayor que 15. El resultado se guarda en un nuevo conjunto de datos llamado datos_finales2.

datos_finales2 = datos_finales_miss_forest %>% left_join(x=datos_finales_miss_forest,y=MAIN.tr3,by=c("CAMPÑ","ID_ESTACION"))





datos_C_16_19 = datos_finales2 %>% filter(CAMPAÑA %in% 16:19)
datos_C_20_22 = datos_finales2 %>% filter(CAMPAÑA %in% 20:22)






###############################################################################
###############################################################################
#                            MODELOS DE PREDICCIÓN
###############################################################################
###############################################################################

################################################################################
#Para evitar problemas con la computacion en paralelo

unregister_dopar <- function() {
  env <- foreach:::.foreachGlobals
  rm(list=ls(name=env), pos=env)
}

unregister_dopar()
################################################################################





#En primer lugar introducieremos el modelo RANDOM FOREST POR CLUSTERS!




PROD.ANT2 = function(v, col.CAMPÑ = 12, diff = 0, total.data = ProdTotal.C) { #col.CAMPÑ es el numero de la columna CAMPÑ
  #diff = el diferencial para que las columnas se correspondan con los años adecuados
  #si se introducen c columnas diff = c
  
  PROD_1 = ifelse(is.na(v[as.numeric(v[col.CAMPÑ]) + 1 + diff]), 0, as.numeric(v[as.numeric(v[col.CAMPÑ]) + 1 + diff]))
  PROD_2 = ifelse(is.na(v[as.numeric(v[col.CAMPÑ]) + diff]), 0, as.numeric(v[as.numeric(v[col.CAMPÑ]) + diff]))
  PROD_mean = mean(as.numeric(v[(14 + diff):(as.numeric(v[col.CAMPÑ]) + 1 + diff)]), na.rm = T)
  prop_1 = ifelse(PROD_1 == 0, 0, PROD_1 / total.data[total.data$CAMPAÑA == (as.numeric(v[col.CAMPÑ]) - 1), 2])
  prop_2 = ifelse(PROD_2 == 0, 0, PROD_2 / total.data[total.data$CAMPAÑA == (as.numeric(v[col.CAMPÑ]) - 2), 2])
  return(c(PROD_1, PROD_2, PROD_mean, prop_1, prop_2))
}


final.fun2 = function(x, col.CAMPÑ = 12, diff = 0, total.data = ProdTotal.C){ #col.CAMPÑ es el numero de la columna CAMPÑ
  #diff = el diferencial para que las columnas se correspondan con los años adecuados
  #si se introducen c columnas diff = c
  #Nuevas variables:
  x = x %>% mutate(CAMPÑ=CAMPAÑA,ID_UNICO=paste(ID_FINCA,VARIEDAD,TIPO,MODO,sep="-"))
  
  #Dataset wide:
  x_wide_aux = x %>% pivot_wider(id_cols = ID_UNICO, names_from = CAMPÑ, values_from = PRODUCCION) %>% 
    mutate('12' = NA, '13' = NA) %>% relocate(after = c(1, 11, 12, 2:10))
  colnames(x_wide_aux) = c('ID_UNICO', as.character(12:22))
  
  #Juntamos el dataset normal con el wide
  x_aux = x %>% left_join(x = x, y = x_wide_aux, by = 'ID_UNICO', relationship = 'many-to-one')
  
  #Creamos las variables de PROD_1, PROD_2 y PROD_mean:
  result = t(apply(x_aux, 1 , PROD.ANT2, col.CAMPÑ = col.CAMPÑ, diff = diff, total.data = total.data))
  x_aux = cbind(x_aux, PROD_1 = result[,1], PROD_2 = result[,2], PROD_mean = result[,3], prop_1 = result[,4], prop_2 = result[,5])
  
  #Creamos las indicadoras de NAs y modificamos mean:
  x_aux$PROD_1na = ifelse(x_aux$PROD_1 == 0, 1, 0)
  x_aux$PROD_2na = ifelse(x_aux$PROD_2 == 0, 1, 0)
  x_aux$PROD_MEAN_NA = ifelse(is.na(x_aux$PROD_mean), 1, 0)
  x_aux$PROD_mean = ifelse(is.na(x_aux$PROD_mean), 0, x_aux$PROD_mean)
  
  #Eliminamos las columnas de wide:
  x_aux = x_aux[, !names(x_aux) %in% as.character(12:22)]
  return(x_aux)
}

datos_finalesaux = datos_finales_miss_forest %>% dplyr::select(-c(PROD_1, PROD_2, PROD_mean, PROD_1na, PROD_2na, PROD_MEAN_NA))
datos_finales2 = final.fun2(datos_finalesaux, col.CAMPÑ = 12, diff = 0, total.data = ProdTotal.C)
datos_finales2 = datos_finales2 %>% left_join(x=datos_finales2,y=MAIN.tr3,by=c("CAMPÑ","ID_ESTACION"))


#AHORA LLEVAREMOS A CABO LO MISMO AJUSTANDO UN MODELO PARA CADA CLUSTER:

########################################################################
############################### CLÚSTER 1 ############################## 
########################################################################




datos_C_16_22 = datos_finales2 %>% filter(CAMPAÑA %in% 16:22) %>% mutate(prop.dif = prop_1 - prop_2) %>% dplyr::select(prop_1, prop_2)
datos_C_16_22_scaled = scale(datos_C_16_22)
fviz_nbclust(x = datos_C_16_22_scaled, FUNcluster = kmeans, method = "wss", k.max = 15, 
             diss = get_dist(datos_C_16_22_scaled, method = "euclidean"), nstart = 50)

set.seed(1234)
km_clusters = kmeans(x = datos_C_16_22_scaled, centers = 3, nstart = 50)
datos_C_16_22 = datos_finales2 %>% filter(CAMPAÑA %in% 16:22) %>% mutate(prop.dif = prop_1 - prop_2)
datos_C_16_22$Cluster = as.factor(km_clusters$cluster)
ggplot(datos_C_16_22, aes(PROD_1, PROD_2, color = Cluster)) +
  geom_point() 


datos_C_16_19 = datos_C_16_22 %>% dplyr::filter(CAMPAÑA %in% 16:19)
datos_C_20_22 = datos_C_16_22 %>% dplyr::filter(CAMPAÑA %in% 20:22)



#------------------------------------------------------------------------------
# 1er RandomForest
#------------------------------------------------------------------------------

datos.train_16_19 = datos_C_16_19 %>% dplyr::filter(Cluster == 1) %>% 
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 


# Creación de los K-folds: 1 con cada uno de los cuatro años 16, 17, 18 y 19. --

Fold1<-datos_C_16_19 %>% filter(CAMPÑ==16) %>% dplyr::filter(Cluster == 1) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 

Fold2<-datos_C_16_19 %>% filter(CAMPÑ==17) %>% dplyr::filter(Cluster == 1) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 


Fold3<-datos_C_16_19 %>% filter(CAMPÑ==18) %>% dplyr::filter(Cluster == 1) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 


Fold4<-datos_C_16_19 %>% filter(CAMPÑ==19) %>% dplyr::filter(Cluster == 1) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 

Folds <- list(as.integer(rownames(Fold1)),
              as.integer(rownames(Fold2)),
              as.integer(rownames(Fold3)),
              as.integer(rownames(Fold4)))


#Datos de test:
datos.test_20_21 = datos_C_20_22 %>% filter(CAMPÑ %in% 20:21) %>% dplyr::filter(Cluster == 1) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 




#Búsqueda de hiperparámetros con Optimización Bayesiana -----------------------

#scoringFunction <- function(ntree, mtry) {
#  
#  tune_grid <- expand.grid(
#    mtry = mtry)
#  
#  
#  ctrl <- trainControl(method='repeatedcv', 
#                       number=4, 
#                       repeats=1, 
#                       search='grid',
#                       index = Folds,
#                       verboseIter = FALSE, 
#                       allowParallel = TRUE)
#  
#  
#  rf <- caret::train(form = PRODUCCION ~ ., data = datos.train_16_19,
#                     trControl = ctrl,
#                     tuneGrid = tune_grid,
#                     method = "rf",
#                     metric = 'RMSE',
#                     ntree = ntree,
#                     verbose = FALSE)  
#  
#  return(list( Score = -min(rf$results$RMSE)))
#}
#
#
#bounds <- list(
#  mtry = c(1L, 82L), 
#  ntree = c(100L, 2500L))
#
#numCores = detectCores()
#cl = makeCluster(numCores)
#registerDoParallel(cl, cores = numCores)
#
#clusterExport(cl, c('Folds', 'datos.train_16_19', 'scoringFunction', 'bounds'))
#clusterEvalQ(cl, {
#  library(hrbrthemes)
#  library(caret)
#  library(ParBayesianOptimization)
#  library(doParallel)
#})
#set.seed(12345)
#bayes_out <- bayesOpt(FUN = scoringFunction, bounds = bounds, 
#                      initPoints = 20, iters.n = 5, parallel =TRUE)
#
#stopCluster(cl)
#
#bayes_out$scoreSummary
#data.frame(getBestPars(bayes_out))
#df<-data.frame(getBestPars(bayes_out))




#Empleando los valores obtenidos por optimización bayesiana:

ctrl <- trainControl(method='repeatedcv', 
                     number=4, 
                     repeats=1, 
                     search='grid',
                     index = Folds,
                     verboseIter = FALSE, 
                     allowParallel = TRUE)

tune_grid <- expand.grid(
  mtry = 69)

set.seed(15032023) #Se fija la semilla a 15-03-2023

pct = proc.time()
numCores = detectCores()
cl = makeCluster(numCores)
registerDoParallel(cl, cores = numCores)

clusterExport(cl, c('Folds', 'datos.train_16_19'))
clusterEvalQ(cl, {
  library(caret)
})

rf1 <- caret::train(form = PRODUCCION ~ ., data = datos.train_16_19,
                    trControl = ctrl,
                    tuneGrid = tune_grid,
                    method = "rf",
                    metric = 'RMSE',
                    ntree = 248,
                    verbose = FALSE)

stopCluster(cl)
proc.time() - pct


y.rf = predict(rf1, newdata = datos.test_20_21)
y.prod = datos.test_20_21$PRODUCCION

print("RandomForest 1   RMSE-----")
sqrt(sum((y.rf-y.prod)^2)/length(y.prod)) 




################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################






#------------------------------------------------------------------------------
# 2do RandomForest
#------------------------------------------------------------------------------

datos.20_21 = datos_C_20_22 %>% filter(CAMPÑ %in% 20:21) %>% dplyr::filter(Cluster == 1) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 

datos.20_21$randomForest = y.rf



# Búsqueda de hiperparámetros con Optimización Bayesiana -----------------------

###################################################################################
#scoringFunction <- function(ntree, mtry) {
#  
#  tune_grid <- expand.grid(
#    mtry = mtry)
#  
#  ctrl <- trainControl(
#    method = "cv",
#    verboseIter = FALSE,
#    allowParallel = TRUE)
#  
#  
#  rf <- caret::train(form = PRODUCCION ~ ., data = datos.20_21,
#                     trControl = ctrl,
#                     tuneGrid = tune_grid,
#                     method = "rf",
#                     metric = 'RMSE',
#                     ntree = ntree,
#                     verbose = FALSE)  
#  
#  
#  return(list( Score = -min(rf$results$RMSE)))
#}
#
#
#bounds <- list(
#  mtry = c(1L, 82L), 
#  ntree = c(100L, 2500L))
#
#numCores = detectCores()
#cl = makeCluster(numCores)
#registerDoParallel(cl, cores = numCores)
#
#clusterExport(cl, c('datos.20_21', 'scoringFunction', 'bounds'))
#clusterEvalQ(cl, {
#  library(hrbrthemes)
#  library(caret)
#  library(ParBayesianOptimization)
#  library(doParallel)
#})

#set.seed(12345)
#bayes_out <- bayesOpt(FUN = scoringFunction, bounds = bounds, 
#                      initPoints = 20, iters.n = 5, parallel =TRUE)
#
#stopCluster(cl)
#
#bayes_out$scoreSummary
#data.frame(getBestPars(bayes_out))
#df<-data.frame(getBestPars(bayes_out))

###################################################################################



#Empleando los valores obtenidos por optimización bayesiana, veamos que RMSE obtiene en la muestra de validación:

tune_grid <- expand.grid(
  mtry = 82)


ctrl <- trainControl(
  method = "cv",
  verboseIter = FALSE,
  allowParallel = TRUE)



set.seed(15032023) #Se fija la semilla a 15-03-2023

pct = proc.time() 

numCores = detectCores()
cl = makeCluster(numCores)
registerDoParallel(cl, cores = numCores)

clusterExport(cl, c('datos.20_21'))
clusterEvalQ(cl, {
  library(caret)
})

rf2 <- caret::train(form = PRODUCCION ~ ., data = datos.20_21,
                    trControl = ctrl,
                    tuneGrid = tune_grid,
                    method = "rf",
                    metric = 'RMSE',
                    ntree = 2500,
                    verbose = FALSE)  

stopCluster(cl)
proc.time() - pct




#Generamos las predicciones:

datos_22 = datos_C_20_22 %>% filter(CAMPÑ %in% 22) %>% dplyr::filter(Cluster == 1) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 

datos_22$randomForest = predict(rf1, newdata = datos_22)


yFinal_C1 = predict(rf2, newdata = datos_22)

windows();hist(yFinal_C1)

datos_22_C1 = datos_22









################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################







########################################################################
############################### CLÚSTER 2 ############################## 
########################################################################

datos_C_16_22 = datos_finales2 %>% filter(CAMPAÑA %in% 16:22) %>% mutate(prop.dif = prop_1 - prop_2) %>% dplyr::select(prop_1, prop_2)
datos_C_16_22_scaled = scale(datos_C_16_22)
fviz_nbclust(x = datos_C_16_22_scaled, FUNcluster = kmeans, method = "wss", k.max = 15, 
             diss = get_dist(datos_C_16_22_scaled, method = "euclidean"), nstart = 50)

set.seed(1234)
km_clusters = kmeans(x = datos_C_16_22_scaled, centers = 3, nstart = 50)
datos_C_16_22 = datos_finales2 %>% filter(CAMPAÑA %in% 16:22) %>% mutate(prop.dif = prop_1 - prop_2)
datos_C_16_22$Cluster = as.factor(km_clusters$cluster)
ggplot(datos_C_16_22, aes(PROD_1, PROD_2, color = Cluster)) +
  geom_point() 


datos_C_16_19 = datos_C_16_22 %>% dplyr::filter(CAMPAÑA %in% 16:19)
datos_C_20_22 = datos_C_16_22 %>% dplyr::filter(CAMPAÑA %in% 20:22)



#------------------------------------------------------------------------------
# 1er RandomForest
#------------------------------------------------------------------------------

datos.train_16_19 = datos_C_16_19 %>% dplyr::filter(Cluster == 2) %>% 
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 


# Creación de los K-folds: 1 con cada uno de los cuatro años 16, 17, 18 y 19. --

Fold1<-datos_C_16_19 %>% filter(CAMPÑ==16) %>% dplyr::filter(Cluster == 2) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 

Fold2<-datos_C_16_19 %>% filter(CAMPÑ==17) %>% dplyr::filter(Cluster == 2) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 


Fold3<-datos_C_16_19 %>% filter(CAMPÑ==18) %>% dplyr::filter(Cluster == 2) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 


Fold4<-datos_C_16_19 %>% filter(CAMPÑ==19) %>% dplyr::filter(Cluster == 2) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 

Folds <- list(as.integer(rownames(Fold1)),
              as.integer(rownames(Fold2)),
              as.integer(rownames(Fold3)),
              as.integer(rownames(Fold4)))


#Datos de test:
datos.test_20_21 = datos_C_20_22 %>% filter(CAMPÑ %in% 20:21) %>% dplyr::filter(Cluster == 2) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 




#Búsqueda de hiperparámetros con Optimización Bayesiana -----------------------

#scoringFunction <- function(ntree, mtry) {
#  
#  tune_grid <- expand.grid(
#    mtry = mtry)
#  
#  
#  ctrl <- trainControl(method='repeatedcv', 
#                       number=4, 
#                       repeats=1, 
#                       search='grid',
#                       index = Folds,
#                       verboseIter = FALSE, 
#                       allowParallel = TRUE)
#  
#  
#  rf <- caret::train(form = PRODUCCION ~ ., data = datos.train_16_19,
#                     trControl = ctrl,
#                     tuneGrid = tune_grid,
#                     method = "rf",
#                     metric = 'RMSE',
#                     ntree = ntree,
#                     verbose = FALSE)  
#  
#  return(list( Score = -min(rf$results$RMSE)))
#}
#
#
#bounds <- list(
#  mtry = c(1L, 82L), 
#  ntree = c(100L, 2500L))
#
#
#
#
#numCores = detectCores()
#cl = makeCluster(numCores)
#registerDoParallel(cl, cores = numCores)
#
#clusterExport(cl, c('Folds', 'datos.train_16_19', 'scoringFunction', 'bounds'))
#clusterEvalQ(cl, {
#  library(hrbrthemes)
#  library(caret)
#  library(ParBayesianOptimization)
#  library(doParallel)
#})
#set.seed(1234)
#bayes_out <- bayesOpt(FUN = scoringFunction, bounds = bounds, 
#                      initPoints = 20, iters.n = 5, parallel =TRUE)
#
#stopCluster(cl)
#
#bayes_out$scoreSummary
#data.frame(getBestPars(bayes_out))
#df<-data.frame(getBestPars(bayes_out))




#Empleando los valores obtenidos por optimización bayesiana:

ctrl <- trainControl(method='repeatedcv', 
                     number=4, 
                     repeats=1, 
                     search='grid',
                     index = Folds,
                     verboseIter = FALSE, 
                     allowParallel = TRUE)

tune_grid <- expand.grid(
  mtry = 78)

set.seed(15032023) #Se fija la semilla a 15-03-2023

pct = proc.time() 
numCores = detectCores()
cl = makeCluster(numCores)
registerDoParallel(cl, cores = numCores)

clusterExport(cl, c('Folds', 'datos.train_16_19'))
clusterEvalQ(cl, {
  library(caret)
})

rf1 <- caret::train(form = PRODUCCION ~ ., data = datos.train_16_19,
                    trControl = ctrl,
                    tuneGrid = tune_grid,
                    method = "rf",
                    metric = 'RMSE',
                    ntree = 1873,
                    verbose = FALSE)

stopCluster(cl)
proc.time() - pct


y.rf = predict(rf1, newdata = datos.test_20_21)
y.prod = datos.test_20_21$PRODUCCION

print("RandomForest 1   RMSE-----")
sqrt(sum((y.rf-y.prod)^2)/length(y.prod)) 




################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################






#------------------------------------------------------------------------------
# 2do RandomForest
#------------------------------------------------------------------------------

datos.20_21 = datos_C_20_22 %>% filter(CAMPÑ %in% 20:21) %>% dplyr::filter(Cluster == 2) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 

datos.20_21$randomForest = y.rf



# Búsqueda de hiperparámetros con Optimización Bayesiana -----------------------

###################################################################################
#scoringFunction <- function(ntree, mtry) {
#  
#  tune_grid <- expand.grid(
#    mtry = mtry)
#  
#  ctrl <- trainControl(
#    method = "cv",
#    verboseIter = FALSE,
#    allowParallel = TRUE)
#  
#  
#  rf <- caret::train(form = PRODUCCION ~ ., data = datos.20_21,
#                     trControl = ctrl,
#                     tuneGrid = tune_grid,
#                     method = "rf",
#                     metric = 'RMSE',
#                     ntree = ntree,
#                     verbose = FALSE)  
#  
#  
#  return(list( Score = -min(rf$results$RMSE)))
#}
#
#
#bounds <- list(
#  mtry = c(1L, 82L), 
#  ntree = c(100L, 2500L))
#
#numCores = detectCores()
#cl = makeCluster(numCores)
#registerDoParallel(cl, cores = numCores)
#
#clusterExport(cl, c('datos.20_21', 'scoringFunction', 'bounds'))
#clusterEvalQ(cl, {
#  library(hrbrthemes)
#  library(caret)
#  library(ParBayesianOptimization)
#  library(doParallel)
#})
#set.seed(12345)
#bayes_out <- bayesOpt(FUN = scoringFunction, bounds = bounds, 
#                      initPoints = 20, iters.n = 5, parallel =TRUE)
#
#stopCluster(cl)
#
#bayes_out$scoreSummary
#data.frame(getBestPars(bayes_out))
#df<-data.frame(getBestPars(bayes_out))

###################################################################################



#Empleando los valores obtenidos por optimización bayesiana, veamos que RMSE obtiene en la muestra de validación:

tune_grid <- expand.grid(
  mtry = 57)


ctrl <- trainControl(
  method = "cv",
  verboseIter = FALSE,
  allowParallel = TRUE)



set.seed(15032023) #Se fija la semilla a 15-03-2023
pct = proc.time()
numCores = detectCores()
cl = makeCluster(numCores)
registerDoParallel(cl, cores = numCores)

clusterExport(cl, c('datos.20_21'))
clusterEvalQ(cl, {
  library(caret)
})

rf2 <- caret::train(form = PRODUCCION ~ ., data = datos.20_21,
                    trControl = ctrl,
                    tuneGrid = tune_grid,
                    method = "rf",
                    metric = 'RMSE',
                    ntree = 786,
                    verbose = FALSE)  

stopCluster(cl)
proc.time() - pct



#Generamos las predicciones para el concurso:

datos_22 = datos_C_20_22 %>% filter(CAMPÑ %in% 22) %>% dplyr::filter(Cluster == 2) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 

datos_22$randomForest = predict(rf1, newdata = datos_22)

yFinal_C2 = predict(rf2, newdata = datos_22)

windows();hist(yFinal_C2)

datos_22_C2 = datos_22



################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################






########################################################################
############################### CLÚSTER 3 ############################## 
########################################################################


datos_C_16_22 = datos_finales2 %>% filter(CAMPAÑA %in% 16:22) %>% mutate(prop.dif = prop_1 - prop_2) %>% dplyr::select(prop_1, prop_2)
datos_C_16_22_scaled = scale(datos_C_16_22)
fviz_nbclust(x = datos_C_16_22_scaled, FUNcluster = kmeans, method = "wss", k.max = 15, 
             diss = get_dist(datos_C_16_22_scaled, method = "euclidean"), nstart = 50)

set.seed(1234)
km_clusters = kmeans(x = datos_C_16_22_scaled, centers = 3, nstart = 50)
datos_C_16_22 = datos_finales2 %>% filter(CAMPAÑA %in% 16:22) %>% mutate(prop.dif = prop_1 - prop_2)
datos_C_16_22$Cluster = as.factor(km_clusters$cluster)
ggplot(datos_C_16_22, aes(PROD_1, PROD_2, color = Cluster)) +
  geom_point() 


datos_C_16_19 = datos_C_16_22 %>% dplyr::filter(CAMPAÑA %in% 16:19)
datos_C_20_22 = datos_C_16_22 %>% dplyr::filter(CAMPAÑA %in% 20:22)



#------------------------------------------------------------------------------
# 1er RandomForest
#------------------------------------------------------------------------------

datos.train_16_19 = datos_C_16_19 %>% dplyr::filter(Cluster == 3) %>% 
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 


# Creación de los K-folds: 1 con cada uno de los cuatro años 16, 17, 18 y 19. --

Fold1<-datos_C_16_19 %>% filter(CAMPÑ==16) %>% dplyr::filter(Cluster == 3) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 

Fold2<-datos_C_16_19 %>% filter(CAMPÑ==17) %>% dplyr::filter(Cluster == 3) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 


Fold3<-datos_C_16_19 %>% filter(CAMPÑ==18) %>% dplyr::filter(Cluster == 3) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 


Fold4<-datos_C_16_19 %>% filter(CAMPÑ==19) %>% dplyr::filter(Cluster == 3) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))]) 

Folds <- list(as.integer(rownames(Fold1)),
              as.integer(rownames(Fold2)),
              as.integer(rownames(Fold3)),
              as.integer(rownames(Fold4)))


#Datos de test:
datos.test_20_21 = datos_C_20_22 %>% filter(CAMPÑ %in% 20:21) %>% dplyr::filter(Cluster == 3) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 




#Búsqueda de hiperparámetros con Optimización Bayesiana -----------------------

#scoringFunction <- function(ntree, mtry) {
#  
#  tune_grid <- expand.grid(
#    mtry = mtry)
#  
#  
#  ctrl <- trainControl(method='repeatedcv', 
#                       number=4, 
#                       repeats=1, 
#                       search='grid',
#                       index = Folds,
#                       verboseIter = FALSE, 
#                       allowParallel = TRUE)
#  
#  
#  rf <- caret::train(form = PRODUCCION ~ ., data = datos.train_16_19,
#                     trControl = ctrl,
#                     tuneGrid = tune_grid,
#                     method = "rf",
#                     metric = 'RMSE',
#                     ntree = ntree,
#                     verbose = FALSE)  
#  
#  return(list( Score = -min(rf$results$RMSE)))
#}
#
#
#bounds <- list(
#  mtry = c(1L, 82L), 
#  ntree = c(100L, 2500L))
#
#
#
#
#numCores = detectCores()
#cl = makeCluster(numCores)
#registerDoParallel(cl, cores = numCores)
#
#clusterExport(cl, c('Folds', 'datos.train_16_19', 'scoringFunction', 'bounds'))
#clusterEvalQ(cl, {
#  library(hrbrthemes)
#  library(caret)
#  library(ParBayesianOptimization)
#  library(doParallel)
#})
#set.seed(1234)
#bayes_out <- bayesOpt(FUN = scoringFunction, bounds = bounds, 
#                      initPoints = 20, iters.n = 10, parallel =TRUE)
#
#stopCluster(cl)
#
#bayes_out$scoreSummary
#data.frame(getBestPars(bayes_out))
#df<-data.frame(getBestPars(bayes_out))




#Empleando los valores obtenidos por optimización bayesiana:

ctrl <- trainControl(method='repeatedcv', 
                     number=4, 
                     repeats=1, 
                     search='grid',
                     index = Folds,
                     verboseIter = FALSE, 
                     allowParallel = TRUE)

tune_grid <- expand.grid(
  mtry = 64)

set.seed(15032023) #Se fija la semilla a 15-03-2023
pct = proc.time()
numCores = detectCores()
cl = makeCluster(numCores)
registerDoParallel(cl, cores = numCores)

clusterExport(cl, c('Folds', 'datos.train_16_19'))
clusterEvalQ(cl, {
  library(caret)
})

rf1 <- caret::train(form = PRODUCCION ~ ., data = datos.train_16_19,
                    trControl = ctrl,
                    tuneGrid = tune_grid,
                    method = "rf",
                    metric = 'RMSE',
                    ntree = 580,
                    verbose = FALSE)

stopCluster(cl)
proc.time() - pct



y.rf = predict(rf1, newdata = datos.test_20_21)
y.prod = datos.test_20_21$PRODUCCION

print("RandomForest 1   RMSE-----")
sqrt(sum((y.rf-y.prod)^2)/length(y.prod)) 




################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################






#------------------------------------------------------------------------------
# 2do RandomForest
#------------------------------------------------------------------------------

datos.20_21 = datos_C_20_22 %>% filter(CAMPÑ %in% 20:21) %>% dplyr::filter(Cluster == 3) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 

datos.20_21$randomForest = y.rf


# Búsqueda de hiperparámetros con Optimización Bayesiana -----------------------

###################################################################################
#scoringFunction <- function(ntree, mtry) {
#  
#  tune_grid <- expand.grid(
#    mtry = mtry)
#  
#  ctrl <- trainControl(
#    method = "cv",
#    verboseIter = FALSE,
#    allowParallel = TRUE)
#  
#  
#  rf <- caret::train(form = PRODUCCION ~ ., data = datos.20_21,
#                     trControl = ctrl,
#                     tuneGrid = tune_grid,
#                     method = "rf",
#                     metric = 'RMSE',
#                     ntree = ntree,
#                     verbose = FALSE)  
#  
#  
#  return(list( Score = -min(rf$results$RMSE)))
#}
#
#
#bounds <- list(
#  mtry = c(1L, 82L), 
#  ntree = c(100L, 2500L))
#
#numCores = detectCores()
#cl = makeCluster(numCores)
#registerDoParallel(cl, cores = numCores)
#
#clusterExport(cl, c('datos.20_21', 'scoringFunction', 'bounds'))
#clusterEvalQ(cl, {
#  library(hrbrthemes)
#  library(caret)
#  library(ParBayesianOptimization)
#  library(doParallel)
#})
#set.seed(12345)
#bayes_out <- bayesOpt(FUN = scoringFunction, bounds = bounds, 
#                      initPoints = 20, iters.n = 5, parallel =TRUE)
#
#stopCluster(cl)
#
#bayes_out$scoreSummary
#data.frame(getBestPars(bayes_out))
#df<-data.frame(getBestPars(bayes_out))

###################################################################################



#Empleando los valores obtenidos por optimización bayesiana, veamos que RMSE obtiene en la muestra de validación:

tune_grid <- expand.grid(
  mtry = 24)


ctrl <- trainControl(
  method = "cv",
  verboseIter = FALSE,
  allowParallel = TRUE)



set.seed(15032023) #Se fija la semilla a 15-03-2023
pct = proc.time() 
numCores = detectCores()
cl = makeCluster(numCores)
registerDoParallel(cl, cores = numCores)

clusterExport(cl, c('datos.20_21'))
clusterEvalQ(cl, {
  library(caret)
})

rf2 <- caret::train(form = PRODUCCION ~ ., data = datos.20_21,
                    trControl = ctrl,
                    tuneGrid = tune_grid,
                    method = "rf",
                    metric = 'RMSE',
                    ntree = 1485,
                    verbose = FALSE)  

stopCluster(cl)
proc.time() - pct



#Generamos las predicciones para el concurso:

datos_22 = datos_C_20_22 %>% filter(CAMPÑ %in% 22) %>% dplyr::filter(Cluster == 3) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" ,'Cluster')) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 

datos_22$randomForest = predict(rf1, newdata = datos_22)


yFinal_C3 = predict(rf2, newdata = datos_22)

windows();hist(yFinal_C3)
windows();hist(datos.20_21$PRODUCCION)






#Las predicciones finales son:

datos_22_C1 = datos_C_20_22 %>% filter(CAMPÑ == 22) %>% dplyr::filter(Cluster == 1)
datos_22_C2 = datos_C_20_22 %>% filter(CAMPÑ == 22) %>% dplyr::filter(Cluster == 2)
datos_22_C3 = datos_C_20_22 %>% filter(CAMPÑ == 22) %>% dplyr::filter(Cluster == 3)


datos_22_C1$PREDICCION = yFinal_C1
datos_22_C2$PREDICCION = yFinal_C2
datos_22_C3$PREDICCION = yFinal_C3

yFinal = c(yFinal_C1, yFinal_C2, yFinal_C3)

datos_predic = rbind(datos_22_C1, datos_22_C2, datos_22_C3)


ENVIAR<-datos_predic %>%dplyr::select(ID_FINCA,VARIEDAD,MODO,TIPO,COLOR,SUPERFICIE,PREDICCION)



ENVIAR2<- ENVIAR %>% 
  arrange(ID_FINCA,VARIEDAD,MODO,TIPO,COLOR,SUPERFICIE)

ENVIAR_FINAL<-ENVIAR2 %>% mutate(VARIEDAD=ifelse(nchar(as.character(VARIEDAD))==2,
                                                 as.character(VARIEDAD),paste0(0,VARIEDAD)),
                                 TIPO=as.character(TIPO),
                                 PREDICCION=round(PREDICCION,2))

ENVIAR_FINAL




################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################











###########################################################################################################
###########################################################################################################
########################################       MODELO GAM      ############################################
###########################################################################################################
###########################################################################################################

library(mgcv)

datos_finales2 = datos_finales_miss_forest %>% left_join(x=datos_finales_miss_forest,y=MAIN.tr3,by=c("CAMPÑ","ID_ESTACION"))

datos_C_20_21 = datos_finales2 %>% filter(CAMPAÑA %in% 20:21)

set.seed(1234)
index = unlist(createDataPartition(datos_C_20_21[,1], p = 0.8))
training = datos_C_20_21[index,]
testing = datos_C_20_21[-index,]


#Modelo sin meteorología:

pct = proc.time()
modGAM<-gam(PRODUCCION~s(SUPERFICIE, by=PROD_MEAN_NA) + s(PROD_mean, by = PROD_MEAN_NA), data=training)
proc.time() - pct


summary(modGAM)
yGAM=predict(modGAM,testing)
y.mg = testing$PRODUCCION

print("GAM----------")
sqrt(sum((yGAM-y.mg)^2)/length(y.mg)) #GAM~

#Pero esto es para una sola semilla (poco fiable)


#Otra medicion del error más rigurosa
B = 1000
set.seed(12345)
result1 = replicate(B, {trainIndex <- createDataPartition(datos_C_20_21$PRODUCCION, p = 0.8, list = FALSE)

#Esto se debe a que solo tengo superficies para estos años
training <- datos_C_20_21[trainIndex,] 
testing <- datos_C_20_21[-trainIndex,] 

y.mg<-testing$PRODUCCION #La produccion del año 21

#Modelo sin meteorología: (El mejor hasta ahora)
modGAM<-gam(PRODUCCION~s(SUPERFICIE, by=PROD_MEAN_NA) + s(PROD_mean, by = PROD_MEAN_NA), data=training)

yGAM=predict(modGAM,testing)
return(sqrt(sum((yGAM-y.mg)^2)/length(y.mg)) )#GAM~
})

mean(result1)
windows();hist(result1)

#El RMSE será de 7229.07 (Bastante alto).


#Predicciones 2022:
datos_22 = datos_finales2 %>% filter(CAMPAÑA == 22)
yfinal = predict(modGAM, datos_22)

hist(yfinal) #NO se asemejademasiado a los histogramas de producciones de otros años
hist(datos_C_20_21$PRODUCCION)










################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################














###########################################################################################################
###########################################################################################################
####################################       MODELO XGBOOST HIST     ########################################
###########################################################################################################
###########################################################################################################






# 2 XGBoost encadenados, 1 para 16-19 y otro para 20-21


# Train y test del modelo 16-19 -----------------------------------------------

mg.train<-datos_finales2 %>% filter(CAMPÑ %in% c(16,17,18,19)) %>%
  filter(PRODUCCION>0)%>% 
  mutate(ALTITUD=ifelse(is.na(ALTITUD),0,ALTITUD))%>%dplyr::select(-c("CAMPAÑA","ID_FINCA",
                                                                      "ID_ESTACION","wPRODUCCION",
                                                                      "ID_UNICO","PROD_mean","PROD_MEAN_NA"))

# Se decidió suprimir también las variables climáticas con agrupación Daytime y Nighttime

mg.train2<-mg.train%>%dplyr::select(-names(mg.train)[grep("Daytime|Nighttime",names(mg.train))])




mg.test2<-datos_finales2 %>% filter(CAMPÑ %in% c(20,21)) %>% filter(PRODUCCION>0)%>%
  dplyr::select(-c("CAMPAÑA","ID_FINCA","ID_ESTACION","wPRODUCCION","ID_UNICO"))%>%
  dplyr::select(-names(mg.train)[grep("Daytime|Nighttime",names(mg.train))])

y.mg<-mg.test2$PRODUCCION






# Creación de los K-folds: 1 con cada uno de los cuatro años 16, 17, 18 y 19. --

Fold1<-datos_finales2 %>% filter(CAMPÑ==16) %>%
  filter(PRODUCCION>0)%>% 
  mutate(ALTITUD=ifelse(is.na(ALTITUD),0,ALTITUD))%>%dplyr::select(-c("CAMPAÑA","ID_FINCA",
                                                                      "ID_ESTACION","wPRODUCCION",
                                                                      "ID_UNICO"))%>%
  dplyr::select(-names(mg.train)[grep("Daytime|Nighttime",names(mg.train))])

Fold2<-datos_finales2 %>% filter(CAMPÑ==17) %>%
  filter(PRODUCCION>0)%>% 
  mutate(ALTITUD=ifelse(is.na(ALTITUD),0,ALTITUD))%>%dplyr::select(-c("CAMPAÑA","ID_FINCA",
                                                                      "ID_ESTACION","wPRODUCCION",
                                                                      "ID_UNICO"))%>%
  dplyr::select(-names(mg.train)[grep("Daytime|Nighttime",names(mg.train))])


Fold3<-datos_finales2 %>% filter(CAMPÑ==18) %>%
  filter(PRODUCCION>0)%>% 
  mutate(ALTITUD=ifelse(is.na(ALTITUD),0,ALTITUD))%>%dplyr::select(-c("CAMPAÑA","ID_FINCA",
                                                                      "ID_ESTACION","wPRODUCCION",
                                                                      "ID_UNICO"))%>%
  dplyr::select(-names(mg.train)[grep("Daytime|Nighttime",names(mg.train))])


Fold4<-datos_finales2 %>% filter(CAMPÑ==19) %>%
  filter(PRODUCCION>0)%>% 
  mutate(ALTITUD=ifelse(is.na(ALTITUD),0,ALTITUD))%>%dplyr::select(-c("CAMPAÑA","ID_FINCA",
                                                                      "ID_ESTACION","wPRODUCCION",
                                                                      "ID_UNICO"))%>%
  dplyr::select(-names(mg.train)[grep("Daytime|Nighttime",names(mg.train))])

Folds <- list(as.integer(rownames(Fold1)),
              as.integer(rownames(Fold2)),
              as.integer(rownames(Fold3)),
              as.integer(rownames(Fold4)))





# Búsqueda de hiperparámetros con Optimización Bayesiana -----------------------


#La ejecución de esto puede llevar tiempo y se realizó varias veces hasta encontrar 
#unos parámetros adecuados. En este script se ajusta directamente los XGBoost con
#los parámetros elegidos



# scoringFunction <- function(max_depth, min_child_weight, subsample,eta) {
#   
#   tune_grid <- expand.grid(
#     nrounds = seq(from = 400, to = 1200, by = 50), 
#     eta = eta,
#     max_depth = max_depth,
#     gamma = 0,
#     colsample_bytree = 1,
#     min_child_weight = min_child_weight,
#     subsample = subsample
#   )
#   
#   ctrl <- trainControl(
#     method = "repeatedcv",
#     number = 4,
#     repeats = 1,
#     index = Folds,
#     verboseIter = FALSE
#   )
#   
#   xgbcv <- caret::train(form = PRODUCCION ~ ., data = mg.train,
#                         trControl = ctrl,
#                         tuneGrid = tune_grid,
#                         method = "xgbTree",
#                         verbose = FALSE,
#                         tree_method="hist")  #Método Hist: La implementación de LightGBM del paquete xgboost con
#                                              # el que interactúa caret
#   
#   return(list( Score = -min(xgbcv$results$RMSE)
#                , nrounds = xgbcv$bestTune$nrounds
#   )
#   )
# }
# 
# 
# bounds <- list( max_depth = c(2L, 15L),
#                 min_child_weight = c(1, 100), 
#                 subsample = c(0.25, 1),
#                 eta = c(0.001, 0.5))
# 
# 
# bayes_out <- bayesOpt(FUN = scoringFunction, bounds = bounds, 
#                       initPoints = 12, iters.n = 8,iters.k=4)
# 
# 
# #Para continuar la búsqueda
# #bayes_out2<-addIterations(bayes_out,iters.n=10,iters.k=4, bounds = bounds)
# 
# bayes_out$scoreSummary
# data.frame(getBestPars(bayes_out))
# df<-data.frame(getBestPars(bayes_out))


#------------------------------------------------------------------------------
# 1er XGBoost
#------------------------------------------------------------------------------



#Parámetros escogidos:
# eta             max_depth   min_child_weight   subsample
# 0.007580668     9           2.683458           0.3409129


tune_grid <- expand.grid(
  nrounds = c(200,400,500),
  eta = 0.007580668,
  max_depth = 9,
  gamma = 0,
  colsample_bytree = 1,
  min_child_weight = 2.683458,
  subsample = 0.3409129
)

ctrl <- trainControl(
  method = "repeatedcv",
  number = 4,
  repeats = 1,
  index = Folds,
  verboseIter = FALSE
)

set.seed(16032023) #Se fija la semilla a 16-03-2023

#Aqui no se aplica computación en paralelo pq se tarda más en crear el cluster que en llevar a cabo el xgboost
pct = proc.time()
xgbcv <- caret::train(form = PRODUCCION ~ ., data = mg.train2,
                      trControl = ctrl,
                      tuneGrid = tune_grid,
                      method = "xgbTree",
                      verbose = FALSE,
                      tree_method="hist")
proc.time() - pct 



xgbcv
mg.test<-datos_finales2 %>% filter(CAMPÑ %in% c(20,21))
y.prod<-mg.test2$PRODUCCION
y1Def.xgb = predict( xgbcv, newdata = mg.test2)


print("XGBoost Tree 1   RMSE-----")
sqrt(sum((y1Def.xgb-y.prod)^2)/length(y.prod)) 





#------------------------------------------------------------------------------
# 2do XGBoost
#------------------------------------------------------------------------------


mg.test2$xgboost<-y1Def.xgb


# scoringFunction <- function(max_depth, min_child_weight, subsample,eta) {
#   
#   tune_grid <- expand.grid(
#     nrounds = seq(from = 400, to = 1200, by = 50),
#     eta = eta,
#     max_depth = max_depth,
#     gamma = 0,
#     colsample_bytree = 1,
#     min_child_weight = min_child_weight,
#     subsample = subsample
#   )
#   
#   ctrl <- trainControl(
#     method = "cv",
#     verboseIter = FALSE
#   )
#   
#   xgbcv <- caret::train(form = PRODUCCION ~ ., data = mg.test2,
#                         trControl = ctrl,
#                         tuneGrid = tune_grid,
#                         method = "xgbTree",
#                         verbose = FALSE,
#                         tree_method="hist")
#   
#   return(list( Score = -min(xgbcv$results$RMSE)
#                , nrounds = xgbcv$bestTune$nrounds
#   )
#   )
# }
# 
# 
# bounds <- list( max_depth = c(2L, 15L),
#                 min_child_weight = c(1, 100), 
#                 subsample = c(0.25, 1),
#                 eta = c(0.001, 0.5))
# 
# 
# bayes_out <- bayesOpt(FUN = scoringFunction, bounds = bounds, 
#                       initPoints = 12, iters.n = 8,iters.k=4)
# 
# bayes_out2<-addIterations(bayes_out,iters.n=10,iters.k=4, bounds = bounds)
# bayes_out3<-addIterations(bayes_out2,iters.n=10,iters.k=4, bounds = bounds)
# 
# 
# bayes_out3$scoreSummary
# df<-data.frame(getBestPars(bayes_out))




#--------------------------

datos20.22<- datos_finales2%>%filter(CAMPÑ %in% c(20,21,22))%>%dplyr::select(-c("ID_FINCA",
                                                                                "ID_ESTACION","wPRODUCCION",
                                                                                "ID_UNICO"))%>%
  dplyr::select(-names(mg.train)[grep("Daytime|Nighttime",names(mg.train))])



training <- datos20.22 %>% filter(CAMPAÑA %in% c(20,21)) %>%
  mutate(ALTITUD=ifelse(is.na(ALTITUD),0,ALTITUD))%>% dplyr::select(-CAMPAÑA)
testing <- datos20.22%>% filter(CAMPAÑA==22) %>%
  mutate(ALTITUD=ifelse(is.na(ALTITUD),0,ALTITUD))%>% dplyr::select(-CAMPAÑA)



y6.mg = predict( xgbcv, newdata = training)
training$xgboost<-y6.mg 

y6.mg2 = predict( xgbcv, newdata = testing)
testing$xgboost<-y6.mg2 







#Parámetros escogidos:
# eta             max_depth   min_child_weight   subsample
# 0.02288690      15          1                  0.6105720



tune_grid <- expand.grid(
  nrounds = c(50,100,200,300),
  eta = 0.02288690,
  max_depth = 15,
  gamma = 0,
  colsample_bytree = 1,
  min_child_weight = 1,
  subsample = 0.6105720
)

ctrl <- trainControl(
  method = "cv",
  verboseIter = FALSE
)

set.seed(16032023)
#De nuevo no sale rentable en cuanto a tiempo llevar a cabo la computación en paralelo, ya que la creación
#del cluster lleva tanto tiempo como el xgboos sin paralelizar
pct = proc.time()
xgbFinal <- caret::train(form = PRODUCCION ~ ., data = training,
                         trControl = ctrl,
                         tuneGrid = tune_grid,
                         method = "xgbTree",
                         verbose = FALSE,
                         tree_method="hist")
proc.time() - pct

xgbFinal

yFinal = predict(xgbFinal, newdata = testing)



mg.test2<-datos_finales2 %>% filter(CAMPÑ%in% c(22)) %>%
  mutate(ALTITUD=ifelse(is.na(ALTITUD),0,ALTITUD))



mg.test2$PREDICCION<-yFinal

ENVIAR<-mg.test2 %>%dplyr::select(ID_FINCA,VARIEDAD,MODO,TIPO,COLOR,SUPERFICIE,PREDICCION)







ENVIAR2<- ENVIAR %>% 
  arrange(ID_FINCA,VARIEDAD,MODO,TIPO,COLOR,SUPERFICIE)

ENVIAR3<-ENVIAR2 %>% mutate(VARIEDAD=ifelse(nchar(as.character(VARIEDAD))==2,
                                            as.character(VARIEDAD),paste0(0,VARIEDAD)),
                            TIPO=as.character(TIPO),
                            PREDICCION=round(PREDICCION,2))

ENVIAR3
