library(ggplot2)
library(dplyr) #Es importante tener instalada la última versión de dplyr
library(hrbrthemes)
library(lubridate)
library(zoo)
library(caret)
library(tidyr)
library(ParBayesianOptimization)
library(missForest)
library(mice)
library(doParallel)
library(factoextra)
library(xgboost) #Es importante tener instalada la última versión de xgboost

setwd("C:/Users/usuario/Documents/AA COMPETICION VINOS")

datosMETEO<- read.table("DATOS_METEO.TXT", sep = "|", header = TRUE, encoding = "UTF-8")
#str(datosMETEO)

datosETO<- read.table("DATOS_ETO.TXT", sep = "|", header = TRUE, encoding = "UTF-8")
#str(datosETO)

Goidanich<- read.csv("Goidanich.csv", sep = ";", header = TRUE, encoding = "UTF-8")
#str(Goidanich)

datosUH<- read.table("UH_2023_TRAIN.txt", sep = "|", header = TRUE, encoding = "UTF-8")



###############################################################################
###############################################################################

# TRANSFORMACIÓN Y PREPROCESADO DE LOS DATOS

###############################################################################
###############################################################################


#En el ficheroGoidanich.csv se encuaentra la tabla propuesta por Goidanich para la evolución
#del riesgo de contagio de mildeo según factores climáticos. La tabla está discretizada y si bien se puede actualizar
#el indicador por los valores de temperatura más cercanos de la tabla, aquí se ha optado
#por ajustar una regresión no paramétrica que sirviese como método de interpolación,
#ya que era una forma rápida y sencilla de interpolar los datos

plot(x=Goidanich$T.media,y=Goidanich$Desar.Hm.mn.75)
Goid.mn.loess <- loess(Desar.Hm.mn.75~T.media, span=0.3,data=Goidanich)
lines(Goid.mn.loess,lwd=2,col="magenta")

Goid.mn.loess.pred<-function(x){
  print(x)
  if (x>=12){
    p.eval <- data.frame(T.media=c(x))
    predG<-as.numeric(predict(Goid.mn.loess, p.eval))}
  else{
    predG<-0
  }
  return(predG)
}

plot(x=Goidanich$T.media,y=Goidanich$Desar.Hm.my.75)
Goid.my.loess <- loess(Desar.Hm.my.75~T.media, span=0.3,data=Goidanich)
lines(Goid.my.loess,lwd=2,col="magenta")

Goid.my.loess.pred<-function(x){
  if (x>=12){
    p.eval <- data.frame(T.media=c(x))
    predG<-as.numeric(predict(Goid.my.loess, p.eval))}
  else{
    predG<-0
  }
  return(predG)
}


#Modificación de los datos



#Vector de producciones totales por campaña
sumprod.A<-function(x){sum(datosUH$PRODUCCION[datosUH$CAMPAÑA==x])}
x<-14:21
ProdTotal.C<-data.frame(CAMPAÑA=x,PROD_total=sapply(x,FUN="sumprod.A"))


#Creamos la media de las temperaturas de los ultimos 5 dias, 
#y la precipitacion media de los últimos 2 dias

datosETO.red<-datosETO %>% 
  mutate(TempLDA=TemperatureLocalDayAvg-273.15,
         PrecipLDA=PrecipAmountLocalDayAvg,
         HmLDA=RelativeHumidityLocalDayAvg)%>%
  #dplyr::select(date,ID_ESTACION,TempLDA,PrecipLDA,HmLDA) %>%
  mutate(dateOK=as.Date(as.character(date), format = "%Y%m%d"))  #%>% dplyr::select(dateOK,ID_ESTACION,TempLDA,PrecipLDA,HmLDA)


datosETO.tr <- datosETO.red %>%
  group_by(ID_ESTACION) %>%
  arrange(dateOK,.by_group=TRUE) %>%
  mutate(TempLDA_5d = rollapply(TempLDA, width = 5, FUN = mean, align = "right", fill = NA),
         PrecipLDA_2d=rollapply(PrecipLDA, width = 2, FUN = mean, align = "right", fill = NA),
         Esporas=0) %>% 
  ungroup() %>% 
  mutate(PLDAc=PrecipLDA*24)%>%
  #dplyr::select(dateOK,ID_ESTACION,TempLDA_5d,PrecipLDA_2d,Esporas,HmLDA,TempLDA,PLDAc) %>%
  group_by(ID_ESTACION) %>% 
  mutate(Goid.pt=ifelse(HmLDA>=75,sapply(TempLDA,FUN="Goid.my.loess.pred"),sapply(TempLDA,FUN="Goid.my.loess.pred")))%>%
  mutate(Goid.Acum=0)%>%
  ungroup() #dplyr::select(dateOK,ID_ESTACION,TempLDA_5d,PrecipLDA_2d,Esporas,HmLDA,TempLDA,PLDAc,Goid.pt,Goid.Acum) 




#-- Parte 2: Creación de los índices de forma recursiva 

df_grupo_estacion <- datosETO.tr %>%
  group_by(ID_ESTACION) %>%
  arrange(dateOK,.by_group=TRUE)

#Se agrupan los datos por estación y ordenados por fecha en cada grupo

df_grupo_estacion <- df_grupo_estacion %>%
  mutate(
    temp5 = ifelse(is.na(TempLDA_5d), 0, TempLDA_5d),
    prep2 = ifelse(is.na(PrecipLDA_2d), 0, PrecipLDA_2d),
    prep.ant = lag(PLDAc),
    temp.ant = lag(TempLDA))

#Solo se mete en el bucle lo imprescindible (ya que es poco eficiente).
#No se puede paralelizar ya que el valor i-esimo depende del que se va a calcular en i-1, por lo tanto han de llevarse a 
#cabo de forma secuencial.
for(i in 5:nrow(df_grupo_estacion)){ #Se empieza en el 5 pq el rollaply dejo 5 NAs
  esp.ant = df_grupo_estacion$Esporas[i-1]
  df_grupo_estacion$Esporas[i] = ifelse(-0.032*(df_grupo_estacion$temp5[i]) + 0.054*48*df_grupo_estacion$prep2[i] + 0.913*esp.ant>=0,
                                        -0.032*(df_grupo_estacion$temp5[i]) + 0.054*48*df_grupo_estacion$prep2[i] + 0.913*esp.ant,0)   
  pt.ant <- df_grupo_estacion$Goid.Acum[i-1]
  if (pt.ant>=100){
    df_grupo_estacion$Goid.Acum[i]<-0
  }
  else if (pt.ant== 0 & df_grupo_estacion$TempLDA[i]>=12 & df_grupo_estacion$TempLDA[i]<=35 & 
           df_grupo_estacion$TempLDA[i-1]>=12 & df_grupo_estacion$TempLDA[i-1]<=35) {
    df_grupo_estacion$Goid.Acum[i] <- df_grupo_estacion$Goid.pt[i]} 
  else if (pt.ant> 0 & pt.ant<99 & 
           df_grupo_estacion$TempLDA[i]>=12 & df_grupo_estacion$TempLDA[i]<=35) {
    df_grupo_estacion$Goid.Acum[i] <- df_grupo_estacion$Goid.pt[i] +pt.ant
  } 
  else {
    df_grupo_estacion$Goid.Acum[i]<-0
  }
}



#Explicación bucle anterior:

#El código primero inicializa un bucle que comienza desde la quinta fila del data frame 
#y luego realiza algunas operaciones en cada fila posterior.
#Para cada fila, el código calcula un nuevo valor para la columna "Esporas" del data frame 
#usando una ecuación lineal basada en los valores de "TempLDA_5d", "PrecipLDA_2d" y el valor 
#anterior de "Esporas". El nuevo valor se establece en el resultado de la ecuación o en 0, dependiendo 
#de si el resultado es mayor o igual a 0.

#El código también calcula un nuevo valor para la columna "Goid.Acum" basado en los valores 
#de "TempLDA", "Goid.pt" y los valores anteriores de "Goid.Acum", "TempLDA" y "PLDAc". El nuevo valor 
#se determina mediante un conjunto de declaraciones if-else basadas en los valores de estas variables.

#El código actualiza las columnas "Esporas" y "Goid.Acum" del data frame para cada fila del bucle.




df_grupo_estacion2 = df_grupo_estacion %>% ungroup() %>%
  mutate(Day=day(ymd(dateOK)),
         Month=month(ymd(dateOK)),
         Year=year(ymd(dateOK)),
         Temporada=dplyr::case_when(
           Month %in% c(10,11,12,1,2) ~ "Parada",
           Month== 3 ~ "Marzo",
           Month==4 | (Month==5 & Day<=15) ~ "Foliacion",
           Month==6 | (Month==5 & Day>15) ~"Floracion")) %>% 
  dplyr::filter(Month %in% c(1,2,3,4,5,6,10,11,12)) %>%
  mutate(CAMPÑ=dplyr::case_when(
    dateOK>="2013-10-01" & dateOK<="2014-06-30" ~ 14,
    dateOK>="2014-10-01" & dateOK<="2015-06-30" ~ 15,
    dateOK>="2015-10-01" & dateOK<="2016-06-30" ~ 16,
    dateOK>="2016-10-01" & dateOK<="2017-06-30" ~ 17,
    dateOK>="2017-10-01" & dateOK<="2018-06-30" ~ 18,
    dateOK>="2018-10-01" & dateOK<="2019-06-30" ~ 19,
    dateOK>="2019-10-01" & dateOK<="2020-06-30" ~ 20,
    dateOK>="2020-10-01" & dateOK<="2021-06-30" ~ 21,
    dateOK>="2021-10-01" & dateOK<="2022-06-30" ~ 22))
#Asignamos las campañas a las fechas correspondientes y creamos los diferentes periodos de la vid dentro cada campaña.


# Agrupar los datos por ID_ESTACION, YEAR y MONTH: ¿porque se hacen los maximos y minimos de las medias???????
datos_agrupados <- df_grupo_estacion2 %>% group_by(ID_ESTACION, CAMPÑ, Temporada)%>% 
  summarise(TEMPDayAvg = mean(TemperatureLocalDayAvg,na.rm=TRUE)-273.15,
            TEMPDaytimeAvg = mean(TemperatureLocalDaytimeAvg,na.rm=TRUE)-273.15,
            TEMPNighttimeAvg = mean(TemperatureLocalNighttimeAvg,na.rm=TRUE)-273.15,
            TEMPDayMax = max(TemperatureLocalDayMax,na.rm=TRUE)-273.15,
            TEMPDaytimeMax = max(TemperatureLocalDaytimeMax,na.rm=TRUE)-273.15,
            TEMPNighttimeMax = max(TemperatureLocalNighttimeMax,na.rm=TRUE)-273.15,
            TEMPDayMin = min(TemperatureLocalDayMin,na.rm=TRUE)-273.15,
            TEMPDaytimeMin = min(TemperatureLocalDaytimeMin,na.rm=TRUE)-273.15,
            TEMPNighttimeMin = min(TemperatureLocalNighttimeMin,na.rm=TRUE)-273.15,
            
            WindDayAvg = mean(WindSpeedLocalDayAvg,na.rm=TRUE),
            WindDaytimeAvg = mean(WindSpeedLocalDaytimeAvg,na.rm=TRUE),
            WindNighttimeAvg = mean(WindSpeedLocalNighttimeAvg,na.rm=TRUE),
            WindDayMax = max(WindSpeedLocalDayMax,na.rm=TRUE),
            WindDaytimeMax = max(WindSpeedLocalDaytimeMax,na.rm=TRUE),
            WindNighttimeMax = max(WindSpeedLocalNighttimeMax,na.rm=TRUE),
            WindDayMin = min(WindSpeedLocalDayMin,na.rm=TRUE),
            WindDaytimeMin = min(WindSpeedLocalDaytimeMin,na.rm=TRUE),
            WindNighttimeMin = min(WindSpeedLocalNighttimeMin,na.rm=TRUE),
            
            RadDayAvg = mean(UVIndexLocalDayAvg,na.rm=TRUE),
            RadDaytimeAvg = mean(UVIndexLocalDaytimeAvg,na.rm=TRUE),
            RadNighttimeAvg = mean(UVIndexLocalNighttimeAvg,na.rm=TRUE),
            RadDayMax = max(UVIndexLocalDayMax,na.rm=TRUE),
            RadDaytimeMax = max(UVIndexLocalDaytimeMax,na.rm=TRUE),
            RadNighttimeMax = max(UVIndexLocalNighttimeMax,na.rm=TRUE),
            RadDayMin = min(UVIndexLocalDayMin,na.rm=TRUE),
            RadDaytimeMin = min(UVIndexLocalDaytimeMin,na.rm=TRUE),
            RadNighttimeMin = min(UVIndexLocalNighttimeMin,na.rm=TRUE),
            
            PreDayAvg = mean(PrecipAmountLocalDayAvg,na.rm=TRUE),
            PreDaytimeAvg = mean(PrecipAmountLocalDaytimeAvg,na.rm=TRUE),
            PreNighttimeAvg = mean(PrecipAmountLocalNighttimeAvg,na.rm=TRUE),
            PreDayMax = max(PrecipAmountLocalDayMax,na.rm=TRUE),
            PreDaytimeMax = max(PrecipAmountLocalDaytimeMax,na.rm=TRUE),
            PreNighttimeMax = max(PrecipAmountLocalNighttimeMax,na.rm=TRUE),
            PreDayMin = min(PrecipAmountLocalDayMin,na.rm=TRUE),
            PreDaytimeMin = min(PrecipAmountLocalDaytimeMin,na.rm=TRUE),
            PreNighttimeMin = min(PrecipAmountLocalNighttimeMin,na.rm=TRUE),
            
            HumDayAvg = mean(RelativeHumidityLocalDayAvg,na.rm=TRUE),
            HumDaytimeAvg = mean(RelativeHumidityLocalDaytimeAvg,na.rm=TRUE),
            HumNighttimeAvg = mean(RelativeHumidityLocalNighttimeAvg,na.rm=TRUE),
            HumDayMax = max(RelativeHumidityLocalDayMax,na.rm=TRUE),
            HumDaytimeMax = max(RelativeHumidityLocalDaytimeMax,na.rm=TRUE),
            HumNighttimeMax = max(RelativeHumidityLocalNighttimeMax,na.rm=TRUE),
            HumDayMin = min(RelativeHumidityLocalDayMin,na.rm=TRUE),
            HumDaytimeMin = min(RelativeHumidityLocalDaytimeMin,na.rm=TRUE),
            HumNighttimeMin = min(RelativeHumidityLocalNighttimeMin,na.rm=TRUE),
            Goid=sum(Goid.Acum>70),
            Espo=sum(Esporas))

#Se han calculado las medias maximos y minimos de las distintas variables que se consideraron de interés 
# En cada una de las etapas del desarrollo de la vid


datos_wide <- datos_agrupados %>% pivot_wider(names_from = Temporada, values_from = -c(ID_ESTACION,CAMPÑ,Temporada)) 
str(datos_wide)

aux<-datos_wide 



#Añade a datos_wide la suma de las variables Goid y Espo a lo largo de las fases de desarollo de la vid
datos_wide2<- aux %>% mutate(Goid=sum(Goid_Foliacion,Goid_Floracion,na.rm=TRUE),
                             Espo=sum(Espo_Parada,Espo_Marzo,Espo_Foliacion,Espo_Floracion,na.rm = TRUE)) %>% dplyr::select(-c(
                               Espo_Parada,Espo_Marzo,Espo_Foliacion,Espo_Floracion,
                               Goid_Parada,Goid_Marzo,Goid_Foliacion,Goid_Floracion))



MAIN.tr = datosUH %>% mutate(CAMPÑ=CAMPAÑA) %>% group_by(CAMPÑ, ID_ESTACION) %>% summarise(PRODUCCION=mean(PRODUCCION,na.rm=TRUE))%>% 
  left_join(datos_wide2,by=c("CAMPÑ","ID_ESTACION")) %>% arrange(ID_ESTACION,CAMPÑ)

#MAIN.tr da la media de la produccion por estaciones



#A los datos con la información acerca de los cultivos en cada finca se les hace un leftjoin con los datos_wide2 que contienen los datos meteorológicos resumidos de interés

MAIN.tr2<- MAIN.tr %>% mutate(PROD_1 = 0,PROD_1na = 0) #A este a mayores se le introducen las variables que hacen referencia a la producción del año anterior

for (i in unique(MAIN.tr2$ID_ESTACION)){
  for (j in MAIN.tr2$CAMPÑ[MAIN.tr2$ID_ESTACION==i]){
    if ((j-1) %in% MAIN.tr2$CAMPÑ[MAIN.tr2$ID_ESTACION==i]){
      MAIN.tr2[MAIN.tr2$ID_ESTACION==i & MAIN.tr2$CAMPÑ==j,"PROD_1"]<- MAIN.tr2[MAIN.tr2$ID_ESTACION==i & MAIN.tr2$CAMPÑ==j-1,"PRODUCCION"]
    }
    else{
      MAIN.tr2[MAIN.tr2$ID_ESTACION==i & MAIN.tr2$CAMPÑ==j,"PROD_1na"]<-1
    }
  }
}


#El código crea una nueva variable llamada PROD_1 en el data frame MAIN.tr2 y la inicializa en cero. Luego, crea otra variable llamada PROD_1na y también la inicializa en cero.
#Ambas variables se usarán más adelante para hacer referencia a la producción del año anterior.

#El código utiliza un bucle for para recorrer cada estación de manera única y otro bucle for para recorrer cada año (CAMPÑ) dentro de cada estación.
#Dentro del segundo bucle for, el código verifica si el año anterior (j-1) está presente en los datos de producción de la misma estación (ID_ESTACION) y si es así,
#se asigna el valor de producción correspondiente a la variable PROD_1 en la fila correspondiente en el data frame. Si el año anterior no está presente en los datos, 
#se asigna el valor 1 a la variable PROD_1na en la fila correspondiente en el data frame. Al final de la ejecución del código, el data frame MAIN.tr2 contendrá las variables 
#PROD_1 y PROD_1na que hacen referencia a la producción del año anterior para cada fila correspondiente a una estación y un año determinado.

#Cambiamos los nombres de las variables de producción añadiéndoles un "w" para indicar que se trata en realidad de producciones agregadas por estaciones meteorológicas.
MAIN.tr3<-MAIN.tr2%>%rename(wPRODUCCION=PRODUCCION,wPROD_1=PROD_1,wPROD_1na=PROD_1na)%>%mutate(ID_ESTACION=as.factor(ID_ESTACION))

#-------------------------------------------------

datos.At <- datosUH %>%
  mutate(aux = ifelse(is.na(ALTITUD), "999", ALTITUD)) %>%
  mutate(contiene_guion = grepl("-", aux, fixed = TRUE)) %>%
  mutate(ALT_med = ifelse(contiene_guion,
                          (as.numeric(substr(aux, start = 5, stop = nchar(aux)))+
                             as.numeric(substr(aux, start = 1, stop = 3)))/2
                          ,as.numeric(aux)))%>% 
  mutate(ALT_med2 = ifelse(is.na(ALTITUD), NA, ALT_med))

#datos.At se emplea para calcular la variable de altura media como numeric. Además se crea la variable ALT_med2 en la que
# los NAs aparecen como tal, al contrario que en aux en la que aparecen como 999 para que se puedan llevar a cabo las operaciones posteriores

datos = datosUH

#En datos2 pasamos todo a factor:
datos2 = datosUH %>% mutate(ALTITUD = datos.At$ALT_med2) %>% mutate(TIPO = as.factor(TIPO)) %>% mutate(MODO = as.factor(MODO)) %>%
  mutate(COLOR = as.factor(COLOR)) %>% mutate(ID_FINCA = as.factor(ID_FINCA))%>% mutate(ID_ESTACION = as.factor(ID_ESTACION)) %>%
  mutate(ID_ZONA = as.factor(ID_ZONA)) %>% mutate(VARIEDAD = as.factor(VARIEDAD)) 
#Finalmente pasamos todo a factor




#FINALMENTE AÑADIMOS EL ID_UNICO Y LAS PRODUCCIONES DE AÑOS ANTERIORES JUNTO A SUS INDICADORES DE NAs

PROD.ANT = function(v, col.CAMPÑ = 12, diff = 0, total.data = ProdTotal.C) { #col.CAMPÑ es el numero de la columna CAMPÑ
  #diff = el diferencial para que las columnas se correspondan con los años adecuados
  #si se introducen c columnas diff = c
  
  PROD_1 = ifelse(is.na(v[as.numeric(v[col.CAMPÑ]) + 1 + diff]), 0, as.numeric(v[as.numeric(v[col.CAMPÑ]) + 1 + diff]))
  PROD_2 = ifelse(is.na(v[as.numeric(v[col.CAMPÑ]) + diff]), 0, as.numeric(v[as.numeric(v[col.CAMPÑ]) + diff]))
  PROD_mean = mean(as.numeric(v[(14 + diff):(as.numeric(v[col.CAMPÑ]) + 1 + diff)]), na.rm = T)
  return(c(PROD_1, PROD_2, PROD_mean))
}


final.fun = function(x, col.CAMPÑ = 12, diff = 0, total.data = ProdTotal.C){ #col.CAMPÑ es el numero de la columna CAMPÑ
  #diff = el diferencial para que las columnas se correspondan con los años adecuados
  #si se introducen c columnas diff = c
  #Nuevas variables:
  x = x %>% mutate(CAMPÑ=CAMPAÑA,ID_UNICO=paste(ID_FINCA,VARIEDAD,TIPO,MODO,sep="-"))
  
  #Dataset wide:
  x_wide_aux = x %>% pivot_wider(id_cols = ID_UNICO, names_from = CAMPÑ, values_from = PRODUCCION) %>% 
    mutate('12' = NA, '13' = NA) %>% relocate(after = c(1, 11, 12, 2:10))
  colnames(x_wide_aux) = c('ID_UNICO', as.character(12:22))
  
  #Juntamos el dataset normal con el wide
  x_aux = x %>% left_join(x = x, y = x_wide_aux, by = 'ID_UNICO', relationship = 'many-to-one')
  
  #Creamos las variables de PROD_1, PROD_2 y PROD_mean:
  result = t(apply(x_aux, 1 , PROD.ANT, col.CAMPÑ = col.CAMPÑ, diff = diff, total.data = total.data))
  x_aux = cbind(x_aux, PROD_1 = result[,1], PROD_2 = result[,2], PROD_mean = result[,3])
  
  #Creamos las indicadoras de NAs y modificamos mean:
  x_aux$PROD_1na = ifelse(x_aux$PROD_1 == 0, 1, 0)
  x_aux$PROD_2na = ifelse(x_aux$PROD_2 == 0, 1, 0)
  x_aux$PROD_MEAN_NA = ifelse(is.na(x_aux$PROD_mean), 1, 0)
  x_aux$PROD_mean = ifelse(is.na(x_aux$PROD_mean), 0, x_aux$PROD_mean)
  
  #Eliminamos las columnas de wide:
  x_aux = x_aux[, !names(x_aux) %in% as.character(12:22)]
  return(x_aux)
}

datos_finales = final.fun(datos2, col.CAMPÑ = 12, diff = 0, total.data = ProdTotal.C)

datos_finales = datos_finales %>% mutate(PROD_MEAN_NA = as.factor(PROD_MEAN_NA), PROD_2na = as.factor(PROD_2na), PROD_1na = as.factor(PROD_1na))

#En resumen, este código indica la producción de los años anteriores en las columnas correspondientes y pone un valor de "1" en las 
#columnas indicadoras si no hay valores disponibles para los años anteriores. Además, se calcula la proporción dentro del mismo año de dicha produccion

#También se crea una variable que sea la media de la producción de la finca a lo largo de los años (al menos de los años en los cuales produce)
#Y en las fincas para las que no hay valores previos de la predicción, se pone una variable indicadora!!! (De esta forma reducimos las fincas cuyo output depende de la indicadora)








########################################################################################################
######################## IMPUTACIÓN DE VALORES FALTANTES ###############################################
########################################################################################################

#Un primer método de imputación será por randomforest. Este lo llevaremos a cabo separando los años 14-21 y el año 22 ( ya que en este último
#no se pueden emplear las producciones para imputar, pq aun no las tenemos).
#En primer lugar se imputarán las altitudes de las fincas ya que algunas de las zonas tienen todas las fincas con la altitud faltante.
#En segundo lugar, y empleando también los valores imputados de las altitudes, generamos valores para las fincas de superficie 0.
#Este método no es el que produce la mejor imputación (no introduce variabilidad en los datos), no obstante, menos costoso computacionalmente
#que la imputación del paquete MICE (que al generar varios datasets y combinar las respuestas, no sería compatible con aplicar el modelo xgboost planteado
#Pues tardaría demasiado en ejecutarse)

#Posteriormente, llevaremos a cabo una imputacion con el paquete MICE de la misma forma y usaremos los datos para
#ajustar un modelo random forest, de forma que el tiempo extra que consuma la imputación lo ahorremos en la creacion del modelo
#generando así un algoritmo igualmente eficiente

#Asignamos NAs a los valores de superficie = 0 de las campañas que hemos mencionado:
x = which( (datos_finales$CAMPAÑA == 20 & datos_finales$SUPERFICIE == 0) | (datos_finales$CAMPAÑA == 21 & datos_finales$SUPERFICIE == 0) | (datos_finales$CAMPAÑA == 22 & datos_finales$SUPERFICIE == 0) )
n = length(x)
for(i in 1:n){datos_finales[x[i],10] = NA}


#################################### PAQUETE MISS FOREST ###############################################

##----------------------------------------- ALTITUD -----------------------------------------##

#### AÑOS 2014 - 2021:

datos_finales_alt_14_21 = datos_finales[which(datos_finales$CAMPAÑA != 22),]

#En primer lugar imputamos los valores de las alturas para estos años, AQUI PODEMOS EMPLEAR LA PRODUCCION:

#Para computar en paralelo (más eficiente):
numCores = detectCores()
if(numCores > 13){numCores = 13} # The number of parallel cores should not exceed the number of variables (p=13)
registerDoParallel(cores = numCores)
set.seed(1234)
datos_imput_alt_14_21 = missForest(datos_finales_alt_14_21[,-c(1,2,3,10,12,13)], maxiter = 1000, ntree = 1000, parallelize = 'forests')$ximp
stopImplicitCluster()

#Ahora agregaremos los valores imputados al dataset:
x1 = as.numeric(rownames(datos_imput_alt_14_21[is.na(datos_finales_alt_14_21$ALTITUD),]))

#Asignación
datos_finales_alt_14_21[x1,5] = as.numeric(datos_imput_alt_14_21$ALTITUD[x1])

#Finalmente comprobaremos que las alturas de las fincas del mismo ID_ZONA, son similares, ya que en general
#si tienen el mismo ID_zona tienen la misma altitud!!
ZONAS = unique(as.numeric(as.character(datos_finales_alt_14_21[x1,3])))

# Calcular la media de la ALTITUD para cada ID_ZONA en el vector ZONAS
altitud_media = datos_finales_alt_14_21 %>%
  filter(ID_ZONA %in% ZONAS) %>%
  group_by(ID_ZONA) %>%
  summarise(media_altitud = mean(ALTITUD, na.rm = TRUE))




#### AÑO 2022:


#Añadimos los valores de 2022
datos_finales_altitud = rbind(datos_finales_alt_14_21, datos_finales[(nrow(datos_finales_alt_14_21)+1):nrow(datos_finales),])

#Imputamos los valores de las alturas para este año y a partir de los valores imputados en años anteriores:
#En este caso no empleamos la produccion porque no la tenemos
#Para computar en paralelo (más eficiente):
numCores = detectCores()
if(numCores > 12){numCores = 12} # The number of parallel cores should not exceed the number of variables (p=12)
registerDoParallel(cores = numCores)
set.seed(1234)
datos_imput_alt_14_22 = missForest(datos_finales_altitud[,-c(1,2,3,10,11,12,13)], maxiter = 1000, ntree = 1000, parallelize = 'forests')$ximp
stopImplicitCluster()


#Ahora añadiremos los valores imputados al dataset:
x1 = as.numeric(rownames(datos_imput_alt_14_22[is.na(datos_finales_altitud$ALTITUD),]))

#Asignación
datos_finales_altitud[x1,5] = as.numeric(datos_imput_alt_14_22$ALTITUD[x1])

#Finalmente comprobaremos que las alturas de las fincas del mismo ID_ZONA, son similares, ya que en general
#si tienen el mismo ID_zona tienen la misma altitud!!
ZONAS = unique(as.numeric(as.character(datos_finales_altitud[x1,3])))
#Son zonas repetidas de las del dataset del 14_21

# Calcular la media de la ALTITUD para cada ID_ZONA en el vector ZONAS
altitud_media = datos_finales_altitud %>%
  filter(ID_ZONA %in% ZONAS) %>%
  group_by(ID_ZONA) %>%
  summarise(media_altitud = mean(ALTITUD, na.rm = TRUE))

#Devuelve las mismas medias que esas mismas zonas en la anterior asignacion





##----------------------------------------- SUPERFICIE -----------------------------------------##


#### AÑOS 2020 - 2021: Ya que no hay superficies en el resto de años:
datos_finales_20_21 = datos_finales_altitud[which(datos_finales_altitud$CAMPAÑA %in% c(20,21)),]

#En primer lugar imputamos los valores de las superficies para estos años, AQUI PODEMOS EMPLEAR LA PRODUCCION:

#Para computar en paralelo (más eficiente):
numCores = detectCores()
if(numCores > 14){numCores = 14} # The number of parallel cores should not exceed the number of variables (p=14)
registerDoParallel(cores = numCores)
set.seed(1234)
datos_imput_sup_20_21 = missForest(datos_finales_20_21[,-c(1,2,3,12,13)], maxiter = 1000, ntree = 1000, parallelize = 'forests')$ximp
stopImplicitCluster()

#Ahora agregaremos los valores imputados al dataset:
x1 = as.numeric(rownames(datos_imput_sup_20_21[is.na(datos_finales_20_21$SUPERFICIE),]))
dif = as.numeric(rownames(datos_imput_sup_20_21[1,])) - 1
#Asignación
datos_finales_altitud$SUPERFICIE[x1] = as.numeric(datos_imput_sup_20_21[x1 - dif,7])

#Veamos que valores se han imputado
datos_imput_sup_20_21$SUPERFICIE[is.na(datos_finales_20_21$SUPERFICIE)]



#### AÑO 2022: AQUI NO PODEMOS EMPLEAR LAS PRODUCCIONES PARA LA IMPUTACION

datos_finales_20_22 = datos_finales_altitud[which(datos_finales_altitud$CAMPAÑA %in% c(20,21,22)),]

#Para computar en paralelo (más eficiente):
numCores = detectCores()
if(numCores > 13){numCores = 13} # The number of parallel cores should not exceed the number of variables (p=13)
registerDoParallel(cores = numCores)
set.seed(1234)
datos_imput_sup_20_22 = missForest(datos_finales_20_22[,-c(1,2,3,11,12,13)], maxiter = 1000, ntree = 1000, parallelize = 'forests')$ximp
stopImplicitCluster()

#Ahora agregaremos los valores imputados al dataset:
x1 = as.numeric(rownames(datos_imput_sup_20_22[is.na(datos_finales_20_22$SUPERFICIE),]))
dif = as.numeric(rownames(datos_imput_sup_20_22[1,])) - 1
#Asignación
datos_finales_altitud$SUPERFICIE[x1] = as.numeric(datos_imput_sup_20_22[x1 - dif,7])

#Veamos que valores se han imputado
datos_imput_sup_20_22$SUPERFICIE[is.na(datos_finales_20_22$SUPERFICIE)]

#SE HA COMPLETADO LA IMPUTACION:

datos_finales_miss_forest = datos_finales_altitud

#Unimos datos_finales y MAIN.tr3 utilizando las columnas "CAMPÑ" e "ID_ESTACION" como claves de unión y luego filtra las filas en las que la columna "CAMPÑ" 
#es mayor que 15. El resultado se guarda en un nuevo conjunto de datos llamado datos_finales2.

datos_finales2 = datos_finales_miss_forest %>% left_join(x=datos_finales_miss_forest,y=MAIN.tr3,by=c("CAMPÑ","ID_ESTACION"))





datos_C_16_19 = datos_finales2 %>% filter(CAMPAÑA %in% 16:19)
datos_C_20_22 = datos_finales2 %>% filter(CAMPAÑA %in% 20:22)


###############################################################################
###############################################################################
#                            MODELOS DE PREDICCIÓN
###############################################################################
###############################################################################


# 2 XGBoost encadenados, 1 para 16-19 y otro para 20-21


datos.train_16_19 = datos_C_16_19 %>% 
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO"))%>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))])

# Creación de los K-folds: 1 con cada uno de los cuatro años 16, 17, 18 y 19. --

Fold1<-datos_C_16_19 %>% filter(CAMPÑ==16) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO"))%>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))])

Fold2<-datos_C_16_19 %>% filter(CAMPÑ==17) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO"))%>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))])


Fold3<-datos_C_16_19 %>% filter(CAMPÑ==18) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO"))%>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))])


Fold4<-datos_C_16_19 %>% filter(CAMPÑ==19) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO"))%>%
  dplyr::select(-names(datos_C_16_19)[grep("Daytime|Nighttime",names(datos_C_16_19))])

Folds <- list(as.integer(rownames(Fold1)),
              as.integer(rownames(Fold2)),
              as.integer(rownames(Fold3)),
              as.integer(rownames(Fold4)))


#Datos de test:
datos.test_20_21 = datos_C_20_22 %>% filter(CAMPÑ %in% 20:21) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO"))%>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))])


datos.test_20 = datos_C_20_22 %>% filter(CAMPÑ == 20) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO"))%>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))])


datos.test_21 = datos_C_20_22 %>% filter(CAMPÑ == 21) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO"))%>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))])





################################################################################
#Para evitar problemas con la computacion en paralelo

unregister_dopar <- function() {
  env <- foreach:::.foreachGlobals
  rm(list=ls(name=env), pos=env)
}

unregister_dopar()
################################################################################







# Búsqueda de hiperparámetros con Optimización Bayesiana -----------------------


#La ejecución de esto puede llevar tiempo y se realizó varias veces por distintos grids iniciales
#hasta encontrar unos parámetros adecuados. En este script se ajusta directamente los XGBoost con
#los parámetros elegidos

#Si se quiere emplear la búsqueda que se llevó a cabo bastaría con descomentar las siguientes líneas.
#No obstante, esta búsqueda llevó entorno a 3h en un ordenador con procesador intelCore i7 - 12700H
#Además los parámetros iter.k e iter.n están asignados teniendo en cuenta el número de threads del procesador
#Por lo que con otros procesadores esto podría llevar aun más tiempo
#De ejecutar dicha función, se recomienda hacerlo en R, en lugar de Rstudio, ya que a veces crashea.


#scoringFunction <- function(max_depth, min_child_weight, subsample,eta) {
#  
#  tune_grid <- expand.grid(
#    nrounds = seq(from = 100, to = 1500, by = 25), 
#    eta = eta,
#    max_depth = max_depth,
#    gamma = 0,
#    colsample_bytree = 1,
#    min_child_weight = min_child_weight,
#    subsample = subsample
#  )
#  
#  ctrl <- trainControl(
#    method = "repeatedcv",
#    number = 4,
#   repeats = 1,
#    index = Folds,
#    verboseIter = FALSE
#  )
#  
#  xgbcv <- caret::train(form = PRODUCCION ~ ., data = datos.train_16_19,
#                        trControl = ctrl,
#                        tuneGrid = tune_grid,
#                        method = "xgbTree",
#                        verbose = FALSE,
#                        tree_method="exact") 
#
#  
#  return(list( Score = -min(xgbcv$results$RMSE), nrounds = xgbcv$bestTune$nrounds))
#}
#
#
#bounds <- list( max_depth = c(1L, 15L),
#                min_child_weight = c(1, 100), 
#                subsample = c(0.25, 1),
#                eta = c(0.0001, 0.5))
#
#
#initGrid<-expand.grid(max_depth = c(9L, 15L),
#                      min_child_weight = c(1, 2.65, 5), 
#                      subsample = c(0.25, 0.34, 0.66, 1),
#                      eta = c(0.0075,0.015,0.075, 0.2))




#pct = proc.time()
#numCores = detectCores()
#cl = makeCluster(numCores)
#registerDoParallel(cl, cores = numCores)
#
#clusterExport(cl, c('Folds', 'datos.train_16_19', 'scoringFunction', 'bounds','datos.test_20_21'))
#clusterEvalQ(cl, {
#  library(hrbrthemes)
#  library(caret)
#  library(ParBayesianOptimization)
#  library(doParallel)
#})
#
#set.seed(1000)
#bayes_out <- bayesOpt(FUN = scoringFunction, bounds = bounds, 
#                      initGrid=initGrid, iters.n = 80,iters.k=40, parallel=TRUE)
#
#stopCluster(cl)
#
#proc.time() - pct
#
#bayes_out$scoreSummary
#bestPars = getBestPars(bayes_out)
#df1 = data.frame(bestPars, nrounds = bayes_out$scoreSummary[which(bayes_out$scoreSummary$subsample == bestPars$subsample & 
#                                                                    bayes_out$scoreSummary$eta == bestPars$eta &
#                                                                    bayes_out$scoreSummary$max_depth == bestPars$max_depth &
#                                                                   bayes_out$scoreSummary$min_child_weight == bestPars$min_child_weight),]$nrounds, 
#                Score = bayes_out$scoreSummary[which(bayes_out$scoreSummary$subsample == bestPars$subsample & 
#                                                       bayes_out$scoreSummary$eta == bestPars$eta &
#                                                       bayes_out$scoreSummary$max_depth == bestPars$max_depth &
#                                                        bayes_out$scoreSummary$min_child_weight == bestPars$min_child_weight),]$Score)
#
#
#df1


################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################









#------------------------------------------------------------------------------
# 1er XGBoost
#------------------------------------------------------------------------------


#Parámetros escogidos:
#     eta         max_depth   min_child_weight   subsample      nrounds
# 0.0075             9               1             0.25           400

tune_grid <- expand.grid(
  nrounds = 400,
  eta = 0.0075,
  max_depth = 9,
  gamma = 0,
  colsample_bytree = 1,
  min_child_weight = 1,
  subsample = 0.25
)

ctrl <- trainControl(
  method = "repeatedcv",
  number = 4,
  repeats = 1,
  index = Folds,
  verboseIter = FALSE,
  allowParallel = T
)


set.seed(16032023)
pct = proc.time()
xgbcv <- caret::train(form = PRODUCCION ~ ., data = datos.train_16_19,
                      trControl = ctrl,
                      tuneGrid = tune_grid,
                      method = "xgbTree",
                      verbose = FALSE,
                      metric = 'RMSE',
                      tree_method="exact")
proc.time() - pct

y.xgbcv.20_21 = predict(xgbcv, newdata = datos.test_20_21)
y.xgbcv.20 = predict(xgbcv, newdata = datos.test_20)
y.xgbcv.21 = predict(xgbcv, newdata = datos.test_21)

y.prod.20_21 = datos.test_20_21$PRODUCCION
y.prod.20 = datos.test_20$PRODUCCION
y.prod.21 = datos.test_21$PRODUCCION

cat("---- XGBoost Tree 1 KFOLDS ----- \n", min(xgbcv$results$RMSE) ) #Kfolds
cat("---- XGBoost Tree 1 RMSE 20-21 ----- \n", sqrt(sum((y.xgbcv.20_21 - y.prod.20_21)^2)/length(y.prod.20_21)) ) #RMSE 20-21
cat("---- XGBoost Tree 1 RMSE 20 ----- \n", sqrt(sum((y.xgbcv.20 - y.prod.20)^2)/length(y.prod.20)) ) #RMSE 20
cat("---- XGBoost Tree 1 RMSE 21 ----- \n", sqrt(sum((y.xgbcv.21 - y.prod.21)^2)/length(y.prod.21)) ) #RMSE 21

#Para la interpretacion
ctrl1 = ctrl
tune_grid1 = tune_grid

#------------------------------------------------------------------------------
# 2do XGBoost
#------------------------------------------------------------------------------



#Separamos en training 20_21 Y testing 22:

datos.20_21 = datos_C_20_22 %>% filter(CAMPÑ %in% 20:21) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" )) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 

datos.20_21$xgboost = predict(xgbcv, newdata = datos.20_21)


datos.22 = datos_C_20_22 %>% filter(CAMPÑ == 22) %>%
  dplyr::select(-c("CAMPAÑA","CAMPÑ","ID_FINCA", "ID_ESTACION","wPRODUCCION", "ID_UNICO" )) %>%
  dplyr::select(-names(datos_C_20_22)[grep("Daytime|Nighttime",names(datos_C_20_22))]) 

datos.22$xgboost = predict(xgbcv, newdata = datos.22)





################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################



# Búsqueda de hiperparámetros con Optimización Bayesiana -----------------------


#La ejecución de esto puede llevar tiempo y se realizó varias veces por distintos grids iniciales
#hasta encontrar unos parámetros adecuados. En este script se ajusta directamente los XGBoost con
#los parámetros elegidos

#Si se quiere emplear la búsqueda que se llevó a cabo bastaría con descomentar las siguientes líneas.
#No obstante, esta búsqueda llevó entorno a 3h en un ordenador con procesador intelCore i7 - 12700H
#Además los parámetros iter.k e iter.n están asignados teniendo en cuenta el número de threads del procesador
#Por lo que con otros procesadores esto podría llevar aun más tiempo
#De ejecutar dicha función, se recomienda hacerlo en R, en lugar de Rstudio, ya que a veces crashea.




#scoringFunction <- function(max_depth, min_child_weight, subsample,eta) {
#  
#  tune_grid <- expand.grid(
#    nrounds = seq(from = 100, to = 1500, by = 25), 
#    eta = eta,
#    max_depth = max_depth,
#    gamma = 0,
#    colsample_bytree = 1,
#    min_child_weight = min_child_weight,
#    subsample = subsample
#  )
#  
#  ctrl <- trainControl(
#    method = "cv",
#    verboseIter = FALSE,
#    allowParallel = TRUE
#  )
#  
#  xgbcv <- caret::train(form = PRODUCCION ~ ., data = datos.20_21,
#                        trControl = ctrl,
#                        tuneGrid = tune_grid,
#                        method = "xgbTree",
#                        verbose = FALSE,
#                        metric = 'RMSE',
#                        tree_method="exact")  
#  
#  return(list( Score = -min(xgbcv$results$RMSE), nrounds = xgbcv$bestTune$nrounds))
#}
#
#
#bounds <- list( max_depth = c(1L, 15L),
#                min_child_weight = c(1, 100), 
#                subsample = c(0.25, 1),
#                eta = c(0.0001, 0.5))
#
#
#initGrid<-expand.grid(max_depth = c(9L, 15L),
#                      min_child_weight = c(1, 2.65, 5), 
#                      subsample = c(0.25, 0.34, 0.61, 1),
#                      eta = c(0.0075,0.015,0.02, 0.1))
#
#
#
#
#pct = proc.time()
#numCores = detectCores()
#cl = makeCluster(numCores)
#registerDoParallel(cl, cores = numCores)
#
#clusterExport(cl, c('datos.20_21', 'scoringFunction', 'bounds'))
#clusterEvalQ(cl, {
#  library(hrbrthemes)
#  library(caret)
#  library(ParBayesianOptimization)
#  library(doParallel)
#})
#
#set.seed(1000)
#bayes_out <- bayesOpt(FUN = scoringFunction, bounds = bounds, 
#                      initGrid=initGrid, iters.n = 80,iters.k=40, parallel=TRUE)
#
#stopCluster(cl)
#proc.time() - pct
#
#bayes_out$scoreSummary
#bestPars = getBestPars(bayes_out)
#df2 = data.frame(bestPars, nrounds = bayes_out$scoreSummary[which(bayes_out$scoreSummary$subsample == bestPars$subsample & 
#                                                                    bayes_out$scoreSummary$eta == bestPars$eta &
#                                                                    bayes_out$scoreSummary$max_depth == bestPars$max_depth &
#                                                                    bayes_out$scoreSummary$min_child_weight == bestPars$min_child_weight),]$nrounds, 
#                 Score = bayes_out$scoreSummary[which(bayes_out$scoreSummary$subsample == bestPars$subsample & 
#                                                        bayes_out$scoreSummary$eta == bestPars$eta &
#                                                        bayes_out$scoreSummary$max_depth == bestPars$max_depth &
#                                                        bayes_out$scoreSummary$min_child_weight == bestPars$min_child_weight),]$Score)
#
#
#df2




################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################



#---------------------------------------------------------------

#Parámetros escogidos:
# eta             max_depth   min_child_weight      subsample     nrounds
# 0.02                9              1                0.61          275



tune_grid <- expand.grid(
  nrounds = 275,
  eta = 0.02,
  max_depth = 9,
  gamma = 0,
  colsample_bytree = 1,
  min_child_weight = 1,
  subsample = 0.61
)

ctrl <- trainControl(
  method = "cv",
  verboseIter = FALSE,
  allowParallel = TRUE
)

set.seed(15032023)
pct = proc.time()

xgbFinal = caret::train(form = PRODUCCION ~ ., data = datos.20_21,
                        trControl = ctrl,
                        tuneGrid = tune_grid,
                        method = "xgbTree",
                        verbose = FALSE,
                        metric = 'RMSE',
                        tree_method="exact")

proc.time() - pct

xgbFinal
yFinal = predict(xgbFinal, newdata = datos.22)

windows();hist(yFinal) #Da algunas menores que 0, lo cual no tiene sentido, ya que no hay ninguna produccion menor que 0 !!


datos.pred = datos_C_20_22 %>% filter(CAMPÑ == 22)
datos.pred$PREDICCION = yFinal

ENVIAR<-datos.pred %>% dplyr::select(ID_FINCA,VARIEDAD,MODO,TIPO,COLOR,SUPERFICIE,PREDICCION)



ENVIAR2<- ENVIAR %>% 
  arrange(ID_FINCA,VARIEDAD,MODO,TIPO,COLOR,SUPERFICIE)

ENVIAR_FINAL<-ENVIAR2 %>% mutate(VARIEDAD=ifelse(nchar(as.character(VARIEDAD))==2,
                                                 as.character(VARIEDAD),paste0(0,VARIEDAD)),
                                 TIPO=as.character(TIPO),
                                 PREDICCION=round(PREDICCION,2))

ENVIAR_FINAL

write.table(ENVIAR_FINAL,"R-Commanders.txt",row.names=FALSE,col.names=FALSE,sep="|")






























##############################################################################################################
############ INTERPRETACION DEL PRIMER MODELO ################################################################
##############################################################################################################


library(DALEX)
exp <- explain(model = xgbcv, data = datos.train_16_19, y = datos.train_16_19$PRODUCCION, type = "xgboost")
shap_values <- predict_parts(exp, datos.test_20_21, type = "shap")

# visualizar los valores SHAP en un gráfico de barras
plot(shap_values, type = "bar") #?


#Se puede ver la importancia en función de los valores de Shapley que de acuerdo con el
#Artículo citado en la memoria son unos de los mejores medidores de la importancia
#de las variables, pues son consistentes y precisos.


################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################



#Para ver la distribución de los residuos separaremos en test y train:

set.seed(15032023)

xgbcv.perf = caret::train(form = PRODUCCION ~ ., data = datos.train_16_19,
                             trControl = ctrl1,
                             tuneGrid = tune_grid1,
                             method = "xgbTree",
                             verbose = FALSE,
                             metric = 'RMSE',
                             tree_method="exact")

exp_xgboost = explain(xgbcv.perf, data = datos.test_20_21, y = datos.test_20_21$PRODUCCION, label = "XGBoost", type = "regression")
perf_xgboost = model_performance(exp_xgboost)

plot(perf_xgboost)



#No obstante, para explicar los resultados obtenidos creamos el explainer del modelo empleado:
ID = 48626
exp_xgboost = explain(xgbcv, data = datos.train_16_19, y = datos.train_16_19$PRODUCCION, label = "XGBoost", type = "regression")
perf_xgboost = model_performance(exp_xgboost)

datos.shiny = datos_C_20_22 %>% filter(CAMPÑ %in% 20:21)
FINCA <- datos.test_20_21[which(datos.shiny$ID_FINCA == ID),]

cp_xgboost <- predict_profile(exp_xgboost, FINCA)


#Veamos como influyen las distintas variables:
plot(cp_xgboost, variables = c("PROD_1", "PROD_2")) #Prod_1 influye más que Prod_2
plot(cp_xgboost, variables = c("PROD_mean"))


#En este caso, el hecho de que las gráficas condicionales sean diferentes a las otras se debe
#mayormente a que existe una alta correlación entre las variables

#Ojo estas gráficas cambian si no se les pone una semilla!

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'ALTITUD', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'ALTITUD', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'ALTITUD', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_1', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_1', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_1', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_2', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_2', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_2', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_mean', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_mean', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_mean', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'HumDayMin_Marzo', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'HumDayMin_Marzo', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'HumDayMin_Marzo', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'TEMPDayMax_Foliacion', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'TEMPDayMax_Foliacion', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'TEMPDayMax_Foliacion', type = "conditional")
plot(mp_xgboost)

#Se puede ver que los valores de conditional, accumulated y partial son muy similares para todas las variables a excepción
#de la ALTITUD, esto puede deberse a alguna interacción de ALTITUD con otra variable que analizaremos más adelante



#Ahora para estudiar si existe sesgo, estudiaremos las variables categóricas

#MODO:
set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'MODO', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'MODO', type = "accumulated")
plot(mp_xgboost)


#TIPO:
set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'TIPO', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'TIPO', type = "accumulated")
plot(mp_xgboost)



#COLOR:
set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'COLOR', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'COLOR', type = "accumulated")
plot(mp_xgboost)


#VARIEDAD:
set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'VARIEDAD', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'VARIEDAD', type = "accumulated")
plot(mp_xgboost)


#NO PARECE HABER SESGO EN EL MODELO





install.packages("SHAPforxgboost")
library(SHAPforxgboost)

y_var <-  "PRODUCCION"

# Obtener el índice de la columna del factor
col_index <- which(names(datos.train_16_19) == "VARIEDAD")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ VARIEDAD - 1, datos.train_16_19[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.train_16_19[, -col_index], one_hot_cols)                       

# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "ID_ZONA")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ ID_ZONA - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)                       



# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "MODO")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ MODO - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   



# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "PROD_MEAN_NA")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ PROD_MEAN_NA - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   





# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "COLOR")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ COLOR - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   





# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "PROD_1na")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ PROD_1na - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   






# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "PROD_2na")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ PROD_2na - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   



# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "TIPO")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ TIPO - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   


dataX<-data.matrix(datos.224[,xgbcv$finalModel$feature_names])

mod = xgbcv$finalModel

# To return the SHAP values and ranked features by mean|SHAP|
shap_values <- shap.values(xgb_model = mod, X_train = dataX)
# The ranked features by mean |SHAP|
shap_values$mean_shap_score


# To prepare the long-format data:
shap_long <- shap.prep(xgb_model = mod, X_train = dataX)
# is the same as: using given shap_contrib
shap_long <- shap.prep(shap_contrib = shap_values$shap_score, X_train = dataX)


g1 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_1', y = 'PROD_1', color_feature = 'ALTITUD')
g2 <- shap.plot.dependence(data_long = shap_long, x = 'ALTITUD', y = 'ALTITUD', color_feature = 'PROD_1') 

gridExtra::grid.arrange(g1, g2, ncol = 2)


g1 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_mean', y = 'PROD_mean', color_feature = 'ALTITUD') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'ALTITUD', y = 'ALTITUD', color_feature = 'PROD_mean')

gridExtra::grid.arrange(g1, g2, ncol = 2)


g1 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_1', y = 'PROD_1', color_feature = 'ALTITUD') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'ALTITUD', y = 'ALTITUD', color_feature = 'PROD_1')

gridExtra::grid.arrange(g1, g2, ncol = 2)

#Cuando la altitud es mayor, se corrigen las producciones anteriores altas hacia abajo, ya que la altitud 
#produce bajada de temperaturas, mayor probabilidad de heladas....

g1 <- shap.plot.dependence(data_long = shap_long, x = 'HumDayMin_Marzo', y = 'HumDayMin_Marzo', color_feature = 'ALTITUD')
g2 <- shap.plot.dependence(data_long = shap_long, x = 'ALTITUD', y = 'ALTITUD', color_feature = 'HumDayMin_Marzo')

gridExtra::grid.arrange(g1, g2, ncol = 2)


g1 <- shap.plot.dependence(data_long = shap_long, x = 'HumDayMin_Marzo', y = 'HumDayMin_Marzo', color_feature = 'PROD_1') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_1', y = 'PROD_1', color_feature = 'HumDayMin_Marzo') 

gridExtra::grid.arrange(g1, g2, ncol = 2)


#OJO, tarda un rato:
shap_int <- shap.prep.interaction(xgb_mod = mod, X_train = dataX)

# **SHAP interaction effect plot **
# if `data_int` is supplied, the same function will plot the interaction effect:


g3 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "PROD_1", y = "ALTITUD", 
                           color_feature = "ALTITUD")
g4 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "ALTITUD", y = "PROD_1", 
                           color_feature = "PROD_1")
gridExtra::grid.arrange(g3, g4, ncol=2)



g3 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "PROD_mean", y = "ALTITUD", 
                           color_feature = "ALTITUD")
g4 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "ALTITUD", y = "PROD_mean", 
                           color_feature = "PROD_mean")
gridExtra::grid.arrange(g3, g4, ncol=2)




g3 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "HumDayMin_Marzo", y = "ALTITUD", 
                           color_feature = "ALTITUD")
g4 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "ALTITUD", y = "HumDayMin_Marzo", 
                           color_feature = "HumDayMin_Marzo")
gridExtra::grid.arrange(g3, g4, ncol=2)




# choose to show top 4 features by setting `top_n = 4`, 
# set 6 clustering groups of observations.  
plot_data <- shap.prep.stack.data(shap_contrib = shap_values$shap_score, top_n = 4, n_groups = 6)
# you may choose to zoom in at a location, and set y-axis limit using `y_parent_limit`  
shap.plot.force_plot(plot_data, zoom_in_location = 500, y_parent_limit = c(-0.1,0.1))



shap.plot.force_plot_bygroup(plot_data)












##############################################################################################################
############ INTERPRETACION DEL SEGUNDO MODELO ################################################################
##############################################################################################################


exp <- explain(model = xgbFinal, data = datos.22, y = datos.22$PRODUCCION, type = "xgboost")
shap_values <- predict_parts(exp,datos.22,type = "shap")

# visualizar los valores SHAP en un gráfico de barras
plot(shap_values, type = "bar") #?


#Se puede ver la importancia en función de los valores de Shapley que de acuerdo con el
#Artículo citado en la memoria son unos de los mejores medidores de la importancia
#de las variables, pues son consistentes y precisos.


################################################################################
#Para evitar problemas con la computacion en paralelo
unregister_dopar()
################################################################################



#Para ver la distribución de los residuos separaremos en test y train:
set.seed(1234)
index = unlist(createDataPartition(datos.20_21$PRODUCCION, p = 0.8))
datos.20_21.train = datos.20_21[index,]
datos.20_21.test = datos.20_21[-index,]


set.seed(15032023)


xgbFinal.perf = caret::train(form = PRODUCCION ~ ., data = datos.20_21.train,
                             trControl = ctrl,
                             tuneGrid = tune_grid,
                             method = "xgbTree",
                             verbose = FALSE,
                             metric = 'RMSE',
                             tree_method="exact")

exp_xgboost = explain(xgbFinal.perf, data = datos.20_21.test, y = datos.20_21.test$PRODUCCION, label = "XGBoost", type = "regression")
perf_xgboost = model_performance(exp_xgboost)

plot(perf_xgboost)
shap_xgboost <- shap_aggregated(exp_xgboost, datos.20_21[1:10,])


#Sin más que poner el número de cada finca, esta función permite visualizar que es lo que más
#ha influido en la predicción.
#De esa forma, podríamos explicar a cada finca el motivo de su estimación



#No obstante, para explicar los resultados obtenidos creamos el explainer del modelo empleado:
ID = 48626
exp_xgboost = explain(xgbFinal, data = datos.22, y = datos.22$PRODUCCION, label = "XGBoost", type = "regression")
perf_xgboost = model_performance(exp_xgboost)

datos.shiny = datos_C_20_22 %>% filter(CAMPÑ %in% 22)
FINCA <- datos.22[which(datos.shiny$ID_FINCA == ID),]


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'ALTITUD', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'ALTITUD', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'ALTITUD', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'xgboost', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'xgboost', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'xgboost', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_1', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_1', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_1', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_2', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_2', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_2', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_mean', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_mean', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'PROD_mean', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'SUPERFICIE', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'SUPERFICIE', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'SUPERFICIE', type = "conditional")
plot(mp_xgboost)


set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'HumDayMin_Marzo', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'HumDayMin_Marzo', type = "accumulated")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'HumDayMin_Marzo', type = "conditional")
plot(mp_xgboost)

#Se puede ver que en algunas variables los valores de conditional plot difieren de accumulated y partial.
#Esto puede deberse a que hay importantes correlaciones entre ellas, pero tb a alguna interacción que mostraremos a continuacion



#Ahora para estudiar si existe sesgo, estudiaremos las variables categóricas

#MODO:
set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'MODO', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'MODO', type = "accumulated")
plot(mp_xgboost)


#TIPO:
set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'TIPO', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'TIPO', type = "accumulated")
plot(mp_xgboost)



#COLOR:
set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'COLOR', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'COLOR', type = "accumulated")
plot(mp_xgboost)


#VARIEDAD:
set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'VARIEDAD', type = "partial")
plot(mp_xgboost)

set.seed(1234)
mp_xgboost <- model_profile(exp_xgboost, variables = 'VARIEDAD', type = "accumulated")
plot(mp_xgboost)


#NO PARECE HABER SESGO EN EL MODELO







install.packages("SHAPforxgboost")
library(SHAPforxgboost)

y_var <-  "PRODUCCION"

# Obtener el índice de la columna del factor
col_index <- which(names(datos.22) == "VARIEDAD")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ VARIEDAD - 1, datos.22[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.22[, -col_index], one_hot_cols)                       

# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "ID_ZONA")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ ID_ZONA - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)                       



# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "MODO")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ MODO - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   



# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "PROD_MEAN_NA")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ PROD_MEAN_NA - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   





# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "COLOR")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ COLOR - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   





# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "PROD_1na")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ PROD_1na - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   






# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "PROD_2na")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ PROD_2na - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   



# Obtener el índice de la columna del factor
col_index <- which(names(datos.224) == "TIPO")

# Aplicar la codificación one-hot
one_hot_cols <- model.matrix(~ TIPO - 1, datos.224[, col_index, drop = FALSE])

# Eliminar la columna original y agregar las nuevas columnas one-hot
datos.224 <- cbind(datos.224[, -col_index], one_hot_cols)   


dataX<-data.matrix(datos.224[,xgbFinal$finalModel$feature_names])

mod = xgbFinal$finalModel




a<-data.matrix(subset(datos.224, select = -PRODUCCION))
b<-datos.224[,xgbFinal$finalModel$feature_names]
library(shapviz)
shp <- shapviz(xgbFinal$finalModel,X_pred=data.matrix(b),interactions = T)
sv_importance(shp, kind = "both",max_display = 40L)



# To return the SHAP values and ranked features by mean|SHAP|
shap_values <- shap.values(xgb_model = mod, X_train = dataX)
# The ranked features by mean |SHAP|
shap_values$mean_shap_score


# To prepare the long-format data:
shap_long <- shap.prep(xgb_model = mod, X_train = dataX)
# is the same as: using given shap_contrib
shap_long <- shap.prep(shap_contrib = shap_values$shap_score, X_train = dataX)

g1 <- shap.plot.dependence(data_long = shap_long, x = 'xgboost', y = 'xgboost', color_feature = 'PROD_1') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_1', y = 'PROD_1', color_feature = 'xgboost')

gridExtra::grid.arrange(g1, g2, ncol = 2)


g1 <- shap.plot.dependence(data_long = shap_long, x = 'xgboost', y = 'xgboost', color_feature = 'PROD_2') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_2', y = 'PROD_2', color_feature = 'xgboost') 

gridExtra::grid.arrange(g1, g2, ncol = 2)


g1 <- shap.plot.dependence(data_long = shap_long, x = 'SUPERFICIE', y = 'SUPERFICIE', color_feature = 'xgboost') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'xgboost', y = 'xgboost', color_feature = 'SUPERFICIE') 

gridExtra::grid.arrange(g1, g2, ncol = 2)


g1 <- shap.plot.dependence(data_long = shap_long, x = 'SUPERFICIE', y = 'SUPERFICIE', color_feature = 'PROD_1') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_1', y = 'PROD_1', color_feature = 'SUPERFICIE')

gridExtra::grid.arrange(g1, g2, ncol = 2)


g1 <- shap.plot.dependence(data_long = shap_long, x = 'SUPERFICIE', y = 'SUPERFICIE', color_feature = 'ALTITUD')
g2 <- shap.plot.dependence(data_long = shap_long, x = 'ALTITUD', y = 'ALTITUD', color_feature = 'SUPERFICIE')

gridExtra::grid.arrange(g1, g2, ncol = 2)


g1 <- shap.plot.dependence(data_long = shap_long, x = 'xgboost', y = 'xgboost', color_feature = 'ALTITUD') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'ALTITUD', y = 'ALTITUD', color_feature = 'xgboost')

gridExtra::grid.arrange(g1, g2, ncol = 2)



g1 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_1', y = 'PROD_1', color_feature = 'ALTITUD') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'ALTITUD', y = 'ALTITUD', color_feature = 'PROD_1') 

gridExtra::grid.arrange(g1, g2, ncol = 2)


g1 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_mean', y = 'PROD_mean', color_feature = 'ALTITUD') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'ALTITUD', y = 'ALTITUD', color_feature = 'PROD_mean')

gridExtra::grid.arrange(g1, g2, ncol = 2)
#Cuando la altitud es mayor, se corrigen las producciones anteriores altas hacia abajo, ya que la altitud 
#produce bajada de temperaturas, mayor probabilidad de heladas....

g1 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_1', y = 'PROD_1', color_feature = 'ALTITUD') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'ALTITUD', y = 'ALTITUD', color_feature = 'PROD_1')

gridExtra::grid.arrange(g1, g2, ncol = 2)



g1 <- shap.plot.dependence(data_long = shap_long, x = 'HumDayMin_Marzo', y = 'HumDayMin_Marzo', color_feature = 'ALTITUD') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'ALTITUD', y = 'ALTITUD', color_feature = 'HumDayMin_Marzo')

gridExtra::grid.arrange(g1, g2, ncol = 2)



g1 <- shap.plot.dependence(data_long = shap_long, x = 'HumDayMin_Marzo', y = 'HumDayMin_Marzo', color_feature = 'xgboost') 
g2 <- shap.plot.dependence(data_long = shap_long, x = 'xgboost', y = 'xgboost', color_feature = 'HumDayMin_Marzo')

gridExtra::grid.arrange(g1, g2, ncol = 2)



g1 <- shap.plot.dependence(data_long = shap_long, x = 'HumDayMin_Marzo', y = 'HumDayMin_Marzo', color_feature = 'PROD_1')
g2 <- shap.plot.dependence(data_long = shap_long, x = 'PROD_1', y = 'PROD_1', color_feature = 'HumDayMin_Marzo') 

gridExtra::grid.arrange(g1, g2, ncol = 2)


shap_int <- shap.prep.interaction(xgb_mod = mod, X_train = dataX)

# **SHAP interaction effect plot **
# if `data_int` is supplied, the same function will plot the interaction effect:
g3 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "xgboost", y = "ALTITUD", 
                           color_feature = "ALTITUD")
g4 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "ALTITUD", y = "xgboost", 
                           color_feature = "xgboost")
gridExtra::grid.arrange(g3, g4, ncol=2)




g3 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "PROD_1", y = "ALTITUD", 
                           color_feature = "ALTITUD")
g4 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "ALTITUD", y = "PROD_1", 
                           color_feature = "PROD_1")
gridExtra::grid.arrange(g3, g4, ncol=2)



g3 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "PROD_mean", y = "ALTITUD", 
                           color_feature = "ALTITUD")
g4 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "ALTITUD", y = "PROD_mean", 
                           color_feature = "PROD_mean")
gridExtra::grid.arrange(g3, g4, ncol=2)




g3 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "HumDayMin_Marzo", y = "ALTITUD", 
                           color_feature = "ALTITUD")
g4 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "ALTITUD", y = "HumDayMin_Marzo", 
                           color_feature = "HumDayMin_Marzo")
gridExtra::grid.arrange(g3, g4, ncol=2)




g3 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "HumDayMin_Marzo", y = "xgboost", 
                           color_feature = "xgboost")
g4 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "xgboost", y = "HumDayMin_Marzo", 
                           color_feature = "HumDayMin_Marzo")
gridExtra::grid.arrange(g3, g4, ncol=2)
#Parece haber algún efecto de interacción, pero la escala es muy pequeña
# 50 uds de media, por lo que no es relevante

#De todas formas, parece que en xgboosts bajos una humedad en marzo
#baja favorece a la predicción, mientras que para xgboosts grandes
#sucede lo contrario. #El segundo modelo lleva a cabo una corrección del primero
#Tal y como esperábamos



g3 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "HumDayMin_Marzo", y = "PROD_1", 
                           color_feature = "PROD_1")
g4 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "PROD_1", y = "HumDayMin_Marzo", 
                           color_feature = "HumDayMin_Marzo")
gridExtra::grid.arrange(g3, g4, ncol=2)






g3 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "SUPERFICIE", y = "xgboost", 
                           color_feature = "xgboost")
g4 <- shap.plot.dependence(data_long = shap_long,
                           data_int = shap_int,
                           x= "xgboost", y = "SUPERFICIE", 
                           color_feature = "SUPERFICIE")
gridExtra::grid.arrange(g3, g4, ncol=2)




# choose to show top 4 features by setting `top_n = 4`, 
# set 6 clustering groups of observations.  
plot_data <- shap.prep.stack.data(shap_contrib = shap_values$shap_score, top_n = 4, n_groups = 6)
# you may choose to zoom in at a location, and set y-axis limit using `y_parent_limit`  
shap.plot.force_plot(plot_data, zoom_in_location = 500, y_parent_limit = c(-0.1,0.1))



shap.plot.force_plot_bygroup(plot_data)











#Creamos una shiny app para ver la importancia de las variables en cada finca y cuanto aporta cada una
#a la prediccion final del modelo

#OJO HAY QUE INTRODUCIR EL ID_UNICO NO EL ID_FINCA

library(shiny)
library(DALEX)
library(ggplot2)

ui <- fluidPage(
  titlePanel("Explicación de las predicciones para cada ID de finca."),
  textInput("ID_UNICO", "Introduzca el ID único:"),
  actionButton("submit", "Submit"),
  plotOutput("shapley_plot"),
  plotOutput("breakdown_plot")
)

server <- function(input, output) {
  FINCA_data <- eventReactive(input$submit, {
    ID <- input$ID_UNICO
    datos.shiny = datos_C_20_22 %>% filter(CAMPÑ %in% 22)
    FINCA <- datos.22[which(datos.shiny$ID_UNICO == ID),]
    exp_xgboost = explain(xgbFinal, data = datos.20_21, y = datos.20_21$PRODUCCION, label = "XGBoost", type = "regression")
    return(list(FINCA = FINCA, exp_xgboost = exp_xgboost))
  })
  
  observeEvent(input$submit, {
    showModal(modalDialog(
      title = "Calculando...",
      "Se está calculando la importancia de las distintas variables en la predicción de la finca y la aportación de cada una de ellas a la misma. \n 
      Esto llevará unos 60s. \n No olvide clickar en 'Dismiss' una vez los gráficos hayan aparecido."
    ))
  })
  
  output$shapley_plot <- renderPlot({
    FINCA_data <- FINCA_data()
    if (!is.null(FINCA_data$FINCA)) {
      sh_xgboost <- predict_parts(FINCA_data$exp_xgboost, FINCA_data$FINCA, type = "shap", B = 1)
      plot(sh_xgboost, show_boxplots = FALSE) + ggtitle(paste("Valores de importancia (Shapley) de cada variable en la predicción de la finca", input$ID_UNICO),"")
    }
  })
  
  
  output$breakdown_plot <- renderPlot({
    FINCA_data <- FINCA_data()
    if (!is.null(FINCA_data$FINCA)) {
      bd_xgboost <- predict_parts(FINCA_data$exp_xgboost, FINCA_data$FINCA, type = "break_down_interactions")
      plot(bd_xgboost, show_boxplots = FALSE) + ggtitle(paste("Aportación de cada variable a la predicción de la finca",input$ID_UNICO) ,"")
    }
  })
}


shinyApp(ui = ui, server = server)
